#include "StdAfx.h"
#include "DriverBase.h"

CDriverBase::CDriverBase(bool bLog) : m_bLog(bLog)
{
	m_pLog = NULL;
}

CDriverBase::~CDriverBase(void)
{
	if (m_bLog && m_pLog != NULL)
	{
		delete m_pLog;
	}
}

string CDriverBase::ReadLine(char cEOL, size_t nSize)
{
	string strLine;

	bool bLog = m_bLog;

	if (m_bLog)
		m_bLog = false;

	while (true)
	{
		string c = Read(1);

		if (!c.empty())
		{
			strLine += c;

            if (!strncmp(c.c_str(), &cEOL, 1))
                break;
			if (nSize != 0 && strLine.size() >= nSize)
                break;
		}
		else
			break;
	}

	if (bLog)
	{
		m_bLog = true;
		WriteLog(strLine, "Read ");
	}

	return strLine;
}

list<string> CDriverBase::ReadLines(char cEOL, int nSizeHint)
{
	list<string> lstrLines;

	if (m_dTimeout < 0)
	{
		CIronLib::WriteDebugLog("Port must have enabled timeout for this function!(%s:%d)", __FILE__, __LINE__);
		return lstrLines;
	}

	while (true)
	{
		string strLine = ReadLine(cEOL);

		if (!strLine.empty())
		{
			lstrLines.push_back(strLine);

			char cEnd = strLine.rbegin()[0];

			if (strncmp(&cEnd, &cEOL, 1))
				break;
		}
		else
			break;
	}

	return lstrLines;
}

list<string> CDriverBase::xreadlines(int nSizeHint)
{
	return ReadLines();
}

bool CDriverBase::writelines(list<string> lstrSequence)
{
	for (list<string>::iterator iter = lstrSequence.begin() ; iter != lstrSequence.end() ; iter++)
	{
		if (!Write(*iter))
		{
			return false;
		}
	}

	return true;
}

void CDriverBase::SetWriteLog(bool bLog)
{
	m_bLog = bLog;
}

bool CDriverBase::GetWriteLog()
{
	return m_bLog;
}

void CDriverBase::WriteLog(string &sLog, string sTag)
{
	if (m_bLog && m_pLog != NULL && sLog.length() > 0)
	{
		if (sLog.size() > 1024)
		{
			unsigned int i = 0;

			for (; i < (sLog.size() / 1024); i++)
			{
				string temp = sLog.substr(i * 1024, 1024);

				m_pLog->WriteLog("%s(%d) >> %s", sTag.c_str(), i + 1, temp.c_str());
			}

			if (sLog.substr(i * 1024).size() > 0)
			{
				m_pLog->WriteLog("%s(%d) >> %s", sTag.c_str(), i + 1, sLog.substr(i * 1024).c_str());
			}
		}
		else
		{
			m_pLog->WriteLog("%s >> %s", sTag.c_str(), sLog.c_str());
		}
	}
}

void CDriverBase::WriteLog(BYTE* byLog, UINT nSize, string sTag)
{
	if (m_bLog && m_pLog != NULL && nSize > 0)
	{
		if (nSize > 1024)
		{
			unsigned int i = 0;

			for (; i < (nSize / 1024); i++)
			{
				string strLog;

				for (UINT nPos = 0 ; nPos < 1024 ; ++nPos)
				{
					if (byLog[i * 1024 + nPos] == 0x00)
					{
						strLog += "0x00";
					}
					else
					{
						strLog.push_back(byLog[i * 1024 + nPos]);
					}
				}

				m_pLog->WriteLog("%s(%d) >> %s", sTag.c_str(), i + 1, strLog.c_str());
			}

			string strLog;

			for (UINT nPos = 0 ; nPos < (nSize % 1024) ; ++nPos)
			{
				if (byLog[i * 1024 + nPos] == 0x00)
				{
					strLog += "0x00";
				}
				else
				{
					strLog.push_back(byLog[i * 1024 + nPos]);
				}
			}

			if (strLog.size() > 0)
			{
				m_pLog->WriteLog("%s(%d) >> %s", sTag.c_str(), i + 1, strLog.c_str());
			}
		}
		else
		{
			string strLog;

			for (UINT nPos = 0 ; nPos < nSize ; ++nPos)
			{
				if (byLog[nPos] == 0x00)
				{
					strLog += "0x00";
				}
				else
				{
					strLog.push_back(byLog[nPos]);
				}
			}

			m_pLog->WriteLog("%s >> %s", sTag.c_str(), strLog.c_str());
		}
	}
}