#include "StdAfx.h"
#include ".\serial.h"

CSerial::CSerial(bool bLog,
				 unsigned nPort,
				 UINT nBaudrate,
				 int nBytesize,
				 WORD wParity,
				 int nStopbits,
				 int nTimeout,
				 int nXOnXOff,
				 int nRtsCts,
				 int nWriteTimeout,
				 int nDsrDtr)
:m_hComPort(NULL), CSerialBase(bLog, nPort, nBaudrate, nBytesize, wParity, nStopbits, nTimeout, nXOnXOff, nRtsCts, nWriteTimeout, nDsrDtr)
{
	ZERO_MEMORY(m_overlappedRead);
	ZERO_MEMORY(m_overlappedWrite);

	Open();
}

CSerial::~CSerial(void)
{
	Close();
}

bool CSerial::Open(void)
{
	if (m_sPort.size() == 0)
	{
		CIronLib::WriteDebugLog("Port must be configured before it can be used.(%s:%d)", __FILE__, __LINE__);
		return false;
	}

	m_hComPort = CreateFile(m_sPort.c_str(),
		GENERIC_READ|GENERIC_WRITE,
		0, 
		0, 
		OPEN_EXISTING, 
		FILE_FLAG_OVERLAPPED, 
		0);

	if (m_hComPort == INVALID_HANDLE_VALUE)
	{
		CIronLib::WriteDebugLog("Could not open port: %s.(%s:%d)", m_sPort.c_str(), __FILE__, __LINE__);
		assert(false);
		return false;
	}

	if (!SetupComm(m_hComPort, 4096, 4096))
	{
		CIronLib::WriteDebugLog("SetupComm() return is false.(%s:%d)", __FILE__, __LINE__);
		return false;
	}

	if (!GetCommTimeouts(m_hComPort, &m_orgTimeout))
	{
		CIronLib::WriteDebugLog("GetCommTimeouts() return is false.(%s:%d)", __FILE__, __LINE__);
		return false;
	}

	m_dwRtsState = RTS_CONTROL_ENABLE;
	m_dwDtrState = RTS_CONTROL_ENABLE;

	ReconfigurePort();

	if (!PurgeComm(m_hComPort, PURGE_TXCLEAR | PURGE_TXABORT | PURGE_RXCLEAR | PURGE_RXABORT))
	{
		CIronLib::WriteDebugLog("PurgeComm() return is false.(%s:%d)", __FILE__, __LINE__);
		return false;
	}

	m_overlappedRead.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	m_overlappedWrite.hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);

	m_bOpen = true;

	return true;
}

void CSerial::Close(void)
{
	if (m_bOpen && m_hComPort)
	{
		if (!SetCommTimeouts(m_hComPort, &m_orgTimeout))
		{
			CIronLib::WriteDebugLog("SetCommTimeouts() return is false.(%s:%d)", __FILE__, __LINE__);
			return;
		}

		CloseHandle(m_hComPort);
		m_hComPort = NULL;
		m_bOpen = false;

	}

	if (m_overlappedRead.hEvent != NULL)
	{
		CloseHandle(m_overlappedRead.hEvent);
	}

	if (m_overlappedWrite.hEvent != NULL)
	{
		CloseHandle(m_overlappedWrite.hEvent);
	}
}

void CSerial::ReconfigurePort(void)
{
	if (m_hComPort == INVALID_HANDLE_VALUE)
	{
		assert(m_hComPort);
		CIronLib::WriteDebugLog("COMPort is not opend.(%s:%d)", __FILE__, __LINE__);
		return;
	}

	COMMTIMEOUTS timeouts;

	if (m_nTimeout < 0)
	{
		timeouts.ReadIntervalTimeout = 0;
		timeouts.ReadTotalTimeoutMultiplier = 0;
		timeouts.ReadTotalTimeoutConstant = 0;
		timeouts.WriteTotalTimeoutMultiplier = 0;
		timeouts.WriteTotalTimeoutConstant = 0;
	}
	else if (m_nTimeout == 0)
	{
		timeouts.ReadIntervalTimeout = MAXDWORD;
		timeouts.ReadTotalTimeoutMultiplier = 0;
		timeouts.ReadTotalTimeoutConstant = 0;
		timeouts.WriteTotalTimeoutMultiplier = 0;
		timeouts.WriteTotalTimeoutConstant = 0;
	}
	else
	{
		timeouts.ReadIntervalTimeout = 0;
		timeouts.ReadTotalTimeoutMultiplier = 0;
		timeouts.ReadTotalTimeoutConstant = m_nTimeout*1000;
		timeouts.WriteTotalTimeoutMultiplier = 0;
		timeouts.WriteTotalTimeoutConstant = 0;
	}

	if (m_nWriteTimeout == 0)
	{
		timeouts.WriteTotalTimeoutMultiplier = 0;
		timeouts.WriteTotalTimeoutConstant = MAXWORD;
	}
	else
	{
		timeouts.WriteTotalTimeoutMultiplier = 0;
		timeouts.WriteTotalTimeoutConstant = m_nWriteTimeout*1000;
	}

	if (!SetCommTimeouts(m_hComPort, &timeouts))
	{
		CIronLib::WriteDebugLog("SetCommTimeouts() return is false.(%s:%d)", __FILE__, __LINE__);
		return;
	}

	if (!SetCommMask(m_hComPort, EV_ERR))
	{
		CIronLib::WriteDebugLog("SetCommMask() return is false.(%s:%d)", __FILE__, __LINE__);
		return;
	}

	DCB comDCB;
	
	if (!GetCommState(m_hComPort, &comDCB))
	{
		CIronLib::WriteDebugLog("GetCommState() return is false.(%s:%d)", __FILE__, __LINE__);
		return;
	}

	comDCB.BaudRate = m_dwBaudrate;

	if (m_nBytesize == FIVEBITS)
		comDCB.ByteSize = 5;
	else if (m_nBytesize == SIXBITS)
		comDCB.ByteSize = 6;
	else if (m_nBytesize == SEVENBITS)
		comDCB.ByteSize = 7;
	else if (m_nBytesize == EIGHTBITS)
		comDCB.ByteSize = 8;
	else
	{
		CIronLib::WriteDebugLog("Unsupported number of data bits: %d,(%s:%d)", m_nBytesize, __FILE__, __LINE__);
        assert(false);
		return;
	}

	if (m_wParity == PARITY_NONE)
	{
		comDCB.Parity = NOPARITY;
		comDCB.fParity = 0;
	}
	else if (m_wParity == PARITY_EVEN)
	{
		comDCB.Parity = EVENPARITY;
		comDCB.fParity = 1;
	}
	else if (m_wParity == PARITY_ODD)
	{
		comDCB.Parity = ODDPARITY;
		comDCB.fParity = 1;
	}
	else
	{
		CIronLib::WriteDebugLog("Unsupported parity mode: %d,(%s:%d)", m_wParity, __FILE__, __LINE__);
		assert(false);
		return;
	}

	if (m_nStopbits = STOPBITS_ONE)
		comDCB.StopBits = ONESTOPBIT;
	else if (m_nStopbits = STOPBITS_TWO)
		comDCB.StopBits = TWOSTOPBITS;
	else
	{
		CIronLib::WriteDebugLog("Unsupported number of stop bits: %d,(%s:%d)", m_nStopbits, __FILE__, __LINE__);
		assert(false);
		return;
	}

	comDCB.fBinary = 1;

	if (m_nRtsCts != 0)
		comDCB.fRtsControl = RTS_CONTROL_HANDSHAKE;
	else
		comDCB.fRtsControl = m_dwRtsState;

	if (m_nDsrDtr != 0)
		comDCB.fDtrControl = DTR_CONTROL_HANDSHAKE;
	else
		comDCB.fDtrControl = m_dwDtrState;

	comDCB.fOutxCtsFlow = m_nRtsCts;
	comDCB.fOutxDsrFlow = m_nDsrDtr;
	comDCB.fOutX = m_nXonXoff;
	comDCB.fInX = m_nXonXoff;
	comDCB.fNull= 0;
	comDCB.fErrorChar = 0;
	comDCB.fAbortOnError = 0;
	comDCB.XonChar = XON;
	comDCB.XoffChar = XOFF;


	if (!SetCommState(m_hComPort, &comDCB))
	{
		CIronLib::WriteDebugLog("Cannot configure port, some setting was wrong.(%s:%d)", __FILE__, __LINE__);
		assert(false);
		return;
	}
}

unsigned int CSerial::InWaiting(void)
{
	DWORD flags = 0;
	COMSTAT comstat;

	if (!ClearCommError(m_hComPort, &flags, &comstat))
	{
		CIronLib::WriteDebugLog("ClearCommError() return is false.(%s:%d)", __FILE__, __LINE__);
		assert(false);
		return -1;
	}

	return comstat.cbInQue;
}

string CSerial::Read(UINT nSize)
{
	if (m_hComPort == INVALID_HANDLE_VALUE)
	{
		assert(m_hComPort);
		CIronLib::WriteDebugLog("COMPort is not opend.(%s:%d)", __FILE__, __LINE__);
		return "";
	}

	if (nSize > MAX_READBUFF_SIZE)
	{
		CIronLib::WriteDebugLog("ReadSize is bigger than buffer size.(%s:%d)", __FILE__, __LINE__);
		
		nSize = MAX_READBUFF_SIZE;
	}
	
	DWORD flags = 0;
	COMSTAT comstat;
	char readBuff[MAX_READBUFF_SIZE + 1];
	DWORD dwReadSize = 0;

	ZERO_MEMORY(readBuff);

	if (nSize > 0)
	{
		ResetEvent(m_overlappedRead.hEvent);

		if (!ClearCommError(m_hComPort, &flags, &comstat))
		{
			CIronLib::WriteDebugLog("ClearCommError() return is false.(%s:%d)", __FILE__, __LINE__);
			assert(false);
			return string("");
		}

		if (m_nTimeout == 0)
		{
			int n = min(static_cast<int>(comstat.cbInQue), nSize);

			if (n > 0)
			{
				ReadFile(m_hComPort, readBuff, n, &dwReadSize, &m_overlappedRead);
				WaitForSingleObject(m_overlappedRead.hEvent, INFINITE);
			}
		}
		else
		{
			ReadFile(m_hComPort, readBuff, nSize, &dwReadSize, &m_overlappedRead);

			GetOverlappedResult(m_hComPort, &m_overlappedRead, &dwReadSize, TRUE);
		}
	}

	string sLog = readBuff;
	WriteLog(sLog, "Read ");

	return string(readBuff);
}

bool CSerial::Read(BYTE* byRead, UINT nSize)
{
	if (m_hComPort == INVALID_HANDLE_VALUE)
	{
		assert(m_hComPort);
		CIronLib::WriteDebugLog("COMPort is not opend.(%s:%d)", __FILE__, __LINE__);
		return false;
	}
	
	DWORD flags = 0;
	COMSTAT comstat;
	DWORD dwReadSize = 0;

	memset(byRead, 0x00, nSize);

	if (nSize > 0)
	{
		ResetEvent(m_overlappedRead.hEvent);

		if (!ClearCommError(m_hComPort, &flags, &comstat))
		{
			CIronLib::WriteDebugLog("ClearCommError() return is false.(%s:%d)", __FILE__, __LINE__);
			assert(false);
			return false;
		}

		if (m_nTimeout == 0)
		{
			int n = min(static_cast<int>(comstat.cbInQue), nSize);

			if (n > 0)
			{
				ReadFile(m_hComPort, byRead, n, &dwReadSize, &m_overlappedRead);
				WaitForSingleObject(m_overlappedRead.hEvent, INFINITE);
			}
		}
		else
		{
			ReadFile(m_hComPort, byRead, nSize, &dwReadSize, &m_overlappedRead);

			GetOverlappedResult(m_hComPort, &m_overlappedRead, &dwReadSize, TRUE);
		}
	}

	WriteLog(byRead, dwReadSize, "Read ");

	if (dwReadSize != nSize)
		return false;

	return true;
}

bool CSerial::Write(string strWrite)
{
	if (m_hComPort == INVALID_HANDLE_VALUE)
	{
		assert(m_hComPort);
		CIronLib::WriteDebugLog("COMPort is not opend.(%s:%d)", __FILE__, __LINE__);
		return false;
	}

	if (!strWrite.empty())
	{
		DWORD dwWriteSize;

		if (!WriteFile(m_hComPort, strWrite.c_str(), static_cast<UINT>(strWrite.size()), &dwWriteSize, &m_overlappedWrite))
		{
			if (GetLastError() != ERROR_IO_PENDING)
			{
				GetOverlappedResult(m_hComPort, &m_overlappedWrite, &dwWriteSize, TRUE);

				if (dwWriteSize != static_cast<int>(strWrite.size()))
				{
					CIronLib::WriteDebugLog("Write Timeout Error.(%s:%d)", __FILE__, __LINE__);
					return false;
				}
			}
		}
	}

	WriteLog(strWrite, "Write");

	return true;
}

bool CSerial::Write(BYTE* byWrite, UINT nSize)
{
	if (m_hComPort == INVALID_HANDLE_VALUE)
	{
		assert(m_hComPort);
		CIronLib::WriteDebugLog("COMPort is not opend.(%s:%d)", __FILE__, __LINE__);
		return false;
	}


	if (nSize > 0)
	{
		DWORD dwWriteSize;

		if (!WriteFile(m_hComPort, byWrite, nSize, &dwWriteSize, &m_overlappedWrite))
		{
			if (GetLastError() == ERROR_IO_PENDING)
			{
				GetOverlappedResult(m_hComPort, &m_overlappedWrite, &dwWriteSize, TRUE);

				if (dwWriteSize != nSize)
				{
					CIronLib::WriteDebugLog("Write Timeout Error.(%s:%d)", __FILE__, __LINE__);
					return false;
				}
			}
		}
	}

	WriteLog(byWrite, nSize, "Write");

	return true;
}

void CSerial::FlushInput(void)
{
	if (m_hComPort == INVALID_HANDLE_VALUE)
	{
		assert(m_hComPort);
		CIronLib::WriteDebugLog("COMPort is not opend.(%s:%d)", __FILE__, __LINE__);
		return;
	}

	if (!PurgeComm(m_hComPort, PURGE_RXCLEAR | PURGE_RXABORT))
	{
		CIronLib::WriteDebugLog("PurgeComm() return is false.(%s:%d)", __FILE__, __LINE__);
		assert(false);
		return;
	}
}

void CSerial::FlushOutput(void)
{
	if (m_hComPort == INVALID_HANDLE_VALUE)
	{
		assert(m_hComPort);
		CIronLib::WriteDebugLog("COMPort is not opend.(%s:%d)", __FILE__, __LINE__);
		return;
	}

	if (!PurgeComm(m_hComPort, PURGE_TXCLEAR | PURGE_TXABORT))
	{
		CIronLib::WriteDebugLog("PurgeComm() return is false.(%s:%d)", __FILE__, __LINE__);
		assert(false);
		return;
	}
}

void CSerial::SendBreak(void)
{
	if (m_hComPort == INVALID_HANDLE_VALUE)
	{
		assert(m_hComPort);
		CIronLib::WriteDebugLog("COMPort is not opend.(%s:%d)", __FILE__, __LINE__);
		return;
	}

	if (!SetCommBreak(m_hComPort))
	{
		CIronLib::WriteDebugLog("SetCommBreak() return is false.(%s:%d)", __FILE__, __LINE__);
		assert(false);
		return;
	}

	Sleep(20);

	if (!ClearCommBreak(m_hComPort))
	{
		CIronLib::WriteDebugLog("ClearCommBreak() return is false.(%s:%d)", __FILE__, __LINE__);
		assert(false);
		return;
	}
}

void CSerial::SetRTS(bool bLevel)
{
	if (m_hComPort == INVALID_HANDLE_VALUE)
	{
		assert(m_hComPort);
		CIronLib::WriteDebugLog("COMPort is not opend.(%s:%d)", __FILE__, __LINE__);
		return;
	}

	if (bLevel)
	{
		m_dwRtsState = RTS_CONTROL_ENABLE;

		if (!EscapeCommFunction(m_hComPort, SETRTS))
		{
			CIronLib::WriteDebugLog("EscapeCommFunction() return is false.(%s:%d)", __FILE__, __LINE__);
			assert(false);
			return;
		}
	}
	else
	{
		m_dwRtsState = RTS_CONTROL_DISABLE;

		if (!EscapeCommFunction(m_hComPort, CLRRTS))
			{
			CIronLib::WriteDebugLog("EscapeCommFunction() return is false.(%s:%d)", __FILE__, __LINE__);
			assert(false);
			return;
		}
	}
}

void CSerial::SetDTR(bool bLevel)
{
	if (m_hComPort == INVALID_HANDLE_VALUE)
	{
		assert(m_hComPort);
		CIronLib::WriteDebugLog("COMPort is not opend.(%s:%d)", __FILE__, __LINE__);
		return;
	}

	if (bLevel)
	{
		m_dwDtrState = DTR_CONTROL_ENABLE;

		if (!EscapeCommFunction(m_hComPort, SETDTR))
		{
			CIronLib::WriteDebugLog("EscapeCommFunction() return is false.(%s:%d)", __FILE__, __LINE__);
			assert(false);
			return;
		}
	}
	else
	{
		m_dwDtrState = DTR_CONTROL_DISABLE;

		if (!EscapeCommFunction(m_hComPort, CLRDTR))
		{
			CIronLib::WriteDebugLog("EscapeCommFunction() return is false.(%s:%d)", __FILE__, __LINE__);
			assert(false);
			return;
		}
	}
}

bool CSerial::GetCTS(void)
{
	if (m_hComPort == INVALID_HANDLE_VALUE)
	{
		assert(m_hComPort);
		CIronLib::WriteDebugLog("COMPort is not opend.(%s:%d)", __FILE__, __LINE__);
		return false;
	}

	DWORD dwFlag;

	if (!GetCommModemStatus(m_hComPort, &dwFlag))
	{
		CIronLib::WriteDebugLog("GetCommModemStatus() return is false.(%s:%d)", __FILE__, __LINE__);
		assert(false);
		return false;
	}

	return (MS_CTS_ON & dwFlag) != 0;
}

bool CSerial::GetDSR(void)
{
	if (m_hComPort == INVALID_HANDLE_VALUE)
	{
		assert(m_hComPort);
		CIronLib::WriteDebugLog("COMPort is not opend.(%s:%d)", __FILE__, __LINE__);
		return false;
	}

	DWORD dwFlag;

	if (!GetCommModemStatus(m_hComPort, &dwFlag))
	{
		CIronLib::WriteDebugLog("GetCommModemStatus() return is false.(%s:%d)", __FILE__, __LINE__);
		assert(false);
		return false;
	}

	return (MS_DSR_ON & dwFlag) != 0;
}

bool CSerial::GetRI(void)
{
	if (m_hComPort == INVALID_HANDLE_VALUE)
	{
		assert(m_hComPort);
		CIronLib::WriteDebugLog("COMPort is not opend.(%s:%d)", __FILE__, __LINE__);
		return false;
	}

	DWORD dwFlag;

	if (!GetCommModemStatus(m_hComPort, &dwFlag))
	{
		CIronLib::WriteDebugLog("GetCommModemStatus() return is false.(%s:%d)", __FILE__, __LINE__);
		assert(false);
		return false;
	}

	return (MS_RING_ON & dwFlag) != 0;
}

bool CSerial::GetCD(void)
{
	if (m_hComPort == INVALID_HANDLE_VALUE)
	{
		assert(m_hComPort);
		CIronLib::WriteDebugLog("COMPort is not opend.(%s:%d)", __FILE__, __LINE__);
		return false;
	}

	DWORD dwFlag;

	if (!GetCommModemStatus(m_hComPort, &dwFlag))
	{
		CIronLib::WriteDebugLog("GetCommModemStatus() return is false.(%s:%d)", __FILE__, __LINE__);
		assert(false);
		return false;
	}

	return (MS_RLSD_ON & dwFlag) != 0;
}

void CSerial::SetXON(bool bLevel)
{
	if (m_hComPort == INVALID_HANDLE_VALUE)
	{
		assert(m_hComPort);
		CIronLib::WriteDebugLog("COMPort is not opend.(%s:%d)", __FILE__, __LINE__);
		return;
	}

	if (bLevel)
	{
		if (!EscapeCommFunction(m_hComPort, SETXON))
		{
			CIronLib::WriteDebugLog("EscapeCommFunction() return is false.(%s:%d)", __FILE__, __LINE__);
			assert(false);
			return;
		}
	}
	else
	{

		if (!EscapeCommFunction(m_hComPort, SETXOFF))
		{
			CIronLib::WriteDebugLog("EscapeCommFunction() return is false.(%s:%d)", __FILE__, __LINE__);
			assert(false);
			return;
		}
	}
}
