#include "StdAfx.h"
#include ".\serialbase.h"

CSerialBase::CSerialBase(bool bLog,
						 unsigned nPort, 
                         UINT nBaudrate,
						 int nBytesize,
						 WORD wParity,
						 int nStopbits,
						 int nTimeout,
						 int nXOnXOff,
						 int nRtsCts,
						 int nWriteTimeout,
						 int nDsrDtr
						)
: CDriverBase(bLog), m_nPort(nPort), m_dwBaudrate(nBaudrate), m_nBytesize(nBytesize), m_wParity(wParity), 
m_nStopbits(nStopbits), m_nTimeout(nTimeout), m_nWriteTimeout(nWriteTimeout), 
m_nXonXoff(nXOnXOff), m_nRtsCts(nRtsCts), m_nDsrDtr(nDsrDtr)
{
	m_bOpen = false;
	m_sPort = MakeDeviceName(m_nPort);

	if (bLog)
	{
		char strFilename[FILENAME_MAX];

		if( !CIronLib::ReadRegistryString(HKEY_LOCAL_MACHINE, "SOFTWARE\\IronTools\\Application\\", NULL, ".\\", strFilename, FILENAME_MAX) )
		{
			CIronLib::WriteDebugLog("ReadRegistryString() return is false.(%s:%d)", __FILE__, __LINE__);
		}

		string strInstallFolder = strFilename;
		
		if( strInstallFolder.find_last_of('\\') != strInstallFolder.size() - 1 )
		{
			strInstallFolder += "\\";
		}

		m_pLog = new CIronLog(m_sPort, strInstallFolder + "CommLog\\");
	}
}

CSerialBase::~CSerialBase(void)
{
}

string CSerialBase::MakeDeviceName(int nPort)
{
	if (nPort <= 0)
		return "";

    string sPort;
	
	if (nPort > 8)
		sPort = "\\\\.\\\\COM";
	else
		sPort = "COM";

	char temp[256];

	_itoa_s(nPort, temp, 256, 10);

	return sPort + temp;
}


const int* CSerialBase::GetSupportedBaudrates()
{
    return BAUDRATES;
}

const int* CSerialBase::GetSupportedByteSizes()
{
    return BYTESIZES;
}

const int* CSerialBase::GetSupportedStopbits()
{
    return STOPBITS;
}

const int* CSerialBase::GetSupportedParities()
{
    return PARITIES;
}

void CSerialBase::SetPort(int nPort)
{
	bool bWasOpen = m_bOpen;

    if (bWasOpen)
		Close();

	m_sPort = MakeDeviceName(nPort);
	m_nPort = nPort;

    if (bWasOpen)
		Open();
}

int CSerialBase::GetPort()
{
	return m_nPort;
}

void CSerialBase::SetBaudrate(UINT nBaudrate)
{
	if (!CIronLib::IsArrayInt(BAUDRATES, sizeof(BAUDRATES)/4, static_cast<int>(nBaudrate)))
	{
		CIronLib::WriteDebugLog("IsArrayInt() return is false.(%s:%d)", __FILE__, __LINE__);
		return;
	}

    m_dwBaudrate = nBaudrate;
	
	if (m_bOpen)
		ReconfigurePort();
}
    
UINT CSerialBase::GetBaudrate()
{
	return m_dwBaudrate;
}

void CSerialBase::SetByteSize(int nBytesize)
{
	if (!CIronLib::IsArrayInt(BYTESIZES, sizeof(BYTESIZES)/4, nBytesize))
	{
		CIronLib::WriteDebugLog("IsArrayInt() return is false.(%s:%d)", __FILE__, __LINE__);
		return;
	}

	m_nBytesize = nBytesize;

	if (m_bOpen)
		ReconfigurePort();
}

int CSerialBase::GetByteSize()
{
	return m_nBytesize;
}

void CSerialBase::SetParity(WORD wParity)
{
	if (!CIronLib::IsArrayInt(PARITIES, sizeof(PARITIES)/4, wParity))
	{
		CIronLib::WriteDebugLog("IsArrayInt() return is false.(%s:%d)", __FILE__, __LINE__);
		return;
	}

	m_wParity = wParity;

	if (m_bOpen)
		ReconfigurePort();
}

WORD CSerialBase::GetParity()
{
	return m_wParity;
}

void CSerialBase::SetStopbits(int nStopbits)
{
	if (!CIronLib::IsArrayInt(STOPBITS, sizeof(STOPBITS)/4, nStopbits))
	{
		CIronLib::WriteDebugLog("IsArrayInt() return is false.(%s:%d)", __FILE__, __LINE__);
		return;
	}

	m_nStopbits = nStopbits;

	if (m_bOpen)
		ReconfigurePort();
}

int CSerialBase::GetStopbits()
{
	return m_nStopbits;
}

void CSerialBase::SetTimeout(double dTimeout)
{
	m_nTimeout = static_cast<int>(dTimeout);

	if (m_bOpen)
		ReconfigurePort();
}

double CSerialBase::GetTimeout()
{
	return static_cast<double>(m_nTimeout);
}

void CSerialBase::SetWriteTimeout(double dTimeout)
{
	if (dTimeout < 0)
		assert(false);

	m_nWriteTimeout = static_cast<int>(dTimeout);

	if (m_bOpen)
		ReconfigurePort();

}

double CSerialBase::GetWriteTimeout()
{
	return static_cast<double>(m_nWriteTimeout);
}

void CSerialBase::SetXonXoff(int nXonXoff)
{
	m_nXonXoff = nXonXoff;

	if (m_bOpen)
		ReconfigurePort();
}

int CSerialBase::GetXonXoff()
{
	return m_nXonXoff;
}

void CSerialBase::SetRtsCts(int nRtsCts)
{
	m_nRtsCts = nRtsCts;

	if (m_bOpen)
		ReconfigurePort();
}

int CSerialBase::GetRtsCts()
{
	return m_nRtsCts;
}

void CSerialBase::SetDsrDtr(int nDsrDtr)
{
	if (nDsrDtr == 0)
		m_nDsrDtr = m_nRtsCts;
    else
        m_nDsrDtr = nDsrDtr;
	
	if (m_bOpen)
		ReconfigurePort();
}

int CSerialBase::GetDsrDtr()
{
	return m_nDsrDtr;
}