#pragma once

#include "DriverBase.h"


enum STOPBITS
{
	STOPBITS_ONE = 1, 
	STOPBITS_TWO
};

enum BYTESIZE
{
	FIVEBITS = 5,
	SIXBITS,
	SEVENBITS,
	EIGHTBITS
};


//#define PARITY_NONE		'None';
//#define PARITY_EVEN		'Even';
//#define PARITY_ODD		'Odd';

#define XON				17;
#define XOFF			19;

static const int BAUDRATES[] = {50,75,110,134,150,200,300,600,1200,1800,2400,4800,9600,\
								19200,38400,57600,115200,230400,460800,500000,576000,921600,\
								1000000,1152000,1500000,2000000,2500000,3000000,3500000,4000000};
static const int BYTESIZES[] = {FIVEBITS, SIXBITS, SEVENBITS, EIGHTBITS};
static const int PARITIES[]  = {PARITY_NONE, PARITY_EVEN, PARITY_ODD};
static const int STOPBITS[]  = {STOPBITS_ONE, STOPBITS_TWO};

class CSerialBase : public CDriverBase
{
public:
	CSerialBase(bool bLog = false,
				unsigned nPort = 0, 
				UINT nBaudrate = 9600,
				int nBytesize = EIGHTBITS,
				WORD wParity = PARITY_NONE,
				int nStopbits = STOPBITS_ONE,
				int nTimeout = -1,
				int nXOnXOff = 0,
				int nRtsCts = 0,
				int nWriteTimeout = 0,
				int nDsrDtr = 0
				);
	virtual ~CSerialBase(void);

	virtual bool Open() = 0;
	virtual void Close() = 0;
	virtual unsigned int InWaiting(void) = 0;
	virtual string Read(UINT nSize = 0) = 0;
	virtual bool Read(BYTE* byRead, UINT nSize) = 0;
	virtual bool Write(string strWrite) = 0;
	virtual bool Write(BYTE* byWrite, UINT nSize) = 0;
	virtual void ReconfigurePort() = 0;

	string MakeDeviceName(int nPort);

	const int* GetSupportedBaudrates();
	const int* GetSupportedByteSizes();
	const int* GetSupportedStopbits();
	const int* GetSupportedParities();
	void SetPort(int nPort);
	int GetPort();
	void SetBaudrate(UINT nBaudrate);
	UINT GetBaudrate();
	void SetByteSize(int nBytesize);
	int GetByteSize();
	void SetParity(WORD wParity);
	WORD GetParity();
	void SetStopbits(int nStopbits);
	int GetStopbits();
	void SetTimeout(double dTimeout);
	double GetTimeout();
	void SetWriteTimeout(double dTimeout);
	double GetWriteTimeout();
	void SetXonXoff(int nXonXoff);
	int GetXonXoff();
	void SetRtsCts(int nRtsCts);
	int GetRtsCts();
	void SetDsrDtr(int nDsrDtr);
	int GetDsrDtr();

	virtual void FlushInput(void) = 0;
	virtual bool IsOpen() = 0;

protected:
	int m_nPort;
	string m_sPort;
    UINT m_dwBaudrate;
    int m_nBytesize;
    WORD m_wParity;
    int m_nStopbits;
    int m_nTimeout;
    int m_nWriteTimeout;
    int m_nXonXoff;
    int m_nRtsCts;
    int m_nDsrDtr;
};
