#include "StdAfx.h"
#include "TCP.h"

CTCP::CTCP(bool bLog, bool bServer, LPCTSTR sAddr, unsigned short nPort)
: CTCPBase(bLog, sAddr, nPort), m_bServer(bServer), m_sAddr(sAddr), m_nPort(nPort), m_hSocket(NULL)
{
	if (WSAStartup(MAKEWORD(2,2), &m_wsaData) != 0)
	{
		CIronLib::WriteDebugLog("WSAStartup() error!(%s:%d)", __FILE__, __LINE__);
	}

	if (m_bServer)
	{
		m_hStopEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

		assert(m_hStopEvent);
	}

	Open();
}

CTCP::~CTCP(void)
{
	if (m_bServer)
	{
		SetEvent(m_hStopEvent);
		WaitFor();

		CloseHandle(m_hStopEvent);
	}

	Close();

	WSACleanup();
}

void CTCP::OnExecute()
{
	SOCKET listen_sock;

	listen_sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	
	bool block = false;
	ioctlsocket(listen_sock, FIONBIO, (u_long*)&block);

	if(listen_sock == INVALID_SOCKET)
	{
		CIronLib::WriteDebugLog("Invalid Socket. ErrCode : %d(%s:%d)", WSAGetLastError(), __FILE__, __LINE__);
	}

	memset(&m_sockAddr, 0, sizeof(m_sockAddr));

	SOCKADDR_IN serv_addr;
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_port = htons(m_nPort);
	serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);

	int retval = bind(listen_sock, (struct sockaddr *)&serv_addr, sizeof(serv_addr));

	if(retval == SOCKET_ERROR)
	{
		CIronLib::WriteDebugLog("bind() Error. ErrCode : %d(%s:%d)", WSAGetLastError(), __FILE__, __LINE__);
	}

	retval = listen(listen_sock, 1);

	if(retval == SOCKET_ERROR)
	{
		CIronLib::WriteDebugLog("listen() Error. ErrCode : %d(%s:%d)", WSAGetLastError(), __FILE__, __LINE__);
	}

	while (WaitForSingleObject(m_hStopEvent, 500))
	{
		if (m_hSocket == NULL || m_hSocket == INVALID_SOCKET)
		{
			int addr_size = sizeof(m_sockAddr);
			m_hSocket = accept(listen_sock, (SOCKADDR*)&m_sockAddr, &addr_size);
		}
	}

	closesocket(listen_sock);
}

bool CTCP::Open()
{
	if (m_bServer)
	{
		BeginThread();
	}
	else
	{
		m_hSocket = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);

		if (m_hSocket == INVALID_SOCKET)
		{
			CIronLib::WriteDebugLog("Invalid Socket. ErrCode : %d(%s:%d)", WSAGetLastError(), __FILE__, __LINE__);
			return false;
		}

		memset(&m_sockAddr, 0, sizeof(m_sockAddr));

		m_sockAddr.sin_family = AF_INET;
		m_sockAddr.sin_addr.s_addr = inet_addr(m_sAddr.c_str());
		m_sockAddr.sin_port = htons(m_nPort);
	
		if (connect(m_hSocket, (SOCKADDR*)&m_sockAddr, sizeof(m_sockAddr)) == SOCKET_ERROR)
		{
			CIronLib::WriteDebugLog("connect() Error. ErrCode : %d(%s:%d)", WSAGetLastError(), __FILE__, __LINE__);
			return false;
		}
	}

	m_bOpen = true;

	return true;
}

void CTCP::Close()
{
	if (m_hSocket != NULL && m_hSocket != INVALID_SOCKET)
	{
		closesocket(m_hSocket);
		m_hSocket = NULL;
	}

	m_bOpen = false;
}

unsigned int CTCP::InWaiting(void)
{
	u_long read = 0;

	ioctlsocket(m_hSocket, FIONREAD, &read);

	return read;
}

string CTCP::Read(UINT nSize)
{
	if (m_hSocket == NULL && m_hSocket == INVALID_SOCKET)
	{
		CIronLib::WriteDebugLog("Don't create Socket or Invalid Socket.(%s:%d)", __FILE__, __LINE__);
		return "";
	}

	if (nSize > MAX_READBUFF_SIZE)
	{
		CIronLib::WriteDebugLog("ReadSize is bigger than buffer size.(%s:%d)", __FILE__, __LINE__);
		
		nSize = MAX_READBUFF_SIZE;
	}

	int nReadSize = 0;
	bool timeout;
	char readBuff[MAX_READBUFF_SIZE + 1];

	ZERO_MEMORY(readBuff);

	if (nSize < 0)
	{
		CIronLib::WriteDebugLog("negative buffersize in recv(%s:%d)", __FILE__, __LINE__);
		return "";
	}

	timeout = InternalSelect(false);

	if (!timeout)
	{
		nReadSize = recv(m_hSocket, readBuff, nSize, 0);
	}
	else
	{
		CIronLib::WriteDebugLog("recv() timeout.(%s:%d)", __FILE__, __LINE__);
		return "";
	}

	if (nReadSize == SOCKET_ERROR)
	{
		CIronLib::WriteDebugLog("recv() Error. ErrCode : %d(%s:%d)", WSAGetLastError(), __FILE__, __LINE__);
		return "";
	}

	WriteLog(string(readBuff), "Read ");

	return string(readBuff);
}

bool CTCP::Read(BYTE* byRead, UINT nSize)
{
	if (m_hSocket == NULL && m_hSocket == INVALID_SOCKET)
	{
		CIronLib::WriteDebugLog("Don't create Socket or Invalid Socket.(%s:%d)", __FILE__, __LINE__);
		return "";
	}

	int nReadSize = 0;
	bool timeout;

	memset(byRead, 0, nSize);

	if (nSize < 0)
	{
		CIronLib::WriteDebugLog("negative buffersize in recv(%s:%d)", __FILE__, __LINE__);
		return "";
	}

	timeout = InternalSelect(false);

	if (!timeout)
	{
		nReadSize = recv(m_hSocket, (char*)byRead, nSize, 0);
	}
	else
	{
		CIronLib::WriteDebugLog("recv() timeout.(%s:%d)", __FILE__, __LINE__);
		return "";
	}

	if (nReadSize == SOCKET_ERROR)
	{
		CIronLib::WriteDebugLog("recv() Error. ErrCode : %d(%s:%d)", WSAGetLastError(), __FILE__, __LINE__);
		return false;
	}

	WriteLog(byRead, nReadSize, "Read ");

	if (nReadSize != nSize)
		return false;

	return true;
}

bool CTCP::Write(string strWrite)
{
	if (m_hSocket == NULL && m_hSocket == INVALID_SOCKET)
	{
		CIronLib::WriteDebugLog("Don't create Socket or Invalid Socket.(%s:%d)", __FILE__, __LINE__);
		return false;
	}

	int nWriteSize = 0;
	bool timeout;

	timeout = InternalSelect(true);

	if (!timeout)
	{
		nWriteSize = send(m_hSocket, strWrite.c_str(), static_cast<int>(strWrite.size()), 0);
	}
	else
	{
		CIronLib::WriteDebugLog("send() timeout.(%s:%d)", __FILE__, __LINE__);
		return false;
	}

	if (nWriteSize == SOCKET_ERROR)
	{
		CIronLib::WriteDebugLog("send() Error. ErrCode : %d(%s:%d)", ::WSAGetLastError(), __FILE__, __LINE__);
		return false;
	}

	WriteLog(strWrite, "Write");

	return true;
}

bool CTCP::Write(BYTE* byWrite, UINT nSize)
{
	if (m_hSocket == NULL && m_hSocket == INVALID_SOCKET)
	{
		CIronLib::WriteDebugLog("Don't create Socket or Invalid Socket.(%s:%d)", __FILE__, __LINE__);
		return false;
	}

	int nWriteSize = 0;
	bool timeout;

	timeout = InternalSelect(true);

	if (!timeout)
	{
		nWriteSize = send(m_hSocket, (char*)byWrite, nSize, 0);
	}
	else
	{
		CIronLib::WriteDebugLog("send() timeout.(%s:%d)", __FILE__, __LINE__);
		return false;
	}

	if (nWriteSize == SOCKET_ERROR)
	{
		CIronLib::WriteDebugLog("send() Error. ErrCode : %d(%s:%d)", WSAGetLastError(), __FILE__, __LINE__);
		return false;
	}

	WriteLog(byWrite, nSize, "Write");

	return true;
}

bool CTCP::InternalSelect(bool bWrite)
{
	fd_set fds;
	struct timeval tv;
	int n;

	if (m_dTimeout <= 0.0)
		return false;

	if (m_hSocket < 0)
		return false;

	tv.tv_sec = static_cast<int>(m_dTimeout);
	tv.tv_usec = static_cast<int>((m_dTimeout - tv.tv_sec) * 1e6);
	FD_ZERO(&fds);
	FD_SET(m_hSocket, &fds);

	if (bWrite)
		n = select(0, NULL, &fds, NULL, &tv);
	else
		n = select(0, &fds, NULL, NULL, &tv);

	if (n == 0)
		return true;

	return false;
}

void CTCP::SetBlocking(bool block)
{
	block = !block;
	ioctlsocket(m_hSocket, FIONBIO, (u_long*)&block);
}

void CTCP::FlushInput(void)
{
	double dOldTimeout = GetTimeout();

	SetTimeout(0.0);

	xreadlines();

	SetTimeout(dOldTimeout);
}