#include "StdAfx.h"
#include "UDP.h"

CUDP::CUDP(bool bServer, bool bLog, LPCTSTR sAddr, unsigned short nPort)
: CTCPBase(bLog, sAddr, nPort), m_bServer(bServer), m_sAddr(sAddr), m_nPort(nPort), m_hSocket(NULL), m_servSize(0)
{
	if (WSAStartup(MAKEWORD(2,2), &m_wsaData) != 0)
	{
		CIronLib::WriteDebugLog("WSAStartup() error!(%s:%d)", __FILE__, __LINE__);
	}
}

CUDP::~CUDP(void)
{
	WSACleanup();
}

bool CUDP::Open()
{
	m_hSocket = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);

	if (m_hSocket == INVALID_SOCKET)
	{
		CIronLib::WriteDebugLog("Invalid Socket!(%s:%d)", __FILE__, __LINE__);
		return false;
	}

	memset(&m_servAddr, 0, sizeof(m_servAddr));

	m_servAddr.sin_family = AF_INET;
	m_servAddr.sin_addr.s_addr = inet_addr(m_sAddr.c_str());
	m_servAddr.sin_port = htons(m_nPort);
	m_servSize = sizeof(m_servAddr);

	if (m_bServer)
		bind(m_hSocket, (sockaddr*)&m_servAddr, sizeof(m_servAddr));

	m_bOpen = true;

	return true;
}

void CUDP::Close()
{
	if (m_hSocket != NULL && m_hSocket != INVALID_SOCKET)
	{
		closesocket(m_hSocket);
	}

	m_bOpen = false;
}

string CUDP::Read(UINT nSize)
{
	if (m_hSocket == NULL && m_hSocket == INVALID_SOCKET)
	{
		CIronLib::WriteDebugLog("Don't create Socket or Invalid Socket!(%s:%d)", __FILE__, __LINE__);
		return "";
	}

	if (nSize > MAX_READBUFF_SIZE)
	{
		CIronLib::WriteDebugLog("ReadSize is bigger than buffer size.(%s:%d)", __FILE__, __LINE__);
		
		nSize = MAX_READBUFF_SIZE;
	}

	int nReadSize = 0;
	bool timeout;
	char readBuff[MAX_READBUFF_SIZE + 1];

	ZERO_MEMORY(readBuff);

	if (nSize < 0)
	{
		CIronLib::WriteDebugLog("negative buffersize in recv(%s:%d)", __FILE__, __LINE__);
		return "";
	}

	timeout = InternalSelect(false);

	if (!timeout)
	{
		int fromSize = sizeof(m_fromAddr);
		nReadSize = recvfrom(m_hSocket, readBuff, nSize, 0, (sockaddr*)&m_fromAddr, &fromSize);
	}
	else
	{
		CIronLib::WriteDebugLog("recv() timeout.(%s:%d)", __FILE__, __LINE__);
		return "";
	}

	if (nReadSize < 0)
	{
		return "";
	}

	WriteLog(string(readBuff), "Read ");

	return string(readBuff);
}

bool CUDP::Write(string strWrite)
{
	if (m_hSocket == NULL && m_hSocket == INVALID_SOCKET)
	{
		CIronLib::WriteDebugLog("Don't create Socket or Invalid Socket!(%s:%d)", __FILE__, __LINE__);
		return false;
	}

	int nWriteSize = 0;
	bool timeout;

	timeout = InternalSelect(true);

	if (!timeout)
	{
		nWriteSize = sendto(m_hSocket, strWrite.c_str(), static_cast<int>(strWrite.size()), 0, (sockaddr*)&m_servAddr, m_servSize);;
	}
	else
	{
		CIronLib::WriteDebugLog("send() timeout.(%s:%d)", __FILE__, __LINE__);
		return false;
	}

	WriteLog(strWrite, "Write");

	if (nWriteSize == SOCKET_ERROR)
	{
		CIronLib::WriteDebugLog("Don't create Socket or Invalid Socket!(%s:%d)", __FILE__, __LINE__);
		return false;
	}

	return true;
}

bool CUDP::InternalSelect(bool bWrite)
{
	fd_set fds;
	struct timeval tv;
	int n;

	if (m_dTimeout <= 0.0)
		return false;

	if (m_hSocket < 0)
		return false;

	tv.tv_sec = static_cast<int>(m_dTimeout);
	tv.tv_usec = static_cast<int>((m_dTimeout - tv.tv_sec) * 1e6);
	FD_ZERO(&fds);
	FD_SET(m_hSocket, &fds);

	if (bWrite)
		n = select(0, NULL, &fds, NULL, &tv);
	else
		n = select(0, &fds, NULL, NULL, &tv);

	if (n == 0)
		return true;

	return false;
}

void CUDP::SetBlocking(bool block)
{
	block = !block;
	ioctlsocket(m_hSocket, FIONBIO, (u_long*)&block);
}

void CUDP::FlushInput(void)
{
	double dOldTimeout = GetTimeout();

	SetTimeout(0.0);

	xreadlines();

	SetTimeout(dOldTimeout);
}