// IronComm.cpp : DLL  α׷   մϴ.
//

#include "stdafx.h"
#include "IronComm.h"

#include <DriverBase.h>
#include <TCP.h>
#include <Serial.h>

#include <string>
#include <map>

using namespace std;



CDriverBase *deviceArray[MAX_DEVICE];
bool serialTypeArray[MAX_DEVICE];

unsigned int currentDeviceCount = 0;

#ifdef _MANAGED
#pragma managed(push, off)
#endif

BOOL APIENTRY DllMain( HMODULE hModule,
                       UINT  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
		memset(deviceArray, 0, MAX_DEVICE * sizeof(CDriverBase*));
		memset(serialTypeArray, 0, MAX_DEVICE * sizeof(bool));
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
    return TRUE;
}

#ifdef _MANAGED
#pragma managed(pop)
#endif

map<string, CDriverBase*> deviceMap;
map<LPVOID, string> objectMap;

IRONDEVICEMASTER_API WORD OpenPort(LPCTSTR address, WORD port, bool logging)
{
	CDriverBase *pDevice = NULL;
	bool serialType = false;

	if (strcmp(address, "COM") == 0)
	{
		serialType = true;
		pDevice = new CSerial(logging, port);
	}
	else
	{
		pDevice = new CTCP(logging, false, address, port);
	}

	if (!pDevice->IsOpen() || currentDeviceCount + 1 >= MAX_DEVICE)
	{
		delete pDevice;
		return NULL;
	}

	serialTypeArray[currentDeviceCount] = serialType;
	deviceArray[currentDeviceCount++] = pDevice;

	return currentDeviceCount;
}

IRONDEVICEMASTER_API WORD OpenSerialPort(bool logging,
										 WORD port,
										 UINT nBaudrate,
										 int nBytesize,
										 WORD wParity,
										 int nStopbits,
										 int nTimeout,
										 int nXOnXOff,
										 int nRtsCts,
										 int nWriteTimeout,
										 int nDsrDtr)
{
	CDriverBase *pDevice = NULL;

	pDevice = new CSerial(logging, port, nBaudrate, nBytesize, wParity, nStopbits, nTimeout, nXOnXOff, nRtsCts, nWriteTimeout, nDsrDtr);

	if (!pDevice->IsOpen() || currentDeviceCount + 1 >= MAX_DEVICE)
	{
		delete pDevice;
		return NULL;
	}

	serialTypeArray[currentDeviceCount] = true;
	deviceArray[currentDeviceCount++] = pDevice;

	return currentDeviceCount;
}

IRONDEVICEMASTER_API WORD CreateTCPServer(LPCTSTR address, WORD port, bool logging)
{
	CDriverBase *pDevice = NULL;
	bool serialType = false;

	pDevice = new CTCP(logging, true, address, port);

	if (!pDevice->IsOpen() || currentDeviceCount + 1 >= MAX_DEVICE)
	{
		delete pDevice;
		return NULL;
	}

	serialTypeArray[currentDeviceCount] = serialType;
	deviceArray[currentDeviceCount++] = pDevice;

	return currentDeviceCount;
}

IRONDEVICEMASTER_API void ClosePort(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL)
	{
		delete deviceArray[portID - 1];
		deviceArray[portID - 1] = NULL;
		serialTypeArray[portID - 1] = NULL;
	}
}

IRONDEVICEMASTER_API UINT InWaiting(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL)
	{
		return deviceArray[portID - 1]->InWaiting();
	}

	return 0;
}


IRONDEVICEMASTER_API bool ReadLine(UINT portID, LPSTR strRead, UINT bufferSize, char EOL)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL)
	{
		string readData = deviceArray[portID - 1]->ReadLine(EOL, bufferSize);

		memset(strRead, 0, bufferSize);
		memcpy_s(strRead, bufferSize, readData.c_str(), readData.size());

		return strcmp(&readData[readData.size() - 1], &EOL) == 0;
	}

	return false;
}

IRONDEVICEMASTER_API bool ReadString(UINT portID, LPSTR strRead, UINT bufferSize, UINT readSize)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL)
	{
		string readData = deviceArray[portID - 1]->Read(readSize);

		memset(strRead, 0, bufferSize);
		memcpy_s(strRead, bufferSize, readData.c_str(), readData.size());

		return readData.size() == readSize;
	}

	return false;
}

IRONDEVICEMASTER_API bool ReadBinary(UINT portID, BYTE* byRead, UINT bufferSize, UINT readSize)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL)
	{
		return deviceArray[portID - 1]->Read(byRead, readSize);
	}

	return false;
}

IRONDEVICEMASTER_API bool WriteString(UINT portID, LPCTSTR strWrite)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL)
	{
		return deviceArray[portID - 1]->Write(strWrite);
	}
	
	return false;
}

IRONDEVICEMASTER_API bool WriteBinary(UINT portID, BYTE* byWrite, UINT writeSize)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL)
	{
		return deviceArray[portID - 1]->Write(byWrite, writeSize);
	}

	return false;
}

IRONDEVICEMASTER_API void FlushInput(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL)
	{
		deviceArray[portID - 1]->FlushInput();
	}
}

//Serial Property
IRONDEVICEMASTER_API void SetSerialPort(UINT portID, int nPort)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		pSerial->SetPort(nPort);
	}
}

IRONDEVICEMASTER_API int GetSerialPort(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		return pSerial->GetPort();
	}

	return -1;
}

IRONDEVICEMASTER_API void SetBaudrate(UINT portID, UINT nBaudrate)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		pSerial->SetBaudrate(nBaudrate);
	}
}

IRONDEVICEMASTER_API UINT GetBaudrate(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		return pSerial->GetBaudrate();
	}

	return -1;
}

IRONDEVICEMASTER_API void SetByteSize(UINT portID, int nBytesize)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		pSerial->SetByteSize(nBytesize);
	}
}

IRONDEVICEMASTER_API int GetByteSize(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		return pSerial->GetByteSize();
	}

	return -1;
}

IRONDEVICEMASTER_API void SetParity(UINT portID, WORD wParity)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		pSerial->SetParity(wParity);
	}
}

IRONDEVICEMASTER_API WORD GetParity(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		return pSerial->GetParity();
	}

	return -1;
}

IRONDEVICEMASTER_API void SetStopbits(UINT portID, int nStopbits)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		pSerial->SetStopbits(nStopbits);
	}
}

IRONDEVICEMASTER_API int GetStopbits(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		return pSerial->GetStopbits();
	}

	return -1;
}

IRONDEVICEMASTER_API void SetTimeout(UINT portID, double dTimeout)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		pSerial->SetTimeout(dTimeout);
	}
}

IRONDEVICEMASTER_API double GetTimeout(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		return pSerial->GetTimeout();
	}

	return -1;
}

IRONDEVICEMASTER_API void SetWriteTimeout(UINT portID, double dTimeout)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		pSerial->SetWriteTimeout(dTimeout);
	}
}

IRONDEVICEMASTER_API double GetWriteTimeout(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		return pSerial->GetWriteTimeout();
	}

	return -1;
}

IRONDEVICEMASTER_API void SetXonXoff(UINT portID, int nXonXoff)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		pSerial->SetXonXoff(nXonXoff);
	}
}

IRONDEVICEMASTER_API int GetXonXoff(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		return pSerial->GetXonXoff();
	}

	return -1;
}

IRONDEVICEMASTER_API void SetRtsCts(UINT portID, int nRtsCts)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		pSerial->SetRtsCts(nRtsCts);
	}
}

IRONDEVICEMASTER_API int GetRtsCts(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		return pSerial->GetRtsCts();
	}

	return -1;
}

IRONDEVICEMASTER_API void SetDsrDtr(UINT portID, int nDsrDtr)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		pSerial->SetDsrDtr(nDsrDtr);
	}
}

IRONDEVICEMASTER_API int GetDsrDtr(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		return pSerial->GetDsrDtr();
	}

	return -1;
}

IRONDEVICEMASTER_API void FlushOutput(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		pSerial->FlushOutput();
	}
}

IRONDEVICEMASTER_API void SendBreak(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		pSerial->SendBreak();
	}
}

IRONDEVICEMASTER_API void SetRTS(UINT portID, bool bLevel)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		pSerial->SetRTS(bLevel);
	}
}

IRONDEVICEMASTER_API void SetDTR(UINT portID, bool bLevel)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		pSerial->SetDTR(bLevel);
	}
}

IRONDEVICEMASTER_API bool GetCTS(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		return pSerial->GetCTS();
	}

	return false;
}

IRONDEVICEMASTER_API bool GetDSR(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		return pSerial->GetDSR();
	}

	return false;
}

IRONDEVICEMASTER_API bool GetRI(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		return pSerial->GetRI();
	}

	return false;
}

IRONDEVICEMASTER_API bool GetCD(UINT portID)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		return pSerial->GetCD();
	}

	return false;
}

IRONDEVICEMASTER_API void SetXON(UINT portID, bool bLevel)
{
	if (portID <= MAX_DEVICE && deviceArray[portID - 1] != NULL && serialTypeArray[portID - 1])
	{
		CSerial* pSerial = (CSerial*)deviceArray[portID - 1];

		pSerial->SetXON(bLevel);
	}
}
