#include "StdAfx.h"
#include <math.h>
#include "DeviceInfo.h"
#include "Device.h"
#include "VirtualActiveElement.h"

static CIronCriticalSection g_cLock;

CDevice::CDevice(bool bReal, bool bThread, unsigned uPollingTime, unsigned uOutput, string strDevice, string strDriverFile, string strIOFile)
: m_bReal(bReal), m_bThread(bThread), m_uPollingTime(uPollingTime), m_uOutput(uOutput), m_strDevice(strDevice), m_strDriverName(strDriverFile), m_strIOFile(strIOFile), m_pDriver(NULL), m_bStart(false)
{

	m_strDriverFolder = CDeviceInfo::GetApplicationFolder() + "Driver\\";
}

CDevice::~CDevice(void)
{
	
}

void CDevice::OnExecute()
{
	CoInitializeEx(NULL, COINIT_MULTITHREADED);

	HRESULT hr = E_FAIL;

	CVirtualActiveElement* pVirDB = NULL;

	if (m_strDriverName == "Virtual")
	{
		pVirDB = new CVirtualActiveElement();

		hr = pVirDB->OpenAll();

		if (FAILED(hr))
		{
			CIronLib::WriteDebugLog("Virtual DB Open Fail.");
			CoUninitialize();
			return;
		}
	}

	while (WaitForSingleObject(m_hStopEvent, m_uPollingTime))
	{
		if (m_strDriverName =="Virtual")
		{
			hr = pVirDB->MoveFirst();

			while (SUCCEEDED(hr) && hr != DB_S_ENDOFROWSET)
			{
				string strIndex = "VE.";

				strIndex += pVirDB->m_Driver;
				strIndex += ".";
				strIndex += pVirDB->m_Name;

				if (m_mVirtual.find(strIndex) != m_mVirtual.end())
				{
					if (m_mVirtual[strIndex] != pVirDB->m_Setting)
					{
						strcpy_s(pVirDB->m_Setting, 51, (LPCTSTR)m_mVirtual[strIndex].c_str());
						pVirDB->m_dwSettingLength = static_cast<DBLENGTH>(m_mVirtual[strIndex].size());

						if( pVirDB->m_dwSettingLength <= 0 )
							pVirDB->m_dwSettingStatus = DBSTATUS_S_ISNULL;
						else
							pVirDB->m_dwSettingStatus = DBSTATUS_S_OK;

						hr = pVirDB->SetData();

						if( FAILED(hr) )
						{
							CIronLib::WriteDebugLog("Virtual DB Update Fail.(%s:%d)", __FUNCTION__, __LINE__);
							pVirDB->CloseAll();
							CoUninitialize();
							return;
						}
					}
				}

				hr = pVirDB->MoveNext();
			}
		}
		else
		{
			for (map<string, IO_INFO>::iterator iter = m_mIOInfo.begin() ; iter != m_mIOInfo.end() ; ++iter)
			{
				string strRead;

				if (ReadIO(iter->first, strRead))
				{
					CIronLock cLock(g_cLock);
					m_mReadVal[iter->first] = strRead;
				}
			}
		}
	}
	
	if (pVirDB != NULL)
	{
		pVirDB->CloseAll();
		delete pVirDB;
	}

	CoUninitialize();
}

bool CDevice::Start(unsigned uPort, int nTimeout, bool bLog)
{
	if (m_bStart == true)
	{
		CIronLib::WriteDebugLog("Already Start Driver[%s].(%s)", m_strDriverName.c_str(), __FUNCTION__);
		return false;
	}

	m_bStart = true;

	if (m_strDriverName == "Virtual")
	{
		if (!GetVirtualIO())
			return false;

		m_hStopEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
		assert(m_hStopEvent);
		BeginThread();
	}
	else
	{
		if (!GetDriverTag())
			return false;

		if (m_bReal)
		{
			if (!ConnectDriver())
			{
				DisconnectDriver();
				return false;
			}

			HRESULT hr = m_pDriver->lpInitDriver(uPort, nTimeout, bLog);

			if (FAILED(hr))
			{
				CIronLib::WriteDebugLog("Driver[%s] is not initialize.(%s)", m_strDriverName.c_str(), __FUNCTION__);
				return false;
			}

			for (map<string, IO_INFO>::iterator iter = m_mIOInfo.begin(); iter != m_mIOInfo.end(); ++iter)
			{
				if (iter->second.nType == IOTYPE_VARIABLE && iter->second.uIndex == 1)
				{
					string strRead;

					ReadIO(iter->first, strRead);
				}
			}

			if (m_bThread)
			{
				m_hStopEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
				assert(m_hStopEvent);
				BeginThread();
				CIronLib::WriteDebugLog("%s Read Thread Start", m_strDriverName.c_str());
			}
		}
	}

	return true;
}

void CDevice::Stop()
{
	if (m_bStart == false)
	{
		CIronLib::WriteDebugLog("Don't Start Driver[%s].(%s)", m_strDriverName.c_str(), __FUNCTION__);
		return;
	}

	if (m_strDriverName == "Virtual")
	{
		SetEvent(m_hStopEvent);
		WaitFor();
		CloseHandle(m_hStopEvent);

		//Virtaul DB Update
		CVirtualActiveElement virDB;

		HRESULT hr = virDB.OpenAll();

		if (FAILED(hr))
		{
			CIronLib::WriteDebugLog("Virtual DB Open Fail");
			return;
		}

		hr = virDB.MoveFirst();

		while (SUCCEEDED(hr) && hr != DB_S_ENDOFROWSET)
		{
			string strIndex = "VE.";

			strIndex += virDB.m_Driver;
			strIndex += ".";
			strIndex += virDB.m_Name;

			if (m_mVirtual.find(strIndex) != m_mVirtual.end())
			{
				strcpy_s(virDB.m_Setting, 51, (LPCTSTR)m_mVirtual[strIndex].c_str());
				virDB.m_dwSettingLength = static_cast<DBLENGTH>(m_mVirtual[strIndex].size());

				if( virDB.m_dwSettingLength <= 0 )
					virDB.m_dwSettingStatus = DBSTATUS_S_ISNULL;
				else
					virDB.m_dwSettingStatus = DBSTATUS_S_OK;
			}

			hr = virDB.SetData();

			if( FAILED(hr) )
			{
				CIronLib::WriteDebugLog("Virtual DB Update Fail.(%s:%d)", __FUNCTION__, __LINE__);
				virDB.CloseAll();
				return;
			}

			hr = virDB.MoveNext();
		}

		virDB.CloseAll();
		return;
	}
	else
	{
		if (m_bStart)
		{
			if (m_bReal)
			{
				if (m_bThread)
				{
					SetEvent(m_hStopEvent);
					WaitFor();

					CloseHandle(m_hStopEvent);
				}

				DisconnectDriver();
			}
		}
	}

	m_bStart = false;
}

bool CDevice::GetVirtualIO()
{
	CVirtualActiveElement virDB;

	HRESULT hr = virDB.OpenAll();

	if (FAILED(hr))
	{
		CIronLib::WriteDebugLog("Virtual DB Open Fail(%x).(%s:%d)", hr, __FILE__, __LINE__);
		return false;
	}

	hr = virDB.MoveFirst();

	m_mVirtual.clear();

	while (SUCCEEDED(hr) && hr != DB_S_ENDOFROWSET)
	{
		string strIndex = "VE.";

		strIndex += virDB.m_Driver;
		strIndex += ".";
		strIndex += virDB.m_Name;

		if (m_mVirtual.find(strIndex) != m_mVirtual.end())
		{
			CIronLib::WriteDebugLog("Virtual %s is redefinition.", strIndex.c_str());
		}
		else
		{
			m_mVirtual[strIndex] = virDB.m_Setting;
		}

		IO_INFO info;

		info.nType = IOTYPE_VIRTUAL;
		info.uAddress = virDB.m_IOAddress;
		info.uIndex = virDB.m_IOIndex;
		info.uTimeout = virDB.m_Timeout;
		info.dOffset = 0;
		info.dGain = 0;
		info.dMin = virDB.m_Minimum;
		info.dMax = virDB.m_Maximum;

		if (m_mIOInfo.find(strIndex) != m_mIOInfo.end())
			CIronLib::WriteDebugLog("%s is redefinition.(%s)", strIndex.c_str(), __FUNCTION__);
		else
			m_mIOInfo[strIndex] = info;

		hr = virDB.MoveNext();
	}

	virDB.CloseAll();

	return true;
}

bool CDevice::GetDriverTag()
{
	FILE *stream;
	char line[1024];

	if( (stream = _fsopen(m_strIOFile.c_str(), "r", _SH_DENYWR)) == NULL )
	{
		CIronLib::WriteDebugLog("%s File Open Fail.(%s)", m_strIOFile.c_str(), __FUNCTION__);
		return false;
	}

	
	string sIOType;
	int nType = IOTYPE_NONE;
	vector<string> item;

	while( !feof(stream) )
	{
		memset(line, '\0', 1024);

		if( fgets( line, 1024, stream ) != NULL)
		{
			string temp(line);

			if( temp.find("//") < temp.size() )
				continue;

			if( temp.find("%") < temp.size() )
			{
				if( temp.find("AnalogInput") < temp.size() )
				{
					sIOType = "AI.";
					nType = IOTYPE_ANALOG_INPUT;
				}
				else if( temp.find("AnalogOutput") < temp.size() )
				{
					sIOType = "AO.";
					nType = IOTYPE_ANALOG_OUTPUT;
				}
				else if( temp.find("DigitalInput") < temp.size() )
				{
					sIOType = "DI.";
					nType = IOTYPE_DIGITAL_INPUT;
				}
				else if( temp.find("DigitalOutput") < temp.size() )
				{
					sIOType = "DO.";
					nType = IOTYPE_DIGITAL_OUTPUT;
				}
				else if( temp.find("StringInput") < temp.size() )
				{
					sIOType = "SI.";
					nType = IOTYPE_STRING_INPUT;
				}
				else if( temp.find("StringOutput") < temp.size() )
				{
					sIOType = "SO.";
					nType = IOTYPE_STRING_OUTPUT;
				}
				else if ( temp.find("Variable") < temp.size() )
				{
					sIOType = "VAR.";
					nType = IOTYPE_VARIABLE;
				}
				else if( temp.find("Volatile") < temp.size() )
				{
					sIOType = "VT.";
					nType = IOTYPE_VIRTUAL;
				}
				else
				{
					sIOType = "";
					nType = IOTYPE_NONE;
				}

				continue;
			}


			item = CIronLib::split(temp, "|");

			//if( item[0] == "File" )
			//	GetIOData(strConfigFolder + item[1]);

			if( nType > IOTYPE_NONE && nType < IOTYPE_VARIABLE && item.size() == 8 )
			{
				string strIndex = sIOType;
				strIndex += m_strDevice;
				strIndex += ".";
				strIndex += item[0];

				IO_INFO info;

				info.nType = nType;
				info.uAddress = atoi(item[1].c_str());
				info.uIndex = atoi(item[2].c_str());
				info.uTimeout = atoi(item[3].c_str());
				info.dOffset = atof(item[4].c_str());
				info.dGain = atof(item[5].c_str());
				info.dMin = atof(item[6].c_str());
				info.dMax = atof(item[7].c_str());
				
				if (m_mIOInfo.find(strIndex) != m_mIOInfo.end())
				{
					CIronLib::WriteDebugLog("%s is redefinition.(%s)", strIndex.c_str(), __FUNCTION__);
				}
				else
				{
					m_mIOInfo[strIndex] = info;
					m_mSimulValue[strIndex] = "0";
				}
			}
			else if( nType == IOTYPE_VIRTUAL && item.size() == 9)
			{
				string strIndex = sIOType;
				strIndex += m_strDevice;
				strIndex += ".";
				strIndex += item[0];

				IO_INFO info;

				info.nType = nType;
				info.uAddress = atoi(item[1].c_str());
				info.uIndex = atoi(item[2].c_str());
				info.uTimeout = atoi(item[3].c_str());
				info.dOffset = atof(item[4].c_str());
				info.dGain = atof(item[5].c_str());
				info.dMin = atof(item[6].c_str());
				info.dMax = atof(item[7].c_str());
				
				if (m_mIOInfo.find(strIndex) != m_mIOInfo.end())
				{
					CIronLib::WriteDebugLog("%s is redefinition.(%s)", strIndex.c_str(), __FUNCTION__);
				}
				else
				{
					m_mIOInfo[strIndex] = info;
					string value = item[8];

					if (value.size() > 0 && value.substr(value.size() - 1, 1) == "\n")
					{
						value = value.substr(0, value.size() - 1);
					}

					m_mVirtual[strIndex] = value;
				}				
			}
			else if( nType == IOTYPE_VARIABLE && item.size() == 11)
			{
				string strIndex = sIOType;
				strIndex += m_strDevice;
				strIndex += ".";
				strIndex += item[0];

				IO_INFO info;

				info.nType = nType;
				info.uAddress = atoi(item[1].c_str());
				info.uIndex = atoi(item[2].c_str());
				info.uTimeout = atoi(item[3].c_str());
				info.dOffset = atof(item[4].c_str());
				info.dGain = atof(item[5].c_str());
				info.dMin = atof(item[6].c_str());
				info.dMax = atof(item[7].c_str());
				info.strVarName = item[8];
				info.strVarType = item[9];
				info.nArrySize = atoi(item[10].c_str());
				
				if (info.uAddress > info.nArrySize)
				{
					CIronLib::WriteDebugLog("Address(%d) is big.(%s)", info.uAddress, __FUNCTION__);
					continue;
				}

				if (m_mIOInfo.find(strIndex) != m_mIOInfo.end())
				{
					CIronLib::WriteDebugLog("%s is redefinition.(%s)", strIndex.c_str(), __FUNCTION__);
				}
				else
				{
					m_mIOInfo[strIndex] = info;
					m_mSimulValue[strIndex] = "0";
				}
			}
		}
	}

	fclose(stream);

	return true;
}

bool CDevice::ConnectDriver()
{
	if (m_pDriver != NULL)
	{
		CIronLib::WriteDebugLog("Already Load Driver[%s].(%s)", m_strDriverName.c_str(), __FUNCTION__);
		return false;
	}

	m_pDriver = new DRIVER_FUNCTION();

	m_pDriver->hIODriver = LoadLibrary((m_strDriverName).c_str());
	
	if (m_pDriver->hIODriver == NULL)
	{
		CIronLib::WriteDebugLog("%s Loadlibrary() is fail.(%s)", m_strDriverName.c_str(), __FUNCTION__);
		return false;
	}


	m_pDriver->lpInitDriver = (LPINITDRIVER)GetProcAddress(m_pDriver->hIODriver, "InitDriver");

	if( m_pDriver->lpInitDriver == NULL)
	{
		CIronLib::WriteDebugLog("'InitDriver' Function of %s is not found.(%s)", m_strDriverName.c_str(), __FUNCTION__);
		return false;
	}


	m_pDriver->lpUninitDriver = (LPUNINITDRIVER)GetProcAddress(m_pDriver->hIODriver, "UninitDriver");

	if( m_pDriver->lpUninitDriver == NULL )
	{
		CIronLib::WriteDebugLog("'UninitDriver' Function of %s is not found.(%s)", m_strDriverName.c_str(), __FUNCTION__);
		return false;
	}


	m_pDriver->lpReadAnalog = (LPREADANALOG)GetProcAddress(m_pDriver->hIODriver, "ReadAnalog");
	
	if( !m_pDriver->lpReadAnalog )
	{
		CIronLib::WriteDebugLog("'ReadAnalog' Function of %s is not found.(%s)", m_strDriverName.c_str(), __FUNCTION__);
		return false;
	}


	m_pDriver->lpReadDigital = (LPREADDIGITAL)GetProcAddress(m_pDriver->hIODriver, "ReadDigital");
	
	if( !m_pDriver->lpReadDigital )
	{
		CIronLib::WriteDebugLog("'ReadDigital' Function of %s is not found.(%s)", m_strDriverName.c_str(), __FUNCTION__);
		return false;
	}


	m_pDriver->lpReadString = (LPREADSTRING)GetProcAddress(m_pDriver->hIODriver, "ReadString");
	
	if( !m_pDriver->lpReadString )
	{
		CIronLib::WriteDebugLog("'ReadString' Function of %s is not found.(%s)", m_strDriverName.c_str(), __FUNCTION__);
		return false;
	}


	m_pDriver->lpReadVariable = (LPREADVARIABLE)GetProcAddress(m_pDriver->hIODriver, "ReadVariable");
	
	if( !m_pDriver->lpReadVariable )
	{
		CIronLib::WriteDebugLog("'ReadVariable' Function of %s is not found.(%s)", m_strDriverName.c_str(), __FUNCTION__);
		return false;
	}


	m_pDriver->lpWriteAnalog = (LPWRITEANALOG)GetProcAddress(m_pDriver->hIODriver, "WriteAnalog");
	
	if( !m_pDriver->lpWriteAnalog )
	{
		CIronLib::WriteDebugLog("'WriteAnalog' Function of %s is not found.(%s)", m_strDriverName.c_str(), __FUNCTION__);
		return false;
	}

	
	m_pDriver->lpWriteDigital = (LPWRITEDIGITAL)GetProcAddress(m_pDriver->hIODriver, "WriteDigital");
	
	if( !m_pDriver->lpWriteDigital )
	{
		CIronLib::WriteDebugLog("'WriteDigital' Function of %s is not found.(%s)", m_strDriverName.c_str(), __FUNCTION__);
		return false;
	}

	
	m_pDriver->lpWriteString = (LPWRITESTRING)GetProcAddress(m_pDriver->hIODriver, "WriteString");
	
	if( !m_pDriver->lpWriteString )
	{
		CIronLib::WriteDebugLog("'WriteString' Function of %s is not found.(%s)", m_strDriverName.c_str(), __FUNCTION__);
		return false;
	}


	m_pDriver->lpWriteVariable = (LPWRITEVARIABLE)GetProcAddress(m_pDriver->hIODriver, "WriteVariable");
	
	if( !m_pDriver->lpWriteVariable )
	{
		CIronLib::WriteDebugLog("'WriteVariable' Function of %s is not found.(%s)", m_strDriverName.c_str(), __FUNCTION__);
		return false;
	}

	return true;
}

void CDevice::DisconnectDriver()
{
	if( m_pDriver != NULL )
	{
		if (m_pDriver->lpUninitDriver != NULL)
		{
			m_pDriver->lpUninitDriver();
		}

		if (m_pDriver->hIODriver != NULL)
		{
			FreeLibrary(m_pDriver->hIODriver);
		}

		delete m_pDriver;

		m_pDriver = NULL;
	}
	else
	{
		CIronLib::WriteDebugLog("%s was not loaded. Can't Unload.", m_strDriverName.c_str());
	}
}

bool CDevice::ReadIO(string sIOIndex, string& strVal)
{
	CIronLock cLock(*this);

	if (m_bStart == false)
	{
		CIronLib::WriteDebugLog("Don't Start Driver[%s].(%s)", m_strDriverName.c_str(), __FUNCTION__);
		return false;
	}

	if (m_mIOInfo.find(sIOIndex) == m_mIOInfo.end())
	{
		CIronLib::WriteDebugLog("Invalid IOName[%s].(%s)", sIOIndex.c_str(), __FUNCTION__);
		return false;
	}

	if (m_strDriverName == "Virtual")
	{
		strVal = m_mVirtual[sIOIndex];
		return true;
	}

	if (m_bReal == true)
	{
		if (m_pDriver == NULL)
		{
			CIronLib::WriteDebugLog("Driver[%s] was not loaded.(%s)", m_strDriverName.c_str(), __FUNCTION__);
			return false;
		}
	}
	else
	{
		if (sIOIndex.size() > 2 && sIOIndex.substr(0,2) == "VT")
			strVal = m_mVirtual[sIOIndex];
		else
			strVal = m_mSimulValue[sIOIndex];
		return true;
	}


	HRESULT hr = E_FAIL;
	double dRet = 0;
	int nRet = 0;
	TCHAR strRet[MAX_STRINGIO_SIZE];
	IO_INFO info = m_mIOInfo[sIOIndex];
	
	switch( info.nType )
	{
	case IOTYPE_ANALOG_INPUT:
		{
			hr = m_pDriver->lpReadAnalog(dRet, info.uAddress, info.uIndex, info.uTimeout);
				
			if( SUCCEEDED(hr) )
			{
				if (info.dGain != 0)
					dRet *= info.dGain;
				
				dRet += info.dOffset;

				strVal = CIronLib::itostring(dRet);
			}
		}
		break;
	case IOTYPE_ANALOG_OUTPUT:
		{
			if (m_uOutput > 0)
			{
				hr = m_pDriver->lpReadAnalog(dRet, info.uAddress + m_uOutput, info.uIndex, info.uTimeout);
				
				if( SUCCEEDED(hr) )
				{
					if (info.dGain != 0)
						dRet *= info.dGain;

					dRet += info.dOffset;

					strVal = CIronLib::itostring(dRet);
				}
			}
			else
			{
				if (m_mWriteValue.find(sIOIndex) != m_mWriteValue.end())
				{
					strVal = CIronLib::itostring(m_mWriteValue[sIOIndex].dVal);
				}
				else
				{
					strVal = "";
				}

				hr = S_OK;
			}
		}
		break;
	case IOTYPE_DIGITAL_INPUT:
		{
			hr = m_pDriver->lpReadDigital(nRet, info.uAddress, info.uIndex, info.uTimeout);
	
			if( SUCCEEDED(hr) )
			{
				strVal = CIronLib::itostring(nRet);
			}
		}		
		break;
	case IOTYPE_DIGITAL_OUTPUT:
		{
			if (m_uOutput > 0)
			{
				hr = m_pDriver->lpReadDigital(nRet, info.uAddress + m_uOutput, info.uIndex, info.uTimeout);
				
				if( SUCCEEDED(hr) )
				{
					strVal = CIronLib::itostring(nRet);
				}
			}
			else
			{
				if (m_mWriteValue.find(sIOIndex) != m_mWriteValue.end())
				{
					strVal = CIronLib::itostring(m_mWriteValue[sIOIndex].nVal);
				}
				else
				{
					strVal = "";
				}

				hr = S_OK;
			}
		}
		break;
	case IOTYPE_STRING_INPUT:
		{
			memset(strRet, 0, MAX_STRINGIO_SIZE);

			hr = m_pDriver->lpReadString(strRet, info.uAddress, info.uIndex, MAX_STRINGIO_SIZE, info.uTimeout);

			if( SUCCEEDED(hr) )
			{
				strVal = strRet;
			}
		}		
		break;
	case IOTYPE_STRING_OUTPUT:
		{
			if (m_uOutput > 0)
			{
				hr = m_pDriver->lpReadString(strRet, info.uAddress + m_uOutput, info.uIndex, MAX_STRINGIO_SIZE, info.uTimeout);
				
				if( SUCCEEDED(hr) )
				{
					strVal = strRet;
				}
			}
			else
			{
				if (m_mWriteValue.find(sIOIndex) != m_mWriteValue.end())
				{
					strVal = m_mWriteValue[sIOIndex].strVal;
					
				}
				else
				{
					strVal = "";
				}

				hr = S_OK;
			}
		}
		break;
	case IOTYPE_VARIABLE:
		{
			unsigned arraySize = 0;
			unsigned arrayIndex = 0;

			if (info.nArrySize > 0)
			{
				arraySize = info.nArrySize;
				arrayIndex = info.uAddress;
			}
			else
			{
				arraySize = 1;
				arrayIndex = 0;
			}

			if (info.strVarType == "BOOL")
			{
				bool* data = new bool[arraySize];

				hr = m_pDriver->lpReadVariable(info.strVarName.c_str(), arrayIndex, info.uIndex, sizeof(bool) * arraySize, data, info.uTimeout);

				if (data[arrayIndex])
					strVal = "TRUE";
				else
					strVal = "FALSE";

				delete[] data;
			}
			else if (info.strVarType == "BYTE")
			{
				BYTE* data = new BYTE[arraySize];

				hr = m_pDriver->lpReadVariable(info.strVarName.c_str(), arrayIndex, info.uIndex, sizeof(BYTE) * arraySize, data, info.uTimeout);
				
				strVal = CIronLib::itostring(data[arrayIndex]);

				delete[] data;
			}
			else if (info.strVarType == "LREAL")
			{
				double* data = new double[arraySize];

				hr = m_pDriver->lpReadVariable(info.strVarName.c_str(), arrayIndex, info.uIndex, sizeof(double) * arraySize, data, info.uTimeout);

				strVal = CIronLib::itostring(data[arrayIndex]);

				delete[] data;
			}
			else if (info.strVarType == "INT")
			{
				short* data = new short[arraySize];

				hr = m_pDriver->lpReadVariable(info.strVarName.c_str(), arrayIndex, info.uIndex, sizeof(short) * arraySize, data, info.uTimeout);

				strVal = CIronLib::itostring(data[arrayIndex]);

				delete[] data;
			}
			else if (info.strVarType == "UINT" || info.strVarType == "WORD")
			{
				unsigned short* data = new unsigned short[arraySize];

				hr = m_pDriver->lpReadVariable(info.strVarName.c_str(), arrayIndex, info.uIndex, sizeof(unsigned short) * arraySize, data, info.uTimeout);

				strVal = CIronLib::itostring(data[arrayIndex]);

				delete[] data;
			}
			else if (info.strVarType == "UDINT" || info.strVarType == "DWORD")
			{
				unsigned long* data = new unsigned long[arraySize];

				hr = m_pDriver->lpReadVariable(info.strVarName.c_str(), arrayIndex, info.uIndex, sizeof(unsigned long) * arraySize, data, info.uTimeout);

				strVal = CIronLib::itostring(data[arrayIndex]);

				delete[] data;
			}
			else
			{
				CIronLib::WriteDebugLog("Variable[%s] Type[%s] is not Support.(%s)", sIOIndex.c_str(), info.strVarType.c_str(), __FUNCTION__);
				return false;
			}
		}
		break;
	case IOTYPE_VIRTUAL:
		{
			if (m_mVirtual.find(sIOIndex) == m_mVirtual.end())
			{
				CIronLib::WriteDebugLog("Virtual[%s] is not find.(%s)", sIOIndex.c_str(), __FUNCTION__);
				return false;
			}

			strVal = m_mVirtual[sIOIndex];
			hr = S_OK;
		}
		break;
	default:
		return false;
	}

	if (FAILED(hr))
		return false;

	return true;
}

HRESULT CDevice::Read(string sIOIndex, string& strVal)
{
	HRESULT hr = E_FAIL;

	if (m_bThread && m_bReal)
	{
		CIronLock cLock(g_cLock);

		if (m_mReadVal.find(sIOIndex) != m_mReadVal.end())
		{
			strVal = m_mReadVal[sIOIndex];
			hr = S_OK;
		}
		else
		{
			CIronLib::WriteDebugLog("IO[%s] Read Delay.(%s)", sIOIndex.c_str(), __FUNCTION__);

			if (ReadIO(sIOIndex, strVal))
				hr = S_OK;
		}
	}
	else
	{
		if (ReadIO(sIOIndex, strVal))
			hr = S_OK;
	}

	return hr;
}

HRESULT CDevice::Write(string sIOIndex, string strVal, string& strResult)
{
	CIronLock cLock(*this);

	if (m_bStart == false)
	{
		CIronLib::WriteDebugLog("Don't Start Driver[%s].(%s)", m_strDriverName.c_str(), __FUNCTION__);
		strResult = "Fail";
		return E_FAIL;
	}

	if (m_mIOInfo.find(sIOIndex) == m_mIOInfo.end())
	{
		CIronLib::WriteDebugLog("Invalid IOName[%s].(%s)", sIOIndex.c_str(), __FUNCTION__);
		strResult = "Fail";
		return E_FAIL;
	}


	if (m_strDriverName == "Virtual")
	{
		m_mVirtual[sIOIndex] = strVal;
		strResult = "Success";
		CIronLib::WriteDebugLog("%s\t\t==>\t%s", sIOIndex.c_str(), strVal.c_str());
		return S_OK;
	}

	if (m_bReal == true)
	{
		if (m_pDriver == NULL)
		{
			CIronLib::WriteDebugLog("Driver[%s] was not loaded.(%s)", m_strDriverName.c_str(), __FUNCTION__);
			strResult = "Fail";
			return E_FAIL;
		}
	}
	else
	{
		if (sIOIndex.size() > 2 && sIOIndex.substr(0,2) == "VT")
			m_mVirtual[sIOIndex] = strVal;
		else
			m_mSimulValue[sIOIndex] = strVal;

		strResult = "Success";
		CIronLib::WriteDebugLog("%s\t\t==>\t%s", sIOIndex.c_str(), strVal.c_str());
		return S_OK;
	}

	HRESULT hr = E_FAIL;
	int nVal = 0;
	double dVal = 0;
	IO_INFO info = m_mIOInfo[sIOIndex];

	switch( info.nType )
	{
	case IOTYPE_ANALOG_INPUT:
	case IOTYPE_ANALOG_OUTPUT:
		{
			dVal = atof(strVal.c_str());
			m_mWriteValue[sIOIndex].dVal = dVal;
		}
		break;
	case IOTYPE_DIGITAL_INPUT:
	case IOTYPE_DIGITAL_OUTPUT:
		{
			nVal = atoi(strVal.c_str());
			m_mWriteValue[sIOIndex].nVal = nVal;
		}
		break;
	case IOTYPE_STRING_INPUT:
	case IOTYPE_STRING_OUTPUT:
	case IOTYPE_VARIABLE:
		{
			if (strVal.size() >= MAX_STRINGIO_SIZE)
			{
				CIronLib::WriteDebugLog("IO[%s] size is big.(%s)", sIOIndex.c_str(), __FUNCTION__);
				strResult = "Fail";
				return E_FAIL;
			}

			strcpy_s(m_mWriteValue[sIOIndex].strVal, MAX_STRINGIO_SIZE, strVal.c_str());
		}
		break;
	case IOTYPE_VIRTUAL:
		break;
	default:
		strResult = "Fail";
		return E_FAIL;
	}


	switch( info.nType )
	{
	case IOTYPE_ANALOG_OUTPUT:
		{
			dVal -= info.dOffset;
			
			if (info.dGain != 0)
				dVal /= info.dGain;

			hr = m_pDriver->lpWriteAnalog(dVal, info.uAddress, info.uIndex, info.uTimeout);
		}
		break;
	case IOTYPE_DIGITAL_OUTPUT:
		{
			hr = m_pDriver->lpWriteDigital(nVal, info.uAddress, info.uIndex, info.uTimeout);
		}
		break;
	case IOTYPE_STRING_OUTPUT:
		{
			hr = m_pDriver->lpWriteString(strVal.c_str(), info.uAddress, info.uIndex, info.uTimeout);
		}
		break;
	case IOTYPE_VARIABLE:
		{
			unsigned arraySize = 0;
			unsigned arrayIndex = 0;

			if (info.nArrySize > 0)
			{
				arraySize = info.nArrySize;
				arrayIndex = info.uAddress;
			}
			else
			{
				arraySize = 1;
				arrayIndex = 0;
			}

			if (info.strVarType == "BOOL")
			{
				bool* data = new bool[arraySize];
			
				if (arraySize > 0)
				{
					hr = m_pDriver->lpReadVariable(info.strVarName.c_str(), arrayIndex, info.uIndex, sizeof(bool) * arraySize, data, info.uTimeout);
				}

				if (strVal == "TRUE")
					data[arrayIndex] = true; 
				else
					data[arrayIndex] = false;
				
				hr = m_pDriver->lpWriteVariable(info.strVarName.c_str(), arrayIndex, info.uIndex, sizeof(bool) * arraySize, data, info.uTimeout);

				delete[] data;
			}
			else if (info.strVarType == "BYTE")
			{
				BYTE* data = new BYTE[arraySize];

				if (arraySize > 0)
				{
					hr = m_pDriver->lpReadVariable(info.strVarName.c_str(), arrayIndex, info.uIndex, sizeof(BYTE) * arraySize, data, info.uTimeout);
				}

				data[arrayIndex] = static_cast<BYTE>(atoi(strVal.c_str()));
				
				hr = m_pDriver->lpWriteVariable(info.strVarName.c_str(), arrayIndex, info.uIndex, sizeof(BYTE) * arraySize, data, info.uTimeout);

				delete[] data;
			}
			else if (info.strVarType == "LREAL")
			{
				double* data = new double[arraySize];
	
				if (arraySize > 0)
				{
					hr = m_pDriver->lpReadVariable(info.strVarName.c_str(), arrayIndex, info.uIndex, sizeof(double) * arraySize, data, info.uTimeout);
				}

				data[arrayIndex] = atof(strVal.c_str()); 
				hr = m_pDriver->lpWriteVariable(info.strVarName.c_str(), arrayIndex, info.uIndex, sizeof(double) * arraySize, data, info.uTimeout);

				delete[] data;
			}
			else if (info.strVarType == "INT")
			{
				short* data = new short[arraySize];

				if (arraySize > 0)
				{
					hr = m_pDriver->lpReadVariable(info.strVarName.c_str(), arrayIndex, info.uIndex, sizeof(short) * arraySize, data, info.uTimeout);
				}

				data[arrayIndex] = atoi(strVal.c_str());
				hr = m_pDriver->lpWriteVariable(info.strVarName.c_str(), arrayIndex, info.uIndex, sizeof(short) * arraySize, data, info.uTimeout);

				delete[] data;
			}
			else if (info.strVarType == "UINT" || info.strVarType == "WORD")
			{
				unsigned short* data = new unsigned short[arraySize];

				if (arraySize > 0)
				{
					hr = m_pDriver->lpReadVariable(info.strVarName.c_str(), arrayIndex, info.uIndex, sizeof(unsigned short) * arraySize, data, info.uTimeout);
				}

				data[arrayIndex] = atoi(strVal.c_str());
				hr = m_pDriver->lpWriteVariable(info.strVarName.c_str(), arrayIndex, info.uIndex, sizeof(unsigned short) * arraySize, data, info.uTimeout);

				delete[] data;
			}
			else if (info.strVarType == "UDINT" || info.strVarType == "DWORD")
			{
				unsigned long* data = new unsigned long[arraySize];

				if (arraySize > 0)
				{
					hr = m_pDriver->lpReadVariable(info.strVarName.c_str(), arrayIndex, info.uIndex, sizeof(unsigned long) * arraySize, data, info.uTimeout);
				}

				data[arrayIndex] = atoi(strVal.c_str());
				hr = m_pDriver->lpWriteVariable(info.strVarName.c_str(), arrayIndex, info.uIndex, sizeof(unsigned long) * arraySize, data, info.uTimeout);

				delete[] data;
			}
			else
			{
				CIronLib::WriteDebugLog("Variable Type[%s] is not Support.(%s)", info.strVarType.c_str(), __FUNCTION__);
				strResult = "Fail";
				return E_FAIL;
			}
		}
		break;
	case IOTYPE_VIRTUAL:
		{
			if (m_mVirtual.find(sIOIndex) == m_mVirtual.end())
			{
				CIronLib::WriteDebugLog("Virtual[%s] is not find.(%s)", sIOIndex.c_str(), __FUNCTION__);
				strResult = "Fail";
				return E_FAIL;
			}

			m_mVirtual[sIOIndex] = strVal;

			if (m_bThread)
			{
				CIronLock cLock(g_cLock);
				m_mReadVal[sIOIndex] = strVal;
			}

			hr = S_OK;
		}
		break;
	default:
		strResult = "Fail";
		return E_FAIL;
	}

	if (SUCCEEDED(hr))
	{
		strResult = "Success";
		CIronLib::WriteDebugLog("%s\t\t==>\t%s", sIOIndex.c_str(), strVal.c_str());
	}
	else
	{
		strResult = "Fail";
	}

	return hr;
}
