#pragma once

#include <string>
#include <map>

using namespace std;

const int MAX_STRINGIO_SIZE = 4095;

typedef DWORD (* LPINITDRIVER)(unsigned, unsigned, bool);
typedef DWORD (* LPUNINITDRIVER)(void);
typedef DWORD (* LPREADANALOG)(double&, unsigned, unsigned, int);
typedef DWORD (* LPREADDIGITAL)(int&, unsigned, unsigned, int);
typedef DWORD (* LPREADSTRING)(LPTSTR, unsigned, unsigned, unsigned, int);
typedef DWORD (* LPREADVARIABLE)(LPCTSTR, unsigned, unsigned, unsigned, void*, int);
typedef DWORD (* LPWRITEANALOG)(double, unsigned, unsigned, int);
typedef DWORD (* LPWRITEDIGITAL)(int, unsigned, unsigned, int);
typedef DWORD (* LPWRITESTRING)(LPCTSTR, unsigned, unsigned, int);
typedef DWORD (* LPWRITEVARIABLE)(LPCTSTR, unsigned, unsigned, unsigned, void*, int);

typedef struct _tagDriverFunction
{
	HINSTANCE hIODriver;
	LPINITDRIVER lpInitDriver;
	LPUNINITDRIVER lpUninitDriver;
	LPREADANALOG lpReadAnalog;
	LPREADDIGITAL lpReadDigital;
	LPREADSTRING lpReadString;
	LPREADVARIABLE lpReadVariable;
	LPWRITEANALOG lpWriteAnalog;
	LPWRITEDIGITAL lpWriteDigital;
	LPWRITESTRING lpWriteString;
	LPWRITEVARIABLE lpWriteVariable;
} DRIVER_FUNCTION;

typedef struct _tagIOInfo
{
	int nType;
	unsigned uAddress;
	unsigned uIndex;
	unsigned uTimeout;
	double dMin;
	double dMax;
	double dOffset;
	double dGain;
	string strVarName;
	string strVarType;
	unsigned nArrySize;
} IO_INFO;

typedef union _tagIOValue
{
	int nVal;
	double dVal;
	char strVal[MAX_STRINGIO_SIZE];
} IO_VALUE;

enum {
	IOTYPE_NONE,
	IOTYPE_ANALOG_INPUT,
	IOTYPE_ANALOG_OUTPUT,
	IOTYPE_DIGITAL_INPUT,
	IOTYPE_DIGITAL_OUTPUT,
	IOTYPE_STRING_INPUT,
	IOTYPE_STRING_OUTPUT,
	IOTYPE_VARIABLE,
	IOTYPE_VIRTUAL
};

class CDevice : 
	public CIronThread,
	public CIronSynchObject
{
public:
	CDevice(bool bReal, bool bThread, unsigned uPollingTime, unsigned uOutput, string strDevice, string strDriverFile, string strIOFile);
	~CDevice(void);

	void OnExecute(void);

	bool Start(unsigned uPort, int nTimeout, bool bLog);
	void Stop();

	bool GetVirtualIO();
	bool GetDriverTag();
	bool ConnectDriver();
	void DisconnectDriver();

	bool ReadIO(string sIOIndex, string& strVal);

	bool GetStart() { return m_bStart; }

	HRESULT Read(string sIOIndex, string& strVal);
	HRESULT Write(string sIOIndex, string strVal, string& strResult);

	map<string, IO_INFO>& GetIOInfo() { return m_mIOInfo; }

private:
	HANDLE m_hStopEvent;

	string m_strDriverFolder;

	bool m_bReal;
	bool m_bThread;
	unsigned m_uPollingTime;
	unsigned m_uOutput;
	string m_strDevice;
	string m_strDriverName;
	string m_strIOFile;

	bool m_bStart;

	DRIVER_FUNCTION* m_pDriver;

	map<string, IO_INFO> m_mIOInfo;
	map<string, IO_VALUE> m_mWriteValue;
	map<string, string> m_mSimulValue;
	map<string, string> m_mVirtual;
	map<string, string> m_mReadVal;
};
