#include "StdAfx.h"
#include "DeviceInfo.h"

CDeviceInfo::CDeviceInfo(void)
{
	m_strConfigFolder = GetApplicationFolder() + "Config\\";
}

CDeviceInfo::~CDeviceInfo(void)
{
	for (map<string, CDevice*>::iterator iter = m_mDriver.begin() ; iter != m_mDriver.end() ; ++iter)
	{
		if (iter->second != NULL)
		{
			iter->second->Stop();
			delete iter->second;
		}
	}
}

string CDeviceInfo::GetApplicationFolder()
{
	char strFilename[FILENAME_MAX];

	if( !CIronLib::ReadRegistryString(HKEY_LOCAL_MACHINE, "SOFTWARE\\IronTools\\Application\\", NULL, "C:\\IronTools\\", strFilename, FILENAME_MAX) )
	{
		CIronLib::WriteDebugLog("ReadRegistryString() return is false.(%s:%d)", __FILE__, __LINE__);
		return ".\\";
	}

	string strFolder = strFilename;
	
	if( strFolder.find_last_of('\\') != strFolder.size() - 1)
	{
		strFolder += "\\";
	}

	return strFolder;
}

bool CDeviceInfo::GetDeviceInfo(map<string, DEVICE_INFO>& devices)
{
	string strFilename = m_strConfigFolder + "DeviceInfo.ini";

	if (_access(strFilename.c_str(), 0) == -1)
	{
		CIronLib::WriteDebugLog("DeviceInfo File is nothing.(%s)", __FUNCTION__);
		return false;
	}

	char strTemp[INISECTION_SIZE];

	memset(strTemp, 0, INISECTION_SIZE);
	GetPrivateProfileSectionNames(strTemp, INISECTION_SIZE, strFilename.c_str());


	vector<string> vSection;
	int nSartIndex = 0;

	for(int i = 0 ;i < INISECTION_SIZE - 1 ; ++i )
	{
		if( strTemp[i] == '\0' && i > 0 )
		{
			vSection.push_back(strTemp + nSartIndex);
			nSartIndex = i + 1;
		}

		if( strTemp[i] == '\0' && strTemp[i+1] == '\0' )
			break;
	}

	for( vector<string>::iterator iter = vSection.begin() ; iter < vSection.end() ; ++iter)
	{
		DEVICE_INFO info;
		char strRet[INISECTION_SIZE];

		info.uPort = GetPrivateProfileInt(iter->c_str(), "Port", 0, strFilename.c_str());		
		info.bReal = GetPrivateProfileInt(iter->c_str(), "Real", 0, strFilename.c_str()) > 0 ? true : false;
		info.bThread = GetPrivateProfileInt(iter->c_str(), "Thread", 0, strFilename.c_str()) > 0 ? true :  false;
		info.uPollingTime = GetPrivateProfileInt(iter->c_str(), "PollingTime", 10, strFilename.c_str());
		info.uOutput = GetPrivateProfileInt(iter->c_str(), "Output", 0, strFilename.c_str());
		info.nTimeout = GetPrivateProfileInt(iter->c_str(), "TimeOut", 0, strFilename.c_str());
		info.bLog = GetPrivateProfileInt(iter->c_str(), "Log", 0, strFilename.c_str()) > 0 ? true : false;

		GetPrivateProfileString(iter->c_str(), "IOFile", "", strRet, INISECTION_SIZE, strFilename.c_str());
		info.strIOFile = strRet;

		GetPrivateProfileString(iter->c_str(), "Driver", "", strRet, INISECTION_SIZE, strFilename.c_str());
		info.strDriver = strRet;

		m_mDriverInfo[*iter] = info;

		devices[*iter] = info;
	}

	return true;
}

CDevice* CDeviceInfo::LoadDriver(string strDevice)
{
	if (m_mDriver.find(strDevice) != m_mDriver.end())
	{
		CIronLib::WriteDebugLog("Already Load Device[%s].(%s)", strDevice.c_str(), __FUNCTION__); 
		return NULL;
	}

	if (m_mDriverInfo.find(strDevice) == m_mDriverInfo.end())
	{
		CIronLib::WriteDebugLog("Invalid Device[%s].(%s)", strDevice.c_str(), __FUNCTION__); 
		return NULL;
	}

	CDevice* pDevice = new CDevice(m_mDriverInfo[strDevice].bReal, m_mDriverInfo[strDevice].bThread, m_mDriverInfo[strDevice].uPollingTime, m_mDriverInfo[strDevice].uOutput, strDevice, m_mDriverInfo[strDevice].strDriver, m_strConfigFolder + m_mDriverInfo[strDevice].strIOFile);

	m_mDriver[strDevice] = pDevice;

	CIronLib::WriteDebugLog("Device[%s] Driver Load.", strDevice.c_str());

	return pDevice;
}

void CDeviceInfo::UnloadDriver(string strDevice)
{
	if (m_mDriver.find(strDevice) == m_mDriver.end())
	{
		CIronLib::WriteDebugLog("Device[%s] was not loaded.(%s)", strDevice.c_str(), __FUNCTION__); 
		return;
	}

	if (m_mDriver[strDevice]->GetStart())
		m_mDriver[strDevice]->Stop();

	delete m_mDriver[strDevice];

	m_mDriver.erase(strDevice);

	CIronLib::WriteDebugLog("Device[%s] Driver Unloaded.", strDevice.c_str());
}