// DeviceManager.cpp : CDeviceManager Դϴ.

#include "stdafx.h"
#include "DeviceManager.h"


// CDeviceManager

STDMETHODIMP CDeviceManager::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IDeviceManager
	};

	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

HRESULT CDeviceManager::FinalConstruct()
{
	CoInitializeEx(NULL, COINIT_MULTITHREADED);

	CIronLock cLock(*this);

	HRESULT hr = S_OK;
	m_pVirtual = NULL;

	if (m_cDeviceInfo.GetDeviceInfo(m_devicses))
	{
		for (map<string, DEVICE_INFO>::iterator iter = m_devicses.begin() ; iter != m_devicses.end() ; ++iter)
		{
			CDevice* pDevice = m_cDeviceInfo.LoadDriver(iter->first);

			if (pDevice != NULL)
			{
				m_mDevice[iter->first] = pDevice;

				if (!pDevice->Start(iter->second.uPort, iter->second.nTimeout, iter->second.bLog))
				{
					CIronLib::WriteDebugLog("Device[%s] Start Fail.(%s)", iter->first.c_str(), __FUNCTION__);
					hr = S_FALSE;
					continue;
				}
			}
			else
			{
				CIronLib::WriteDebugLog("Device[%s] Load Fail.(%s)", iter->first.c_str(), __FUNCTION__);
				hr = S_FALSE;
				continue;
			}
		}

		m_pVirtual = new CDevice(true, false, 100, 0, "Virtual", "Virtual", "");

		m_pVirtual->Start(0, 0, 0);

		CIronLib::WriteDebugLog("IronSAN Construct");
	}
	else
	{
		CIronLib::WriteDebugLog("DeviceInfo Load Fail.(%s)", __FUNCTION__);
		hr = S_FALSE;
	}

	return hr;
}

void CDeviceManager::FinalRelease()
{
	CIronLock cLock(*this);

	for (map<string, DEVICE_INFO>::iterator iter = m_devicses.begin() ; iter != m_devicses.end() ; ++iter)
	{
		m_cDeviceInfo.UnloadDriver(iter->first);
	}

	if (m_pVirtual != NULL)
	{
		m_pVirtual->Stop();
		delete m_pVirtual;
		CIronLib::WriteDebugLog("Virtual DB Stop");
	}

	CoUninitialize();

	CIronLib::WriteDebugLog("IronSAN Release");
}

STDMETHODIMP CDeviceManager::Read(BSTR tag, BSTR* value)
{
	USES_CONVERSION;

	string strTag = OLE2A(tag);
	
	size_t begin = strTag.find('.');

	if (begin >= strTag.size())
	{
		CIronLib::WriteDebugLog("Invalid IOName[%s].(%s)", strTag.c_str(), __FUNCTION__);
		return S_FALSE;
	}

	size_t end = strTag.find('.', begin + 1);

	if (end >= strTag.size())
	{
		CIronLib::WriteDebugLog("Invalid IOName[%s].(%s)", strTag.c_str(), __FUNCTION__);
		return S_FALSE;
	}

	string strDevice = strTag.substr(begin + 1, end - begin - 1);


	HRESULT hr = E_FAIL;
	string strVal;

	if (strTag.size() > 2 && strTag.substr(0,2) == "VE")
	{
		if (m_pVirtual != NULL)
		{
			hr = m_pVirtual->Read(strTag, strVal);
		}
		else
		{
			CIronLib::WriteDebugLog("Can't find Virtual[%s].(%s)", strTag.c_str(), __FUNCTION__);
			return S_FALSE;
		}
	}
	else
	{
		if (m_mDevice.find(strDevice) == m_mDevice.end())
		{
			CIronLib::WriteDebugLog("Can't find Device[%s].(%s)", strTag.c_str(), __FUNCTION__);
			return S_FALSE;
		}

		hr = m_mDevice[strDevice]->Read(strTag, strVal);
	}

	if (FAILED(hr))
	{
		CIronLib::WriteDebugLog("IO[%s] Read Fail.(%s)", strTag.c_str(), __FUNCTION__);
		return S_FALSE;
	}

	/*if (strVal.size() == 0)
	{
		CIronLib::WriteDebugLog("IO[%s] Read Value Length is zero.(%s)", strTag.c_str(), __FUNCTION__);
		return S_FALSE;
	}*/

	*value = SysAllocString(A2W(strVal.c_str()));

	return S_OK;
}

STDMETHODIMP CDeviceManager::Write(BSTR tag, BSTR value, BSTR* result)
{
	USES_CONVERSION;

	string strTag = OLE2A(tag);
	string strVal = OLE2A(value);
	
	size_t begin = strTag.find('.');

	if (begin >= strTag.size())
	{
		CIronLib::WriteDebugLog("Invalid IOName[%s].(%s)", strTag.c_str(), __FUNCTION__);
		return S_FALSE;
	}

	size_t end = strTag.find('.', begin + 1);

	if (end >= strTag.size())
	{
		CIronLib::WriteDebugLog("Invalid IOName[%s].(%s)", strTag.c_str(), __FUNCTION__);
		return S_FALSE;
	}

	string strDevice = strTag.substr(begin + 1, end - begin - 1);


	HRESULT hr = E_FAIL;
	string strResult;

	if (strTag.size() > 2 && strTag.substr(0,2) == "VE")
	{
		if (m_pVirtual != NULL)
		{
			hr = m_pVirtual->Write(strTag, strVal, strResult);
		}
		else
		{
			CIronLib::WriteDebugLog("Can't find Virtual[%s].(%s)", strTag.c_str(), __FUNCTION__);
			return S_FALSE;
		}
	}
	else
	{
		if (m_mDevice.find(strDevice) == m_mDevice.end())
		{
			CIronLib::WriteDebugLog("Can't find Device[%s].(%s)", strTag.c_str(), __FUNCTION__);
			return S_FALSE;
		}

		hr = m_mDevice[strDevice]->Write(strTag, strVal, strResult);
	}

	*result = SysAllocString(A2W(strResult.c_str()));

	if (FAILED(hr))
	{
		CIronLib::WriteDebugLog("IO[%s] Write Fail.(%s)", strTag.c_str(), __FUNCTION__);
		return S_FALSE;
	}

	return S_OK;
}
