#pragma once

#include <windows.h>
#include <assert.h>

class CIronCriticalSection
{
public:
	inline CIronCriticalSection(void)
	{
		m_ulLocks  = 0;
		m_dwThread = -1;

		InitializeCriticalSection(&m_csLock);
	}

	inline ~CIronCriticalSection(void)
	{
		DeleteCriticalSection(&m_csLock);
	}

	inline void Lock()
	{
		EnterCriticalSection(&m_csLock);

		if (m_dwThread == -1)
		{
			m_dwThread = GetCurrentThreadId();
		}

		assert(m_dwThread == GetCurrentThreadId());

		m_ulLocks++;
	}

	inline void Unlock()
	{
		assert(m_dwThread == GetCurrentThreadId());
		assert(m_ulLocks > 0);

		m_ulLocks--;

		if (m_ulLocks == 0)
		{
			m_dwThread = -1;
		}

		LeaveCriticalSection(&m_csLock);
	}

	inline bool HasLock()
	{
		return (m_dwThread == GetCurrentThreadId());
	}

private: 
	CRITICAL_SECTION m_csLock;
	DWORD            m_dwThread;
	ULONG            m_ulLocks;
};


class CIronLock
{
public:
	inline CIronLock(const CIronCriticalSection& cLock)
    : m_pLock(NULL)
    {
        m_pLock = (CIronCriticalSection*)&cLock;
        m_pLock->Lock();
        m_uLocks = 1;
    }

    inline ~CIronLock(void)
    {
        while (m_uLocks > 0) Unlock();
    }

    inline void Unlock()
    {
        assert(m_uLocks > 0);

        m_uLocks--;

        if (m_uLocks == 0)
        {
            m_pLock->Unlock();
        }
    }

    inline void Lock()
    {
        if (m_uLocks == 0)
        {
            m_pLock->Lock();
        }

        m_uLocks++;
    }

private:
    UINT m_uLocks;
    CIronCriticalSection* m_pLock; 
};


class CIronSynchObject
{
public:
    operator CIronCriticalSection&() { return m_cLock; }
    operator const CIronCriticalSection&() const { return m_cLock; }

	bool HasLock() { return m_cLock.HasLock(); }

private:
    CIronCriticalSection m_cLock;
};