#include "StdAfx.h"
#include "IronFile.h"

CIronFile::CIronFile(void)
{
	m_hFile     = INVALID_HANDLE_VALUE;
	m_hMapping  = NULL;
	m_pView     = NULL;
	m_bReadOnly = false;
}

CIronFile::~CIronFile(void)
{
	Close();
}

bool CIronFile::Create(const string& cFileName)
{
	if (m_hFile != INVALID_HANDLE_VALUE)
	{
		return false;
	}

	m_hFile = CreateFile(
		cFileName.c_str(),
		GENERIC_READ | GENERIC_WRITE,
		NULL,
		NULL,
		CREATE_ALWAYS,
		FILE_ATTRIBUTE_NORMAL,
		NULL
	);

	if (m_hFile == INVALID_HANDLE_VALUE)
	{
		return false;
	}

	return true;
}

bool CIronFile::Open(const string& cFileName, bool bReadOnly)
{
	if (m_hFile != INVALID_HANDLE_VALUE)
	{
		return false;
	}

	m_bReadOnly = bReadOnly;

	m_hFile = CreateFile(
		cFileName.c_str(),
		(bReadOnly)?GENERIC_READ:(GENERIC_READ | GENERIC_WRITE),
		NULL,
		NULL,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL,
		NULL
	);

	if (m_hFile == INVALID_HANDLE_VALUE)
	{
		return false;
	}

	return true;
}

void CIronFile::Close()
{
	if (m_hFile != INVALID_HANDLE_VALUE)
	{
		UnmapViewOfFile(m_pView);
		CloseHandle(m_hMapping);
		CloseHandle(m_hFile);

		m_hFile    = INVALID_HANDLE_VALUE;
		m_hMapping = NULL;
		m_pView    = NULL;
	}
}

UINT CIronFile::Read(BYTE* pBuffer, UINT uSize)
{
	if (m_hFile == INVALID_HANDLE_VALUE)
	{
		return 0;
	}

	memset(pBuffer, 0, uSize);

	DWORD dwBytesRead = 0;

	BOOL bResult = ReadFile(
		m_hFile,
		pBuffer,
		uSize,
		&dwBytesRead,
		NULL
	);

	if (!bResult)
	{
		return 0;
	}

	return (UINT)dwBytesRead;
}

UINT CIronFile::Write(BYTE* pBuffer, UINT uSize)
{
	if (m_hFile == INVALID_HANDLE_VALUE)
	{
		return 0;
	}

	DWORD dwBytesWritten = 0;

	BOOL bResult = WriteFile(
		m_hFile,
		pBuffer,
		uSize,
		&dwBytesWritten,
		NULL
	);

	if (!bResult)
	{
		return 0;
	}

	return (UINT)dwBytesWritten;
}

UINT CIronFile::GetFileSize()
{
	return (UINT)::GetFileSize(m_hFile, NULL);
}

FILETIME CIronFile::GetLastModified()
{
	FILETIME ftLastModified = { 0, 0 };
	::GetFileTime(m_hFile, NULL, NULL, &ftLastModified);
	return ftLastModified;
}

BYTE* CIronFile::GetMemoryMapping()
{
	if (m_hMapping != NULL && m_pView != NULL)
	{
		return m_pView;
	}

	m_hMapping = CreateFileMapping(
		m_hFile,
		NULL,
		(m_bReadOnly)?PAGE_READONLY:PAGE_READWRITE,
		NULL,
		NULL,
		NULL
	);

	if (m_hMapping == NULL)
	{
		return NULL;
	}

	m_pView = (BYTE*)MapViewOfFile(
		m_hMapping,
		(m_bReadOnly)?FILE_MAP_READ:FILE_MAP_WRITE,
		0,
		0,
		0
	);

	if (m_pView == NULL)
	{
		return NULL;
	}

	return m_pView;
}