#include "StdAfx.h"
#include "IronLib.h"

CIronLib::CIronLib(void)
{
}

CIronLib::~CIronLib(void)
{
}

double CIronLib::GetTime(void)
{
	struct timeb t;
	ftime(&t);
	return (double)t.time + (double)t.millitm * (double)0.001;
}

bool CIronLib::CheckRetArray(int* array, int nArraySize)
{
	for (int i = 0 ; i < nArraySize ; ++i)
	{
		if (*(array + i) == FALSE)
		{
			return false;
		}
	}

	return true;
}

bool CIronLib::IsArrayInt(const int* array, int nSize, int nVal)
{
	for (int index = 0 ; index < nSize ; index++)
	{
		if (array[index] == nVal)
			return true;
	}

	return false;
}

void CIronLib::WriteDebugLog(const char *format, ...)
{
	va_list args;
	char buffer[4096] = {"IronTools>>"};

	va_start(args, format);
	vsprintf_s(buffer +11, 4084, format, args);
	va_end(args);

	OutputDebugString(buffer); 
}

string CIronLib::itostring(int nVal)
{
	char temp[1024];

	_itoa_s(nVal, temp, 1024, 10);

	return string(temp);
}

string CIronLib::itostring(unsigned long nVal)
{
	char temp[1024];

	_ultoa_s(nVal, temp, 1024, 10);

	return string(temp);
}

string CIronLib::itostring(double dVal)
{
	char temp[1024];

	sprintf_s(temp, 256, "%.10lf", dVal);

	string sRet = temp;

	int end = static_cast<int>(sRet.find("."));

	for (int index =  end; index < static_cast<int>(sRet.size()) ; index++)
	{
		if(sRet.substr(index, 1) != "0")
			end = index;
	}

	if (end == static_cast<int>(sRet.find(".")))
		return sRet.substr(0, end);

	return sRet.substr(0, end + 1);
}

vector<string> CIronLib::split(string &str, string sep)
{
	typedef string::size_type string_size;
	vector<string> ret;

	string_size i = 0, j = 0;
	string sItem;

	while (i < str.size())
	{
		j = str.find(sep, i);
		
		sItem = str.substr(i, j - i);

		if (!sItem.empty())
		{
			string_size begin, end;

			begin = sItem.find_first_not_of(" ");
			end = sItem.find_last_not_of(" ");

			if (begin < sItem.size() && end < sItem.size())
				sItem = sItem.substr(begin, end-begin+1);
			else
				sItem = "";

			ret.push_back(sItem);
		}

		if (j > str.size())
			i = str.size();
		else
			i = j + sep.size();
	}

	return ret;
}

void CIronLib::ReplaceHex(string &strSource)
{
	typedef string::size_type str_size;

	str_size po, i = 0;

	while ((po = strSource.find("\n", i)) < strSource.size())
	{
		strSource.replace(po, 1, "\\n");
		i = po;
	}

	i = 0;

	while ((po = strSource.find("\r", i)) < strSource.size())
	{
		strSource.replace(po, 1, "\\r");
		i = po;
	}

	unsigned char nChar= 0;

	for (unsigned int nPos = 0 ; nPos < strSource.size() ; ++nPos)
	{
		nChar = static_cast<unsigned char>(strSource[nPos]);

		if (nChar < 32 || nChar > 127)
		{
			string sHex = CIronLib::DecToHex(nChar);

			if (sHex.size() == 1)
			{
				sHex = "/x0" + sHex;
			}
			else
			{
				sHex = "/x" + sHex;
			}

			strSource.replace(nPos, 1, sHex.c_str());
			nPos += 3;
		}
	}

/*	i = 0;
	while ((po = strSource.find("'", i)) < strSource.size())
	{
		strSource.replace(po, 1, "\'");
		i = po;
	}*/
}

BYTE CIronLib::ASCIItoBinary(char ch)
{
	BYTE num = 0;
	
	switch (ch)
	{
	case 'A': 
	case 'a':
		num = 0xA;
		break;
	case 'B': 
	case 'b':
		num = 0xB;
		break;
	case 'C': 
	case 'c':
		num = 0xC;
		break;
	case 'D': 
	case 'd':
		num = 0xD;
		break;
	case 'E': 
	case 'e':
		num = 0xE;
		break;
	case 'F': 
	case 'f':
		num = 0xF;
		break;
	default :
		num = ch - 48;
		break;
	}

	return num;
}

BYTE CIronLib::AsciiToDec(BYTE byVal)
{
	switch (byVal){
	case 'A': 
	case 'a': return 10;
	case 'B': 
	case 'b': return 11;
	case 'C': 
	case 'c': return 12;
	case 'D': 
	case 'd': return 13;
	case 'E': 
	case 'e': return 14;
	case 'F': 
	case 'f': return 15;
	default : return byVal - 48 ;
	}
}

int CIronLib::HextoDec(string sVal, bool bMinus)
{
	int nRet = 0;
	BYTE num = 0;
	int multi = 0;
	bool minus = false;

	if (bMinus)
	{
		if (sVal.size() > 0)
		{
			num = ASCIItoBinary(sVal[0]);
			
			if ((num & (1 << 3)) > 0)
			{
				minus = true;
			}
		}
	}

	for (unsigned int index = 0; index < sVal.size() ; ++index)
	{
		num = ASCIItoBinary(sVal[index]);

		if (minus)
		{
			num = ~num;
		}

		num &= ~(0xf << 4);

		multi = static_cast<int>(sVal.size()) - (index + 1);

		nRet += num << multi * 4;
	}

	if (minus)
	{
		nRet += 1;
		nRet = ~nRet + 1;
	}

	return nRet;
}

string CIronLib::DecToHex(int nVal)
{
	string strRet;
	int quotient = nVal;
	int remainder = 0;

	while (quotient >= 16)
	{
		remainder = quotient % 16;
		quotient = quotient / 16;

		if (remainder < 10)
		{
			strRet = static_cast<char>(remainder + 48) + strRet;
		}
		else
		{
			strRet = static_cast<char>(remainder + 55) + strRet;
		}

		if (quotient < 16)
			break;
	}

	if (quotient < 10)
	{
		strRet = static_cast<char>(quotient + 48) + strRet;
	}
	else
	{
		strRet = static_cast<char>(quotient + 55) + strRet;
	}

	return strRet;
}

bool CIronLib::CreateDir(LPCSTR createDir)
{
	string dest(createDir);

	if (dest.size() == 0)
	{
		return false;
	}

	char buffer[1024];
	
	GetCurrentDirectory(1024, buffer);
	string currentDir(buffer);

	string::size_type find = 0;

	if (dest.substr(0, 2) == "..")
	{
		int nDirCnt1 = 0, nDirCnt2 = 0;

		while ((find = dest.find("..", find)) < dest.size())
		{
			++nDirCnt1;
			++find;
		}

		nDirCnt2 = nDirCnt1;

		string::size_type curDirsize = currentDir.size();
		string::size_type lastDir = curDirsize;

		while (nDirCnt1--)
		{
			lastDir = currentDir.find_last_of('\\', curDirsize - (curDirsize - lastDir));
			--lastDir;
		}

		if (lastDir < 1)
			return false;

		string::size_type startDir = 2 * nDirCnt2 + (nDirCnt2-1);
		dest = currentDir.substr(0, lastDir + 1) + dest.substr(startDir, dest.size() - startDir);
	}
	else if (dest.substr(0, 1) == ".")
	{
		dest = currentDir + dest.substr(1, dest.size());
	}

	if (dest.substr(dest.size() - 1, 1) == "\\")
		dest = dest.substr(0, dest.size() -1);

	find = 0;

	while (find < dest.size())
	{
		find = dest.find('\\', find);

		if (find <= dest.size())
		{
			CreateDirectory(dest.substr(0, find).c_str(), NULL);

			++find;
		}
	}

	if (CreateDirectory(dest.c_str() , NULL))
		return true;
	
	return false;
}

UINT CIronLib::ReadRegistryInt(HKEY hKey, LPCTSTR lpKey, LPCTSTR lpValue, INT nDefault)
{
	HKEY key;
	DWORD dwDisp;
	UINT nResult;
	DWORD dwSize;

	if (RegCreateKeyEx(hKey, lpKey, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_READ, NULL, &key, &dwDisp) != ERROR_SUCCESS)
	{
		return 0;
	}
	
	dwSize = sizeof(LONG);

	if (RegQueryValueEx(key, lpValue, 0, NULL, (LPBYTE)&nResult, &dwSize) != ERROR_SUCCESS)
	{
		nResult = nDefault;
	}

	RegCloseKey(key);

	return nResult;
}

bool CIronLib::ReadRegistryString(HKEY hKey, LPCTSTR lpKey, LPCTSTR lpValue, LPCTSTR lpDefault, LPTSTR lpRet, DWORD nSize)
{
	HKEY key;
	DWORD dwDisp;
	DWORD dwSize;

	if (RegCreateKeyEx(hKey, lpKey, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_READ, NULL, &key, &dwDisp) != ERROR_SUCCESS)
	{
		return 0;
	}
	
	dwSize = nSize;

	if (RegQueryValueEx(key, lpValue, 0, NULL, (LPBYTE)lpRet, &dwSize) != ERROR_SUCCESS)
	{
		strcpy_s(lpRet, nSize, lpDefault);
		return false;
	}

	RegCloseKey(key);

	return true;
}

bool CIronLib::WriteRegistryInt(HKEY hKey, LPCTSTR lpKey, LPCTSTR lpValue, UINT nData)
{
	HKEY key;
	DWORD dwDisp;

	if (RegCreateKeyEx(hKey, lpKey, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &key, &dwDisp) != ERROR_SUCCESS)
	{
		return false;
	}

	if (RegSetValueEx(key, lpValue, 0, REG_DWORD, (LPBYTE)&nData, sizeof(UINT)) != ERROR_SUCCESS)
	{
		return false;
	}

	RegCloseKey(key);

	return true;
}

bool CIronLib::WriteRegistryString(HKEY hKey, LPCTSTR lpKey, LPCTSTR lpValue, LPCTSTR lpData)
{
	HKEY key;
	DWORD dwDisp;

	if (RegCreateKeyEx(hKey, lpKey, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &key, &dwDisp) != ERROR_SUCCESS)
	{
		return false;
	}

	if (RegSetValueEx(key, lpValue, 0, REG_SZ, (LPBYTE)lpData, static_cast<DWORD>(strlen(lpData)+1)) != ERROR_SUCCESS)
	{
		return false;
	}

	RegCloseKey(key);

	return true;
}