#include "StdAfx.h"
#include "IronLog.h"

CIronLog::CIronLog(void) : m_fp(NULL)
{
}

CIronLog::CIronLog(string sDrivername, string sFoldername)
: m_sDrivername(sDrivername), m_sFoldername(sFoldername), m_fp(NULL)
{
	if (sFoldername.size() > 0)
		Open(sDrivername, sFoldername);
	else
		Open(sDrivername);
}

CIronLog::~CIronLog(void)
{
	if (m_fp != NULL)
	{
		fclose(m_fp);
		m_fp = NULL;
	}
}

void CIronLog::Open(string sDrivername, string sFoldername)
{
	m_sFoldername = sFoldername;

	if (_access(m_sFoldername.c_str(), 0) == -1)
	{
		bool bRet = CIronLib::CreateDir(m_sFoldername.c_str());

		if (!bRet)
			CIronLib::WriteDebugLog("%s Folder is not created.", m_sFoldername.c_str());
	}

	UpdateFileName();
	OpenFile();
}

void CIronLog::SetFoldername(string sFoldername)
{
	m_sFoldername = sFoldername;

	UpdateFileName();
	OpenFile();
}

void CIronLog::SetDrivername(string sComport)
{
	m_sDrivername = sComport;

	UpdateFileName();
	OpenFile();
}

void CIronLog::WriteLog(const char *format, ...)
{
	CIronLock cLock(*this);

	UpdateFileName();

	if (m_fp == NULL)
	{
		return;
	}

	va_list args;
	char buffer[4096];

	va_start(args, format);
	vsprintf_s(buffer, 4096, format, args);
	va_end(args);

	string sLog = buffer;

	char time[256];

	SYSTEMTIME st;

	GetLocalTime(&st);

	GetTimeFormat(LOCALE_SYSTEM_DEFAULT,0,&st,"HH:mm:ss", time, 256);

	CIronLib::ReplaceHex(sLog);
	fprintf(m_fp, "%s.%.3d\t%s\n", time,st.wMilliseconds, sLog.c_str());

	fflush(m_fp);	
}

void CIronLog::UpdateFileName(void)
{
	char filename[_MAX_FNAME];

	GetDateFormat(LOCALE_SYSTEM_DEFAULT, 0, NULL, "_yyMMdd", filename, _MAX_FNAME);

	m_sFilename = m_sFoldername + m_sDrivername + filename + ".log";

	WIN32_FIND_DATA FindFileData;
	HANDLE hFind;

	hFind = FindFirstFile(m_sFilename.c_str(), &FindFileData);

	if (hFind == INVALID_HANDLE_VALUE)
	{
		OpenFile();	
	}
	else
	{
		FindClose(hFind);
	}
}

void CIronLog::OpenFile(void)
{                                                                                                                                                        
	if (m_fp != NULL)
	{
		fclose(m_fp);
		m_fp = NULL;
	}

	m_fp = _fsopen(m_sFilename.c_str(), "a+", _SH_DENYWR);
}

