#include "StdAfx.h"
#include "Device.h"

CDevice::CDevice(void)
{
}

CDevice::CDevice(bool bLog, unsigned uPort)
: CSerial(bLog, uPort), m_nAddress(254)
{
}

CDevice::~CDevice(void)
{
}

int CDevice::CheckSum(string& strVal)
{
	int sum = 0;

	for (unsigned int i = 1 ; i < strVal.length() ; i++)
	{
		sum += strVal[i];
	}

	return sum % 256;
}

LONG CDevice::GetStatus(int& nVal)
{
	CIronLock cLock(*this);

	string strWrite = ">";

	strWrite += CIronLib::DecToHex(m_nAddress);
	strWrite += "F01";

	strWrite += CIronLib::DecToHex(CheckSum(strWrite));
	strWrite += "\r";

	if (!Write(strWrite))
	{
		return E_FAIL;
	}

	string strRead = ReadLine('\r');

	if (strRead.length() > 8 && strRead.substr(0, 1) == "A" && strRead.substr(strRead.length() - 1, 1) == "\r")
	{
		nVal = CIronLib::HextoDec(strRead.substr(1, 4));
		return S_OK;
	}

	return E_FAIL;
}

LONG CDevice::SetAddress(int nVal)
{
	CIronLock cLock(*this);

	if (nVal < 0 && nVal > 255)
	{
		return E_FAIL;
	}

	m_nAddress = nVal;

	return S_OK;
}

LONG CDevice::GetOxygen(double& dVal)
{
	CIronLock cLock(*this);

	string strWrite = ">";

	strWrite += CIronLib::DecToHex(m_nAddress);
	strWrite += "F69";

	strWrite += CIronLib::DecToHex(CheckSum(strWrite));
	strWrite += "\r";

	if (!Write(strWrite))
	{
		return E_FAIL;
	}

	string strRead = ReadLine('\r');
	
	size_t begin = strRead.find('A');
	size_t end = strRead.find("ppm");

	if (begin >= strRead.size() || end >= strRead.size() || strRead.find('\r') + 1 != strRead.size() )
	{
		CIronLib::WriteDebugLog("O2Analyzer>> Communication Error(%s:%d)", __FUNCTION__, __LINE__);
		return E_FAIL;
	}

	dVal = atof(strRead.substr(begin + 1, end - begin).c_str());
		
	return S_OK;
}
