#include "StdAfx.h"
#include "Device.h"

CDevice::CDevice(void)
{
}

CDevice::CDevice(bool bLog, unsigned uPort)
: CSerial(bLog, uPort)
{
}

CDevice::~CDevice(void)
{
}

bool CDevice::GetDeviceStatus()
{
	Write("R37\r\n");

	string strRead =ReadLine('\n');

	if( strRead.size() < 4)
	{
		FlushInput();
		CIronLib::WriteDebugLog("APC>> Communication Error(%s:%d)", __FUNCTION__, __LINE__);
		return false;
	}

	if( strRead.substr(0,1) == "M" )
	{
		m_nAccessMode = atoi(strRead.substr(1,1).c_str());
		m_nThrottleStatus = atoi(strRead.substr(3,1).c_str());

		if( m_nThrottleStatus == 0 )
			m_nCloseOpen = 1;
		else if( m_nThrottleStatus == 1)
			m_nCloseOpen = 0;
		else 
			m_nThrottleStatus = 2;
	}
	else
	{
		FlushInput();
		CIronLib::WriteDebugLog("APC>> Communication Error(%s:%d)", __FUNCTION__, __LINE__);
		return false;
	}

	return true;
}

LONG CDevice::GetControlMode(int& nVal)
{
	if( GetDeviceStatus() )
	{
		nVal = m_nAccessMode;
		return S_OK;
	}

	return E_FAIL;
}

LONG CDevice::GetCloseOpen(int& nVal)
{
	if( GetDeviceStatus() )
	{
		nVal = m_nCloseOpen;
		return S_OK;
	}

	return E_FAIL;
}

LONG CDevice::GetThrottleStatus(int& nVal)
{
	if( GetDeviceStatus() )
	{
		nVal = m_nThrottleStatus;
		return S_OK;
	}

	return E_FAIL;
}

LONG CDevice::GetPressure(double& dVal)
{
	Write("R5\r\n");

	string strRead = ReadLine('\n');

	if( strRead.empty() )
	{
		FlushInput();
		CIronLib::WriteDebugLog("APC>> Communication Error(%s:%d)", __FUNCTION__, __LINE__);
		return E_FAIL;
	}

	size_t begin = strRead.find("P");
	size_t end = strRead.find("\r\n");

	if( begin < strRead.size() && end < strRead.size() )
	{
		dVal = atof(strRead.substr(begin + 1, end - begin - 1).c_str()) + 760;
	}
	else
	{
		FlushInput();
		CIronLib::WriteDebugLog("APC>> Communication Error(%s:%d)", __FUNCTION__, __LINE__);
		return E_FAIL;
	}

	return S_OK;
}

LONG CDevice::GetPosition(double& dVal)
{
	Write("R6\r\n");

	string strRead = ReadLine('\n');

	if( strRead.empty() )
	{
		FlushInput();
		CIronLib::WriteDebugLog("APC>> Communication Error(%s:%d)", __FUNCTION__, __LINE__);
		return E_FAIL;
	}

	size_t begin = strRead.find("V");
	size_t end = strRead.find("\r\n");

	if( begin < strRead.size() && end < strRead.size() )
	{
		dVal = atof(strRead.substr(begin + 1, end - begin - 1).c_str());
	}
	else
	{
		FlushInput();
		CIronLib::WriteDebugLog("APC>> Communication Error(%s:%d)", __FUNCTION__, __LINE__);
		return E_FAIL;
	}

	return S_OK;
}

LONG CDevice::SetPressure(double dVal)
{
	string strWrite = "S1";

	strWrite += CIronLib::itostring(dVal - 760) + "\r\n";

	if( !Write(strWrite) )
	{
		FlushInput();
		CIronLib::WriteDebugLog("APC>> Communication Error(%s:%d)", __FUNCTION__, __LINE__);
		return E_FAIL;
	}

	if( !Write("T11\r\n") )
	{
		FlushInput();
		CIronLib::WriteDebugLog("APC>> Communication Error(%s:%d)", __FUNCTION__, __LINE__);
		return E_FAIL;
	}

	if( !Write("D1\r\n") )
	{
		FlushInput();
		CIronLib::WriteDebugLog("APC>> Communication Error(%s:%d)", __FUNCTION__, __LINE__);
		return E_FAIL;
	}

	return S_OK;
}

LONG CDevice::SetPosition(double dVal)
{
	string strWrite = "S2";

	strWrite += CIronLib::itostring(dVal) + "\r\n";

	if( !Write(strWrite) )
	{
		FlushInput();
		CIronLib::WriteDebugLog("APC>> Communication Error(%s:%d)", __FUNCTION__, __LINE__);
		return E_FAIL;
	}

	if( !Write("T20\r\n") )
	{
		FlushInput();
		CIronLib::WriteDebugLog("APC>> Communication Error(%s:%d)", __FUNCTION__, __LINE__);
		return E_FAIL;
	}

	if( !Write("D2\r\n") )
	{
		FlushInput();
		CIronLib::WriteDebugLog("APC>> Communication Error(%s:%d)", __FUNCTION__, __LINE__);
		return E_FAIL;
	}

	return S_OK;
}

LONG CDevice::SetControlMode(int nVal)
{
	return S_OK;
}

LONG CDevice::SetCloseOpen(int nVal)
{
	if( nVal == 0 )
	{
		if( !Write("C\r\n") )
		{
			FlushInput();
			CIronLib::WriteDebugLog("APC>> Communication Error(%s:%d)", __FUNCTION__, __LINE__);
			return E_FAIL;
		}
	}
	else if( nVal == 1 )
	{
		if( !Write("O\r\n") )
		{
			FlushInput();
			CIronLib::WriteDebugLog("APC>> Communication Error(%s:%d)", __FUNCTION__, __LINE__);
			return E_FAIL;
		}
	}

	return S_OK;
}
/*
LONG CDevice::WriteSelectSetPoint(int nVal)
{
	if (nVal > 0 && nVal <7)
	{
		string strWrite = "D" + CIronLib::itostring(nVal);

		if (Write(strWrite))
		{
			return S_OK;
		}
	}

	return E_FAIL;
}

LONG CDevice::WriteSensorRange(int nVal)
{
	if (nVal >= 0 && nVal <= 19)
	{
		string strWrite = "E" + CIronLib::itostring(nVal);

		if (Write(strWrite))
		{
			return S_OK;
		}
	}

	return E_FAIL;
}

LONG CDevice::WritePressureUnit(int nVal)
{
	if (nVal >= 0 && nVal <= 7)
	{
		string strWrite = "F" + CIronLib::itostring(nVal);

		if (Write(strWrite))
		{
			return S_OK;
		}
	}

	return E_FAIL;
}

LONG CDevice::WriteSensorVolRange(int nVal)
{
	if (nVal >= 0 && nVal <= 2)
	{
		string strWrite = "G" + CIronLib::itostring(nVal);

		if (Write(strWrite))
		{
			return S_OK;
		}
	}

	return E_FAIL;
}

LONG CDevice::WriteSetPointA(double dVal)
{
	string strWrite = "S1" + CIronLib::itostring(dVal);
	
	if( Write(strWrite) )
	{
		return S_OK;
	}

	return E_FAIL;
}

LONG CDevice::WriteSetPointB(double dVal)
{
	string strWrite = "S2" + CIronLib::itostring(dVal);
	
	if( Write(strWrite) )
	{
		return S_OK;
	}

	return E_FAIL;
}

LONG CDevice::WriteSetPointC(double dVal)
{
	string strWrite = "S3" + CIronLib::itostring(dVal);
	
	if( Write(strWrite) )
	{
		return S_OK;
	}

	return E_FAIL;
}

LONG CDevice::WriteSetPointD(double dVal)
{
	string strWrite = "S4" + CIronLib::itostring(dVal);
	
	if( Write(strWrite) )
	{
		return S_OK;
	}

	return E_FAIL;
}

LONG CDevice::WriteSetPointE(double dVal)
{
	string strWrite = "S5" + CIronLib::itostring(dVal);
	
	if( Write(strWrite) )
	{
		return S_OK;
	}

	return E_FAIL;
}

LONG CDevice::WriteAnalogSetPoint(double dVal)
{
	string strWrite = "S6" + CIronLib::itostring(dVal);
	
	if( Write(strWrite) )
	{
		return S_OK;
	}

	return E_FAIL;
}*/
