// MKS.cpp : NymDriver  α׷   մϴ.
//

#include "stdafx.h"
#include "MKS.h"
#include <DriverBase.h>
#include "Device.h"


#ifdef _MANAGED
#pragma managed(push, off)
#endif

BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
    return TRUE;
}

#ifdef _MANAGED
#pragma managed(pop)
#endif

CDriverBase* g_pDriver = NULL;
double g_nOldTimeout = 0;
double g_nOldWriteTimeout = 0;

void SetTimeout(int nTimeout)
{
	if (g_pDriver == NULL)
		return;

	//Read Timeout Setting
	g_nOldTimeout = g_pDriver->GetTimeout();
			
	if (g_nOldTimeout != nTimeout)
	{
		g_pDriver->SetTimeout(nTimeout);
	}

	//Write Timeout Setting
	int nWriteTimeout = 0;

	if (nTimeout > 0)
		nWriteTimeout = nTimeout;

	g_nOldWriteTimeout = g_pDriver->GetWriteTimeout();
		
	if (g_nOldWriteTimeout != nWriteTimeout)
	{
		g_pDriver->SetWriteTimeout(nWriteTimeout);
	}
}

void ResetTimeout(int nTimeout)
{
	if (g_pDriver == NULL)
		return;

	//ReadTimeout Setting
	if (g_nOldTimeout != nTimeout)
	{
		g_pDriver->SetTimeout(g_nOldTimeout);
	}

	int nWriteTimeout = 0;

	if (nTimeout > 0)
		nWriteTimeout = nTimeout;

	//WtieTimeout Setting
	if (g_nOldWriteTimeout != nWriteTimeout)
	{
		g_pDriver->SetWriteTimeout(g_nOldWriteTimeout);
	}
}

MKS_API LONG InitDriver(unsigned uPort, int nTimeout, bool bLog)
{
	if (g_pDriver == NULL)
		g_pDriver = new CDevice(bLog, uPort);
	else
		return E_FAIL;

	return S_OK;
}

MKS_API LONG UninitDriver(void)
{
	if (g_pDriver != NULL)
		delete g_pDriver;
	else
		return E_FAIL;

	return S_OK;
}

MKS_API LONG ReadAnalog(double &dInput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);
	
	switch (uAddr)
	{
	case AI_Pressure:
		hr = pDevice->GetPressure(dInput);
		break;
	case AI_Position:
		hr = pDevice->GetPosition(dInput);
		break;
	default:
		CIronLib::WriteDebugLog("MKS>>Invalid AI Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}

	ResetTimeout(nTimeout);
	
	return hr;
}

MKS_API LONG ReadDigital(int &nInput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);
	
	switch (uAddr)
	{
	case DI_ControlMode:
		hr = pDevice->GetControlMode(nInput);
		break;
	case DI_CloseOpen:
		hr = pDevice->GetCloseOpen(nInput);
		break;
	case DI_Status:
		hr = pDevice->GetThrottleStatus(nInput);
		break;
	default:
		CIronLib::WriteDebugLog("MKS>>Invalid DI Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}
	
	ResetTimeout(nTimeout);
	
	return hr;
}

MKS_API LONG ReadString(LPTSTR strInput, unsigned uAddr, unsigned uIndex, unsigned uSize, int nTimeout)
{
	HRESULT hr = E_FAIL;
	
	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);

	switch (uAddr)
	{
	case 0:
/*		hr = pDevice->GetSI(strInput);
		break;*/
	default:
		CIronLib::WriteDebugLog("MKS>>Invalid SI Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}
	
	ResetTimeout(nTimeout);

	return hr;
}

MKS_API LONG ReadVariable(LPCTSTR strName, unsigned uAddr, unsigned uIndex, unsigned uSize, void* pData, int nTimeout)
{
		return E_FAIL;
}

MKS_API LONG WriteAnalog(double dOutput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);

	switch (uAddr)
	{
	case AI_Pressure:
		hr = pDevice->SetPressure(dOutput);
		break;
	case AI_Position:
		hr = pDevice->SetPosition(dOutput);
		break;
	default:
		CIronLib::WriteDebugLog("MKS>>Invalid AO Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}

	ResetTimeout(nTimeout);
	
	return hr;
}

MKS_API LONG WriteDigital(int nOutput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;
	
	SetTimeout(nTimeout);
	
	switch (uAddr)
	{
	case DO_ControlMode:
		hr = pDevice->SetControlMode(nOutput);
		break;
	case DO_CloseOpen:
		hr = pDevice->SetCloseOpen(nOutput);
		break;
	default:
		CIronLib::WriteDebugLog("MKS>>Invalid DI Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}

	ResetTimeout(nTimeout);
	
	return hr;
}

MKS_API LONG WriteString(LPCTSTR strOutput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);
	
	switch (uAddr)
	{
	case 0:
/*		hr = pDevice->SetSO(strOutput);
		break;*/
	default:
		CIronLib::WriteDebugLog("MKS>>Invalid SO Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}
	
	ResetTimeout(nTimeout);
	
	return hr;
}

MKS_API LONG WriteVariable(LPCTSTR strName, unsigned uAddr, unsigned uIndex, unsigned uSize, void* pData, int nTimeout)
{
		return E_FAIL;
}