#include "StdAfx.h"
#include "Device.h"

CDevice::CDevice(void)
{
}

CDevice::CDevice(bool bLog, unsigned uPort)
: CSerial(bLog, uPort, 56000), m_strExtractMode("0"), m_nError(0), m_nFlag9(0), m_nFlag10(0),
m_nOriginComplete(0), m_nStopComplete(0), m_nResetComplete(0), m_nGetReadyComplete(0), m_nPutReadyComplete(0),
m_nGetUpDownComplete(0), m_nPutUpDownComplete(0), m_nGetComplete(0), m_nPutComplete(0), m_nGripComplete(0), 
m_nRetractComplete(0), m_nExtractComplete(0), m_nHomeComplete(0), m_nMapComplete(0), m_nMapDataComplete(0)
{
	m_hStopEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

	assert(m_hStopEvent);

	BeginThread();
}

CDevice::~CDevice(void)
{
	SetEvent(m_hStopEvent);
	WaitFor();

	CloseHandle(m_hStopEvent);
}

void CDevice::OnExecute(void)
{
	string strRead = "";

	while (WaitForSingleObject(m_hStopEvent, 10))
	{
		CIronLock cLock(*this);

		if (InWaiting() > 0)
		{
			strRead += ReadLine();

			if (strRead.rfind("\r\n") == strRead.length() - 2)
			{
				CheckRecieveData(strRead);

				strRead = "";
			}
		}
	}
}

void CDevice::CheckRecieveData(string& strRead)
{
	if (strRead == "ORG\r\n")
	{
		m_nOriginComplete = 1;
	}
	else if (strRead == "DRT\r\n")
	{
		m_nResetComplete = 1;
	}
	else if (strRead == "GRDY\r\n")
	{
		m_nGetReadyComplete = 1;
	}
	else if (strRead == "PRDY\r\n")
	{
		m_nPutReadyComplete = 1;
	}
	else if (strRead == "GZDN\r\n" || strRead == "GZUP\r\n")
	{
		m_nGetUpDownComplete = 1;
	}
	else if (strRead == "PZDN\r\n" || strRead == "PZUP\r\n")
	{
		m_nPutUpDownComplete = 1;
	}
	else if (strRead == "GET\r\n")
	{
		m_nGetComplete = 1;
	}
	else if (strRead == "PUT\r\n")
	{
		m_nPutComplete = 1;
	}
	else if (strRead == "OUT\r\n")
	{
		m_nGripComplete = 1;
	}
	else if (strRead == "RETA\r\n")
	{
		m_nRetractComplete = 1;
	}
	else if (strRead == "EXTA\r\n")
	{
		m_nExtractComplete = 1;
	}
	else if (strRead == "HOME\r\n")
	{
		m_nHomeComplete = 1;
	}
	else if (strRead == "MAP\r\n")
	{
		m_nMapComplete = 1;
	}
	else if (strRead.length() == 56 && strRead.substr(0, 3) == "MLD")
	{
		m_strMapData = strRead.substr(14, 25);
	}
	else if (strRead == "MLD EOD\r\n")
	{
		m_nMapDataComplete = 1;
	}
	else if (strRead.find("ERR") < strRead.length() && strRead.length() == 9)
	{
		m_nError = atoi(strRead.substr(4, 3).c_str());
	}
	else if (strRead.find("VAR F09") < strRead.length() && strRead.length() == 11)
	{
		m_nFlag9 = atoi(strRead.substr(8, 1).c_str());
	}
	else if (strRead.find("VAR F10") < strRead.length() && strRead.length() == 11)
	{
		m_nFlag10 = atoi(strRead.substr(8, 1).c_str());
	}
}

string CDevice::MakeCheckSum(string strData)
{
	unsigned char sum = 0;

	for (size_t i = 0; i < strData.length() ; i++)
	{
		sum -= strData[i];
	}

	return CIronLib::DecToHex(sum);
}

LONG CDevice::GetError(int& nVal)
{
	CIronLock cLock(*this);

	Write("ERR\r\n");

	nVal = m_nError;
	return S_OK;
}

LONG CDevice::GetFlag9(int& nVal)
{
	CIronLock cLock(*this);

	Write("VAR F09\r\n");

	nVal = m_nFlag9;
	return S_OK;
}

LONG CDevice::GetFlag10(int& nVal)
{
	CIronLock cLock(*this);

	Write("VAR F10\r\n");

	nVal = m_nFlag10;
	return S_OK;
}

LONG CDevice::GetOriginComplete(int& nVal)
{
	nVal = m_nOriginComplete;
	return S_OK;
}

LONG CDevice::GetStopComplete(int& nVal)
{
	nVal = m_nStopComplete;
	return S_OK;
}

LONG CDevice::GetResetComplete(int& nVal)
{
	nVal = m_nResetComplete;
	return S_OK;
}

LONG CDevice::GetGetReadyComplete(int& nVal)
{
	nVal = m_nGetReadyComplete;
	return S_OK;
}

LONG CDevice::GetPutReadyComplete(int& nVal)
{
	nVal = m_nPutReadyComplete;
	return S_OK;
}

LONG CDevice::GetGetUpDownComplete(int& nVal)
{
	nVal = m_nGetUpDownComplete;
	return S_OK;
}

LONG CDevice::GetPutUpDownComplete(int& nVal)
{
	nVal = m_nPutUpDownComplete;
	return S_OK;
}

LONG CDevice::GetGetComplete(int& nVal)
{
	nVal = m_nGetComplete;
	return S_OK;
}

LONG CDevice::GetPutComplete(int& nVal)
{
	nVal = m_nPutComplete;
	return S_OK;
}

LONG CDevice::GetGripComplete(int& nVal)
{
	nVal = m_nGripComplete;
	return S_OK;
}

LONG CDevice::GetRetractComplete(int& nVal)
{
	nVal = m_nRetractComplete;
	return S_OK;
}

LONG CDevice::GetExtractComplete(int& nVal)
{
	nVal = m_nExtractComplete;
	return S_OK;
}

LONG CDevice::GetHomeComplete(int& nVal)
{
	nVal = m_nHomeComplete;
	return S_OK;
}

LONG CDevice::GetMapDataComplete(int& nVal)
{
	nVal = m_nMapDataComplete;
	return S_OK;
}

LONG CDevice::GetMapComplete(int& nVal)
{
	nVal = m_nMapComplete;
	return S_OK;
}

LONG CDevice::GetMapData(string& strVal)
{
	strVal = m_strMapData;
	return S_OK;
}

LONG CDevice::SetStage(int nVal)
{
	CIronLock cLock(*this);

	if (nVal < 0 && nVal > MAX_STAGE)
	{
		return E_FAIL;
	}

	m_strStage = CIronLib::itostring(nVal);
	return S_OK;
}

LONG CDevice::SetSlot(int nVal)
{
	CIronLock cLock(*this);

	if (nVal < 0 && nVal > MAX_SLOT)
	{
		return E_FAIL;
	}

	m_strSlot = CIronLib::itostring(nVal);
	return S_OK;
}

LONG CDevice::SetArm(int nVal)
{
	CIronLock cLock(*this);

	if (nVal < 0 && nVal > 2)
	{
		return E_FAIL;
	}

	m_strArm = CIronLib::itostring(nVal);
	return S_OK;
}

LONG CDevice::SetExtractMode(int nVal)
{
	CIronLock cLock(*this);

	if (nVal < 0 && nVal > 2)
	{
		return E_FAIL;
	}

	m_strExtractMode = CIronLib::itostring(nVal);
	return S_OK;
}

LONG CDevice::SetOrigin(int nVal)
{
	CIronLock cLock(*this);

	m_nOriginComplete = 0;

	if (nVal == 1)
	{
		Write("ORG\r\n");
	}

	return S_OK;
}

LONG CDevice::SetStop(int nVal)
{
	CIronLock cLock(*this);

	m_nStopComplete = 0;

	return S_OK;
}

LONG CDevice::SetReset(int nVal)
{
	CIronLock cLock(*this);

	m_nResetComplete = 0;

	Write("DRT\r\n");

	return S_OK;
}

LONG CDevice::SetGetReady(int nVal)
{
	CIronLock cLock(*this);

	m_nGetReadyComplete = 0;

	if (nVal == 1)
	{
		string strWrite = "GRDY ";
		strWrite += m_strStage + ",";
		strWrite += m_strSlot + ",";
		strWrite += m_strArm + "\r\n";

		Write(strWrite);
	}

	return S_OK;
}

LONG CDevice::SetPutReady(int nVal)
{
	CIronLock cLock(*this);

	m_nPutReadyComplete = 0;

	if (nVal == 1)
	{
		string strWrite = "PRDY ";
		strWrite += m_strStage + ",";
		strWrite += m_strSlot + ",";
		strWrite += m_strArm + "\r\n";

		Write(strWrite);
	}

	return S_OK;
}

LONG CDevice::SetGetUpDown(int nVal)
{
	CIronLock cLock(*this);

	m_nGetUpDownComplete = 0;

	if (nVal == 0)
	{
		Write("GZDN\r\n");
	}
	else if (nVal == 1)
	{
		Write("GZUP\r\n");
	}
	else
	{
		return E_FAIL;
	}

	return S_OK;
}

LONG CDevice::SetPutUpDown(int nVal)
{
	CIronLock cLock(*this);

	m_nPutUpDownComplete = 0;

	if (nVal == 0)
	{
		Write("PZDN\r\n");
	}
	else if (nVal == 1)
	{
		Write("PZUP\r\n");
	}
	else
	{
		return E_FAIL;
	}

	return S_OK;
}

LONG CDevice::SetGet(int nVal)
{
	CIronLock cLock(*this);

	m_nGetComplete = 0;

	if (nVal == 1)
	{
		string strWrite = "GET ";
		strWrite += m_strStage + ",";
		strWrite += m_strSlot + ",";
		strWrite += m_strArm + "\r\n";

		Write(strWrite);
	}

	return S_OK;
}

LONG CDevice::SetPut(int nVal)
{
	CIronLock cLock(*this);

	m_nPutComplete = 0;

	if (nVal == 1)
	{
		string strWrite = "PUT ";
		strWrite += m_strStage + ",";
		strWrite += m_strSlot + ",";
		strWrite += m_strArm + "\r\n";

		Write(strWrite);
	}

	return S_OK;
}

LONG CDevice::SetGrip(int nVal)
{
	CIronLock cLock(*this);

	m_nGripComplete = 0;

	if (!(nVal == 0 || nVal == 1))
	{
		return E_FAIL;
	}

	string strWrite = "OUT ";
	strWrite += m_strArm + "," + CIronLib::itostring(nVal) + "\r\n";

	Write(strWrite);

	return S_OK;
}

LONG CDevice::SetRetract(int nVal)
{
	CIronLock cLock(*this);

	m_nRetractComplete = 0;

	if (nVal == 1)
	{
		string strWrite = "RETA ";
		strWrite += m_strArm + "\r\n";

		Write(strWrite);
	}

	return S_OK;
}

LONG CDevice::SetExtract(int nVal)
{
	CIronLock cLock(*this);

	m_nExtractComplete = 0;

	if (nVal == 1)
	{
		string strWrite = "EXTA ";
		strWrite += m_strArm + ",";
		strWrite += m_strExtractMode + "\r\n";

		Write(strWrite);
	}

	return S_OK;
}

LONG CDevice::SetHome(int nVal)
{
	CIronLock cLock(*this);

	m_nHomeComplete = 0;

	if (nVal == 1)
	{
		Write("HOME\r\n");
	}

	return S_OK;
}

LONG CDevice::SetMap(int nVal)
{
	CIronLock cLock(*this);

	m_nMapComplete = 0;

	if (nVal == 1)
	{
		string strWrite = "MAP ";
		strWrite += m_strStage + "\r\n";

		Write(strWrite);
	}

	return S_OK;
}

LONG CDevice::SetMapData(int nVal)
{
	CIronLock cLock(*this);

	m_nMapDataComplete = 0;

	if (nVal == 1)
	{
		string strWrite = "MLD ";
		strWrite += m_strStage + "\r\n";

		Write(strWrite);
	}

	return S_OK;
}
