// RND.cpp : NymDriver  α׷   մϴ.
//

#include "stdafx.h"
#include "RND.h"
#include <DriverBase.h>
#include "Device.h"


#ifdef _MANAGED
#pragma managed(push, off)
#endif

BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
    return TRUE;
}

#ifdef _MANAGED
#pragma managed(pop)
#endif

CDriverBase* g_pDriver = NULL;
double g_nOldTimeout = 0;
double g_nOldWriteTimeout = 0;

void SetTimeout(int nTimeout)
{
	if (g_pDriver == NULL)
		return;

	//Read Timeout Setting
	g_nOldTimeout = g_pDriver->GetTimeout();
			
	if (g_nOldTimeout != nTimeout)
	{
		g_pDriver->SetTimeout(nTimeout);
	}

	//Write Timeout Setting
	int nWriteTimeout = 0;

	if (nTimeout > 0)
		nWriteTimeout = nTimeout;

	g_nOldWriteTimeout = g_pDriver->GetWriteTimeout();
		
	if (g_nOldWriteTimeout != nWriteTimeout)
	{
		g_pDriver->SetWriteTimeout(nWriteTimeout);
	}
}

void ResetTimeout(int nTimeout)
{
	if (g_pDriver == NULL)
		return;

	//ReadTimeout Setting
	if (g_nOldTimeout != nTimeout)
	{
		g_pDriver->SetTimeout(g_nOldTimeout);
	}

	int nWriteTimeout = 0;

	if (nTimeout > 0)
		nWriteTimeout = nTimeout;

	//WtieTimeout Setting
	if (g_nOldWriteTimeout != nWriteTimeout)
	{
		g_pDriver->SetWriteTimeout(g_nOldWriteTimeout);
	}
}

RND_API LONG InitDriver(unsigned uPort, int nTimeout, bool bLog)
{
	if (g_pDriver == NULL)
		g_pDriver = new CDevice(bLog, uPort);
	else
		return E_FAIL;

	return S_OK;
}

RND_API LONG UninitDriver(void)
{
	if (g_pDriver != NULL)
		delete g_pDriver;
	else
		return E_FAIL;

	return S_OK;
}

RND_API LONG ReadAnalog(double &dInput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);
	
	switch (uAddr)
	{
	case 0:
/*		hr = pDevice->GetAI(dInput);
		break;*/
	default:
		CIronLib::WriteDebugLog("RND>>Invalid AI Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}

	ResetTimeout(nTimeout);
	
	return hr;
}

RND_API LONG ReadDigital(int &nInput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);
	
	switch (uAddr)
	{
	case DI_ERROR:
		hr = pDevice->GetError(nInput);
		break;
	case DI_FLAG9:
		hr = pDevice->GetFlag9(nInput);
		break;
	case DI_FLAG10:
		hr = pDevice->GetFlag10(nInput);
		break;
	case DI_ORIGIN_COMPLETE:
		hr = pDevice->GetOriginComplete(nInput);
		break;
	case DI_STOP_COMPLETE:
		hr = pDevice->GetStopComplete(nInput);
		break;
	case DI_RESET_COMPLETE:
		hr = pDevice->GetResetComplete(nInput);
		break;
	case DI_GETREADY_COMPLETE:
		hr = pDevice->GetGetReadyComplete(nInput);
		break;
	case DI_PUTREADY_COMPLETE:
		hr = pDevice->GetPutReadyComplete(nInput);
		break;
	case DI_GETUPDOWN_COMPLETE:
		hr = pDevice->GetGetUpDownComplete(nInput);
		break;
	case DI_PUTUPDOWN_COMPLETE:
		hr = pDevice->GetPutUpDownComplete(nInput);
		break;
	case DI_GET_COMPLETE:
		hr = pDevice->GetGetComplete(nInput);
		break;
	case DI_PUT_COMPLETE:
		hr = pDevice->GetPutComplete(nInput);
		break;
	case DI_GRIP_COMPLETE:
		hr = pDevice->GetGripComplete(nInput);
		break;
	case DI_RETRACT_COMPLETE:
		hr = pDevice->GetRetractComplete(nInput);
		break;
	case DI_EXTRACT_COMPLETE:
		hr = pDevice->GetExtractComplete(nInput);
		break;
	case DI_HOME_COMPLETE:
		hr = pDevice->GetHomeComplete(nInput);
		break;
	case DI_MAP_COMPLETE:
		hr = pDevice->GetMapComplete(nInput);
		break;
	case DI_MAPDATA_COMPLETE:
		hr = pDevice->GetMapDataComplete(nInput);
		break;
	default:
		CIronLib::WriteDebugLog("RND>>Invalid DI Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}
	
	ResetTimeout(nTimeout);
	
	return hr;
}

RND_API LONG ReadString(LPTSTR strInput, unsigned uAddr, unsigned uIndex, unsigned uSize, int nTimeout)
{
	HRESULT hr = E_FAIL;
	
	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);

	string strRead;

	switch (uAddr)
	{
	case SI_MAPDATA:
		hr = pDevice->GetMapData(strRead);

		strcpy_s(strInput, uSize, strRead.c_str());
		break;
	default:
		CIronLib::WriteDebugLog("RND>>Invalid SI Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}
	
	ResetTimeout(nTimeout);

	return hr;
}

RND_API LONG ReadVariable(LPCTSTR strName, unsigned uAddr, unsigned uIndex, unsigned uSize, void* pData, int nTimeout)
{
		return E_FAIL;
}

RND_API LONG WriteAnalog(double dOutput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);

	switch (uAddr)
	{
	case 0:
/*		hr = pDevice->SetAO(dOutput);
		break;*/
	default:
		CIronLib::WriteDebugLog("RND>>Invalid AO Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}

	ResetTimeout(nTimeout);
	
	return hr;
}

RND_API LONG WriteDigital(int nOutput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;
	
	SetTimeout(nTimeout);
	
	switch (uAddr)
	{
	case DO_STAGE:
		hr = pDevice->SetStage(nOutput);
		break;
	case DO_SLOT:
		hr = pDevice->SetSlot(nOutput);
		break;
	case DO_ARM:
		hr = pDevice->SetArm(nOutput);
		break;
	case DO_EXTRACTMODE:
		hr = pDevice->SetExtractMode(nOutput);
		break;
	case DO_ORIGIN:
		hr = pDevice->SetOrigin(nOutput);
		break;
	case DO_STOP:
		hr = pDevice->SetStop(nOutput);
		break;
	case DO_RESET:
		hr = pDevice->SetReset(nOutput);
		break;
	case DO_GETREADY:
		hr = pDevice->SetGetReady(nOutput);
		break;
	case DO_PUTREADY:
		hr = pDevice->SetPutReady(nOutput);
		break;
	case DO_GETUPDOWN:
		hr = pDevice->SetGetUpDown(nOutput);
		break;
	case DO_PUTUPDOWN:
		hr = pDevice->SetPutUpDown(nOutput);
		break;
	case DO_GET:
		hr = pDevice->SetGet(nOutput);
		break;
	case DO_PUT:
		hr = pDevice->SetPut(nOutput);
		break;
	case DO_GRIP:
		hr = pDevice->SetGrip(nOutput);
		break;
	case DO_RETRACT:
		hr = pDevice->SetRetract(nOutput);
		break;
	case DO_EXTRACT:
		hr = pDevice->SetExtract(nOutput);
		break;
	case DO_HOME:
		hr = pDevice->SetHome(nOutput);
		break;
	case DO_MAP:
		hr = pDevice->SetMap(nOutput);
		break;
	case DO_MAPDATA:
		hr = pDevice->SetMapData(nOutput);
		break;
	default:
		CIronLib::WriteDebugLog("RND>>Invalid DI Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}

	ResetTimeout(nTimeout);
	
	return hr;
}

RND_API LONG WriteString(LPCTSTR strOutput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);
	
	switch (uAddr)
	{
	case 0:
/*		hr = pDevice->SetSO(strOutput);
		break;*/
	default:
		CIronLib::WriteDebugLog("RND>>Invalid SO Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}
	
	ResetTimeout(nTimeout);
	
	return hr;
}

RND_API LONG WriteVariable(LPCTSTR strName, unsigned uAddr, unsigned uIndex, unsigned uSize, void* pData, int nTimeout)
{
		return E_FAIL;
}