#pragma once

#include <Serial.h>

typedef enum { R_MOVING, R_COMPLETE, R_ERROR } ROBOT_STATUS;

class CDevice :
	public CSerial,
	public CIronSynchObject,
	public CIronThread
{
public:
	CDevice(void);
	CDevice(bool bLog, unsigned nPort);
	~CDevice(void);

private:
	void OnExecute();
	bool CheckComplete(string strRead);

	bool GetRobotState();

public:
	LONG GetLowArmWaferExist(int& nVal);
	LONG GetUpperArmWaferExist(int& nVal);
	LONG GetInitialStatus(int& nVal);
	LONG GetFGReadyStatus(int& nVal);
	LONG GetPutIntoStatus(int& nVal);
	LONG GetGetFromStatus(int& nVal);
	LONG GetMapScanStatus(int& nVal);
	
	LONG GetRunning(int& nVal);
	LONG GetError(int& nVal);
	LONG GetServo(int& nVal);
	LONG GetMode(int& nVal);
	LONG GetPosition(int& nVal);
	LONG GetArmState(int& nVal);
	LONG GetMapData(string& strVal);

	LONG SetInit(int nVal);
	LONG SetServo(int nVal);
	LONG SetStation(int nVal);
	LONG SetSlot(int nVal);
	LONG SetArm(int nVal);
	LONG SetFGReady(int nVal);
	LONG SetPutInto(int nVal);
	LONG SetGetFrom(int nVal);
	LONG SetMapScan(int nVal);
	LONG SetStop(int nVal);
	LONG SetResume(int nVal);
	LONG SetReset(int nVal);
	LONG SetPause(int nVal);
	LONG SetVacCtrl(int nVal);
	LONG SetWaferCheck(int nVal);

private:
	bool CheckPosition();

	HANDLE m_hStopEvent;
	int m_nStation;
	int m_nSlot;
	int m_nArm;

	int m_nLowArmWaferExist;
	int m_nUpperArmWaferExist;
	int m_nInitialStatus;
	int m_nFGReadyStatus;
	int m_nPutIntoStatus;
	int m_nGetFromStatus;
	int m_nMapScanStatus;
	int m_nVacCheckStatus;

	int m_nRunning;
	int m_nError;
	int m_nServo;
	int m_nMode;
	int m_nPosition;
	int m_nArmState;
	string m_strMapData;
};
