// Robostar.cpp : NymDriver  α׷   մϴ.
//

#include "stdafx.h"
#include "Robostar.h"
#include <DriverBase.h>
#include "Device.h"


#ifdef _MANAGED
#pragma managed(push, off)
#endif

BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
    return TRUE;
}

#ifdef _MANAGED
#pragma managed(pop)
#endif

CDriverBase* g_pDriver = NULL;
double g_nOldTimeout = 0;
double g_nOldWriteTimeout = 0;

void SetTimeout(int nTimeout)
{
	if (g_pDriver == NULL)
		return;

	//Read Timeout Setting
	g_nOldTimeout = g_pDriver->GetTimeout();
			
	if (g_nOldTimeout != nTimeout)
	{
		g_pDriver->SetTimeout(nTimeout);
	}

	//Write Timeout Setting
	int nWriteTimeout = 0;

	if (nTimeout > 0)
		nWriteTimeout = nTimeout;

	g_nOldWriteTimeout = g_pDriver->GetWriteTimeout();
		
	if (g_nOldWriteTimeout != nWriteTimeout)
	{
		g_pDriver->SetWriteTimeout(nWriteTimeout);
	}
}

void ResetTimeout(int nTimeout)
{
	if (g_pDriver == NULL)
		return;

	//ReadTimeout Setting
	if (g_nOldTimeout != nTimeout)
	{
		g_pDriver->SetTimeout(g_nOldTimeout);
	}

	int nWriteTimeout = 0;

	if (nTimeout > 0)
		nWriteTimeout = nTimeout;

	//WtieTimeout Setting
	if (g_nOldWriteTimeout != nWriteTimeout)
	{
		g_pDriver->SetWriteTimeout(g_nOldWriteTimeout);
	}
}

ROBOSTAR_API LONG InitDriver(unsigned uPort, int nTimeout, bool bLog)
{
	if (g_pDriver == NULL)
		g_pDriver = new CDevice(bLog, uPort);
	else
		return E_FAIL;

	return S_OK;
}

ROBOSTAR_API LONG UninitDriver(void)
{
	if (g_pDriver != NULL)
		delete g_pDriver;
	else
		return E_FAIL;

	return S_OK;
}

ROBOSTAR_API LONG ReadAnalog(double &dInput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);
	
	switch (uAddr)
	{
	case 0:
/*		hr = pDevice->GetAI(dInput);
		break;*/
	default:
		CIronLib::WriteDebugLog("Robostar>>Invalid AI Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}

	ResetTimeout(nTimeout);
	
	return hr;
}

ROBOSTAR_API LONG ReadDigital(int &nInput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);
	
	switch (uAddr)
	{
	case DI_LowArmWaferExist:
		hr = pDevice->GetLowArmWaferExist(nInput);
		break;
	case DI_UpperArmWaferExist:
		hr = pDevice->GetUpperArmWaferExist(nInput);
		break;
	case DI_InitialStaus:
		hr = pDevice->GetFGReadyStatus(nInput);
		break;
	case DI_FGReadyStaus:
		hr = pDevice->GetFGReadyStatus(nInput);
		break;
	case DI_PutIntoStatus:
		hr = pDevice->GetPutIntoStatus(nInput);
		break;
	case DI_GetFromStatus:
		hr = pDevice->GetGetFromStatus(nInput);
		break;
	case DI_MapScanStatus:
		hr = pDevice->GetMapScanStatus(nInput);
		break;
	case DI_Running:
		hr = pDevice->GetRunning(nInput);
		break;
	case DI_Error:
		hr = pDevice->GetError(nInput);
		break;
	case DI_Servo:
		hr = pDevice->GetServo(nInput);
		break;
	case DI_Mode:
		hr = pDevice->GetMode(nInput);
		break;
	case DI_Position:
		hr = pDevice->GetPosition(nInput);
		break;
	case DI_ArmState:
		hr = pDevice->GetArmState(nInput);
		break;
	default:
		CIronLib::WriteDebugLog("Robostar>>Invalid DI Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}
	
	ResetTimeout(nTimeout);
	
	return hr;
}

ROBOSTAR_API LONG ReadString(LPTSTR strInput, unsigned uAddr, unsigned uIndex, unsigned uSize, int nTimeout)
{
	HRESULT hr = E_FAIL;
	
	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);

	string strRead;

	switch (uAddr)
	{
	case SI_MapData:
		hr = pDevice->GetMapData(strRead);

		strcpy_s(strInput, uSize, strRead.c_str());
		break;
	default:
		CIronLib::WriteDebugLog("Robostar>>Invalid SI Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}
	
	ResetTimeout(nTimeout);

	return hr;
}

ROBOSTAR_API LONG ReadVariable(LPCTSTR strName, unsigned uAddr, unsigned uIndex, unsigned uSize, void* pData, int nTimeout)
{
		return E_FAIL;
}

ROBOSTAR_API LONG WriteAnalog(double dOutput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);

	switch (uAddr)
	{
	case 0:
/*		hr = pDevice->SetAO(dOutput);
		break;*/
	default:
		CIronLib::WriteDebugLog("Robostar>>Invalid AO Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}

	ResetTimeout(nTimeout);
	
	return hr;
}

ROBOSTAR_API LONG WriteDigital(int nOutput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;
	
	SetTimeout(nTimeout);
	
	switch (uAddr)
	{
	case DO_Initialize:
		hr = pDevice->SetInit(nOutput);
		break;
	case DO_ServoOffOn:
		hr = pDevice->SetServo(nOutput);
		break;
	case DO_Station:
		hr = pDevice->SetStation(nOutput);
		break;
	case DO_Slot:
		hr = pDevice->SetSlot(nOutput);
		break;
	case DO_Arm:
		hr = pDevice->SetArm(nOutput);
		break;
	case DO_FGReady:
		hr = pDevice->SetFGReady(nOutput);
		break;
	case DO_PutInto:
		hr = pDevice->SetPutInto(nOutput);
		break;
	case DO_GetFrom:
		hr = pDevice->SetGetFrom(nOutput);
		break;
	case DO_MapScan:
		hr = pDevice->SetMapScan(nOutput);
		break;
	case DO_Stop:
		hr = pDevice->SetStop(nOutput);
		break;
	case DO_Resume:
		hr = pDevice->SetResume(nOutput);
		break;
	case DO_Reset:
		hr = pDevice->SetReset(nOutput);
		break;
	case DO_Pause:
		hr = pDevice->SetPause(nOutput);
		break;
	case DO_VacCtrl:
		hr = pDevice->SetVacCtrl(nOutput);
		break;
	case DO_WaferCheck:
		hr = pDevice->SetWaferCheck(nOutput);
		break;
	default:
		CIronLib::WriteDebugLog("Robostar>>Invalid DI Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}

	ResetTimeout(nTimeout);
	
	return hr;
}

ROBOSTAR_API LONG WriteString(LPCTSTR strOutput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);
	
	switch (uAddr)
	{
	case 0:
/*		hr = pDevice->SetSO(strOutput);
		break;*/
	default:
		CIronLib::WriteDebugLog("Robostar>>Invalid SO Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}
	
	ResetTimeout(nTimeout);
	
	return hr;
}

ROBOSTAR_API LONG WriteVariable(LPCTSTR strName, unsigned uAddr, unsigned uIndex, unsigned uSize, void* pData, int nTimeout)
{
		return E_FAIL;
}