#include "StdAfx.h"
#include "Device.h"
#include <math.h>

static CIronCriticalSection g_cLock;

CDevice::CDevice(void)
{
}

CDevice::CDevice(bool bLog,
				 bool bThread,
				 unsigned char* netId,
				 unsigned uPort,
				 int nDISize,
				 int nDIOffset,
				 int nDOSize,
				 int nDOOffset,
				 int nAISize,
				 int nAIOffset,
				 int nAOSize,
				 int nAOOffset)
: m_pLog(NULL), m_bThread(bThread), m_nDIOffset(nDIOffset), m_nDOOffset(nDOOffset),m_nAISize(nAISize), 
m_nAIOffset(nAIOffset),m_nAOSize(nAOSize), m_nAOOffset(nAOOffset), m_nErr(0), m_nPrevGetErr(-1), m_nPrevSetErr(-1)
{
	Open(uPort, netId);

	if( nDISize % 8 > 0 )
		m_nDISize = nDISize / 8 + 1;
	else
		m_nDISize = nDISize / 8;

	if( nDOSize % 8 > 0 )
		m_nDOSize = nDOSize / 8 + 1;
	else
		m_nDOSize = nDOSize / 8;

	m_pDIBuffer = (BYTE*)malloc(sizeof(BYTE) * m_nDISize);
	m_pDOBuffer = (BYTE*)malloc(sizeof(BYTE) * m_nDOSize);
	m_pAIBuffer = (WORD*)malloc(sizeof(WORD) * m_nAISize);
	m_pAOBuffer = (WORD*)malloc(sizeof(WORD) * m_nAOSize);

	if (bLog)
	{
		m_pLog = new CIronLog("TWinCAT", "");
	}

	if (m_bThread)
	{
		m_hStopEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

		assert(m_hStopEvent);

		BeginThread();
	}
}

CDevice::~CDevice(void)
{
	if (m_bThread)
	{
		SetEvent(m_hStopEvent);
		WaitFor();

		CloseHandle(m_hStopEvent);
	}

	for (map<ULONG, string>::iterator iter = m_mNotification.begin(); iter != m_mNotification.end(); ++iter)
	{
		m_nErr = AdsSyncDelDeviceNotificationReq(m_pAddr, iter->first);

		if (m_nErr != 0)
		{
			CIronLib::WriteDebugLog("Error: AdsSyncDelDeviceNotificationReq: %d, Variable Name(%d) : %s (%s:%d)", m_nErr, iter->first, iter->second.c_str(), __FILE__, __LINE__);
		}
	}

	for (map<string, ULONG>::iterator iter = m_mHandle.begin(); iter != m_mHandle.end(); ++iter)
	{
		m_nErr = AdsSyncWriteReq(m_pAddr, ADSIGRP_SYM_RELEASEHND, iter->second, 0, 0);

		if (m_nErr != 0)
		{
			CIronLib::WriteDebugLog("Error: AdsSyncWriteReq: %d, Variable Name : %s (%s:%d)", m_nErr, iter->first.c_str(), __FILE__, __LINE__);
		}
	}

	m_nErr = AdsPortClose();
	
	if (m_nErr != 0)
	{
		CIronLib::WriteDebugLog("Error: AdsPortClose: %d (%s:%d)", m_nErr, __FILE__, __LINE__);
	}

	if( m_pLog )
	{
		delete m_pLog;
	}

	free(m_pDIBuffer);
	free(m_pDOBuffer);
	free(m_pAIBuffer);
	free(m_pAOBuffer);

	for (map<string, void*>::iterator iter = m_mVariable.begin() ; iter != m_mVariable.end() ; ++iter)
	{
		if (iter->second != NULL)
		{
			delete[] iter->second;
		}
	}
}

void CDevice::Open(unsigned uPort, unsigned char* netId)
{
	m_pAddr = &m_Addr;

	m_nPort = AdsPortOpen();
	m_nErr = AdsGetLocalAddress(m_pAddr);
	
	if (m_nErr != 0)
	{
		CIronLib::WriteDebugLog("Error: AdsGetLocalAddress: %d (%s:%d)", m_nErr, __FILE__, __LINE__);
	}
	else
	{
		if (netId != NULL)
		{
			memcpy(m_pAddr->netId.b, netId, sizeof(unsigned char) * 6);
		}

		m_pAddr->port = uPort;
	}
}

void CDevice::OnExecute(void)
{
	LONG nPrevDIError = 0;
	LONG nPrevDOError = 0;
	LONG nPrevAIError = 0;
	LONG nPrevAOError = 0;

	while( WaitForSingleObject(m_hStopEvent, 1) )
	{
		CIronLock cLock(*this);

		//Digital Input
		if( m_nDISize > 0 )
		{
			m_nErr = AdsSyncReadReq(m_pAddr, ADSIGRP_IOIMAGE_RWIB, m_nDIOffset / 8, m_nDISize, m_pDIBuffer);

			if( m_nErr != 0 && m_nErr != nPrevDIError)
			{
				CIronLib::WriteDebugLog("Error: AdsSyncReadReq(DI): %d (%s:%d)", m_nErr, __FILE__, __LINE__);
			}

			nPrevDIError = m_nErr;
		}

		//Digital Output
		if( m_nDOSize > 0 )
		{
			m_nErr = AdsSyncReadReq(m_pAddr, ADSIGRP_IOIMAGE_RWOB, m_nDOOffset / 8, m_nDOSize, m_pDOBuffer);

			if( m_nErr != 0 && m_nErr != nPrevDOError)
			{
				CIronLib::WriteDebugLog("Error: AdsSyncReadReq(DO): %d (%s:%d)", m_nErr, __FILE__, __LINE__);
			}

			nPrevDOError = m_nErr;
		}

		//Analog Input
		if( m_nAISize > 0 )
		{
			m_nErr = AdsSyncReadReq(m_pAddr, ADSIGRP_IOIMAGE_RWIB, m_nAIOffset, m_nAISize * 2, m_pAIBuffer);
			
			if( m_nErr != 0 && m_nErr != nPrevAIError)
			{
				CIronLib::WriteDebugLog("Error: AdsSyncReadReq(AI): %d (%s:%d)", m_nErr, __FILE__, __LINE__);
			}
			
			nPrevAIError = m_nErr;
		}

		//Analog Output
		if( m_nAOSize > 0 )
		{
			m_nErr = AdsSyncReadReq(m_pAddr, ADSIGRP_IOIMAGE_RWOB, m_nAOOffset, m_nAOSize * 2, m_pAOBuffer);

			if( m_nErr != 0 && m_nErr != nPrevAOError )
			{
				CIronLib::WriteDebugLog("Error: AdsSyncReadReq(AO): %d (%s:%d)", m_nErr, __FILE__, __LINE__);
			}
			
			nPrevAOError = m_nErr;
		}
	}
}

double CDevice::DataToVol(WORD nVal)
{
	double dRet = 0;

	if( nVal >= 0x7FF0 && nVal <= 0x7FFF )
	{
		return 10;
	}
	if( nVal >= 0x8000 && nVal <= 0x800F)
	{
		return -10;
	}
	else if( nVal > 0x800F )
	{
		dRet = -10;
	}

	for( int index = 0 ; index < 15 ; ++index )
	{
		if( nVal & 1 << (14 - index) )
		{
			dRet += 10.0 / pow(2.0, index + 1);
		}
	}

	return dRet;
}

WORD CDevice::VolToData(double dVal)
{
	WORD nRet = 0;
	double dVolt = 0;
	bool bMinus = false;

	if( dVal < 0 )
	{
		bMinus = true;
		dVal = -dVal;
	}

	for( int index = 0 ; index < 15 ; ++index )
	{
		dVolt = 10.0 / pow(2.0, index + 1);

		if( dVal >=  dVolt)
		{
			nRet ^= 1 << (14 - index);
			                                                                                                                                                                                                                                                                                                                                                                                 
			dVal -= dVolt;
		}
	}

	if( bMinus )
	{
		if( nRet >= 0x7FF0 && nRet <= 0x7FFF )
			nRet = ~nRet;
		else
			nRet = ~nRet + 1;
	}

	return nRet;
}

double CDevice::DataToCurrent(WORD nVal)
{
	double dRet = 4;

	if( nVal >= 0x7FFF )
	{
		return 20;
	}

	if( nVal <= 0x0000 )
	{
		return 4;
	}

	for( int index = 0 ; index < 15 ; ++index )
	{
		if( nVal & 1 << (14 - index) )
		{
			dRet += 8.0 / pow(2.0, index);
		}
	}

	return dRet;
}

WORD CDevice::CurrentToData(double dVal)
{
	WORD nRet = 0;
	double dCurrent = 0;

	if( dVal >= 20 )
	{
		return 0x7FFF;
	}

	if( dVal <= 4 )
	{
		return 0x0000;
	}

	for( int index = 0 ; index < 15 ; ++index )
	{
		dCurrent = 4 + 8.0 / pow(2.0, index);

		if( dVal >=  dCurrent)
		{
			nRet ^= 1 << (14 -index);

			dVal -= dCurrent - 4;
		}
	}

	return nRet;
}

bool CDevice::GetDI(unsigned uAddr, unsigned uIndex, int &nData)
{
	if( m_bThread )
	{
		m_pDIBuffer[uAddr/8] & 1 << (uAddr%8) ? nData = 1 : nData = 0;
	}
	else
	{
		BYTE byRead = 0;

		CIronLock cLock(*this);

		m_nErr = AdsSyncReadReq(m_pAddr, ADSIGRP_IOIMAGE_RWIX, m_nDIOffset + uIndex, 0x01, &byRead);

		if (m_nErr != 0)
		{
			CIronLib::WriteDebugLog("Error: AdsSyncReadReq(DI): %d (%s:%d)", m_nErr, __FILE__, __LINE__);
			return false;
		}
		
		byRead ? nData = 1 : nData = 0;
	}

	return true;
}

bool CDevice::GetDO(unsigned uAddr, unsigned uIndex, int &nData)
{
	if( m_bThread )
	{
		m_pDOBuffer[uAddr/8] & 1 << (uAddr%8) ? nData = 1 : nData = 0;
	}
	else
	{
		BYTE byRead = 0;

		CIronLock cLock(*this);

		m_nErr = AdsSyncReadReq(m_pAddr, ADSIGRP_IOIMAGE_RWOX, m_nDOOffset + uIndex, 0x01, &byRead);

		if (m_nErr != 0)
		{
			CIronLib::WriteDebugLog("Error: AdsSyncReadReq(DO): %d (%s:%d)", m_nErr, __FILE__, __LINE__);
			return false;
		}
		
		byRead ? nData = 1 : nData = 0;
	}

	return true;
}

bool CDevice::GetAI(unsigned uAddr, unsigned uIndex, double &dData, int nType)
{
	WORD wRead;

	if( m_bThread )
	{
		wRead = m_pAIBuffer[uAddr];
	}
	else
	{
		WORD wRet = 0;

		CIronLock cLock(*this);

		m_nErr = AdsSyncReadReq(m_pAddr, ADSIGRP_IOIMAGE_RWIB, m_nAIOffset + uIndex, 0x02, &wRet);

		if (m_nErr != 0)
		{
			CIronLib::WriteDebugLog("Error: AdsSyncReadReq(AI): %d (%s:%d)", m_nErr, __FILE__, __LINE__);
			return false;
		}
		
		wRead = wRet;
	}

	switch( nType )
	{
	case NONE:
		dData = wRead;
		break;
	case VOL_0_10:
		dData = DataToVol(wRead);
		break;
	case CUR_4_20:
		dData = DataToCurrent(wRead);
		break;
	default:
		dData = wRead;
		break;
	}

	return true;
}

bool CDevice::GetAO(unsigned uAddr, unsigned uIndex, double &dData, int nType)
{
	WORD wRead;

	if( m_bThread )
	{
		wRead = m_pAOBuffer[uAddr];
	}
	else
	{
		WORD wRet = 0;

		CIronLock cLock(*this);

		m_nErr = AdsSyncReadReq(m_pAddr, ADSIGRP_IOIMAGE_RWOB, m_nAOOffset + uIndex, 0x02, &wRet);

		if (m_nErr != 0)
		{
			CIronLib::WriteDebugLog("Error: AdsSyncReadReq(AO): %d (%s:%d)", m_nErr, __FILE__, __LINE__);
			return false;
		}
		
		wRead = wRet;
	}

	switch( nType )
	{
	case NONE:
		dData = wRead;
		break;
	case VOL_0_10:
		dData = DataToVol(wRead);
		break;
	case CUR_4_20:
		dData = DataToCurrent(wRead);
		break;
	default:
		dData = wRead;
		break;
	}

	return true;
}

bool CDevice::SetDO(unsigned uAddr, unsigned uIndex, int nData)
{
	BYTE byWrite = 0;

	nData ? byWrite = 1 : byWrite = 0;

	CIronLock cLock(*this);

	m_nErr = AdsSyncWriteReq(m_pAddr, ADSIGRP_IOIMAGE_RWOX, m_nDOOffset + uIndex, 0x01, &byWrite);

	if (m_nErr != 0)
	{
		CIronLib::WriteDebugLog("Error: AdsSyncWriteReq(DO): %d (%s:%d)", m_nErr, __FILE__, __LINE__);
		return false;
	}
	
	return true;
}

bool CDevice::SetAO(unsigned uAddr, unsigned uIndex, double dData, int nType)
{
	WORD wWrite;

	switch( nType )
	{
	case NONE:
		wWrite = static_cast<WORD>(dData);
		break;
	case VOL_0_10:
		wWrite = VolToData(dData);
		break;
	case CUR_4_20:
		wWrite = CurrentToData(dData);
		break;
	default:
		wWrite = static_cast<WORD>(dData);
		break;
	}

	CIronLock cLock(*this);

	m_nErr = AdsSyncWriteReq(m_pAddr, ADSIGRP_IOIMAGE_RWOB, m_nAOOffset + uIndex, 0x02, &wWrite);

	if (m_nErr != 0)
	{
		CIronLib::WriteDebugLog("Error: AdsSyncWriteReq(AO): %d (%s:%d)", m_nErr, __FILE__, __LINE__);
		return false;
	}
	
	return true;
}

bool CDevice::GetVariable(LPTSTR varName, unsigned varSize, unsigned uSize, void* pData, bool bEvent)
{
	if (m_mVariable.find(varName) != m_mVariable.end() && m_mVariable[varName] != NULL)
	{
		CIronLock cLock(g_cLock);
		memcpy(pData, m_mVariable[varName], uSize);
	}
	else
	{
		CIronLock cLock(*this);

		if (m_mHandle.find(varName) == m_mHandle.end())
		{
			unsigned long lHdlVar;

			m_nErr = AdsSyncReadWriteReq(m_pAddr, ADSIGRP_SYM_HNDBYNAME, 0x0, sizeof(lHdlVar), &lHdlVar, varSize, varName);

			if (m_nErr != 0 && m_nErr != m_nPrevGetErr)
			{
				CIronLib::WriteDebugLog("Error: AdsSyncReadWriteReq: %d, Variable Name : %s (%s:%d)", m_nErr, varName, __FILE__, __LINE__);
				m_nPrevGetErr = m_nErr;

				return false;
			}

			m_mHandle[varName] = lHdlVar;
		}

		m_nErr = AdsSyncReadReq(m_pAddr, ADSIGRP_SYM_VALBYHND, m_mHandle[varName], uSize, pData);

		if (m_nErr != 0 && m_nErr != m_nPrevGetErr)
		{
			CIronLib::WriteDebugLog("Error: AdsSyncReadReq: %d, Variable Name : %s (%s:%d)", m_nErr, varName, __FILE__, __LINE__);
			m_nPrevGetErr = m_nErr;

			return false;
		}

		if (bEvent && m_mVariable.find(varName) == m_mVariable.end())
		{
			ULONG hNotification = 0;
			ULONG hUser = 0;
			AdsNotificationAttrib  adsNotificationAttrib;
			
			adsNotificationAttrib.cbLength = uSize;
			adsNotificationAttrib.nTransMode = ADSTRANS_SERVERONCHA;
			adsNotificationAttrib.nMaxDelay = 10000000; // 1sec
			adsNotificationAttrib.nCycleTime = 1000000; // 0.1sec
			
			m_nErr = AdsSyncAddDeviceNotificationReq(m_pAddr, ADSIGRP_SYM_VALBYHND, m_mHandle[varName], &adsNotificationAttrib, Callback, hUser, &hNotification);
			
			if (m_nErr != 0 && m_nErr != m_nPrevGetErr)
			{
				CIronLib::WriteDebugLog("Error: AdsSyncAddDeviceNotificationReq: %d, Variable Name : %s (%s:%d)", m_nErr, varName, __FILE__, __LINE__);
				m_nPrevGetErr = m_nErr;

				return false;
			}

			CIronLock cLock(g_cLock);
			m_mVariable[varName] = NULL;
			m_mNotification[hNotification] = varName;

			//CIronLib::WriteDebugLog("%s Notification Handle = %d", varName, hNotification);
		}
	}

	return true;
}

bool CDevice::SetVariable(LPTSTR varName, unsigned varSize, unsigned uSize, void* pData)
{
	CIronLock cLock(*this);

	if (m_mHandle.find(varName) == m_mHandle.end())
	{
		unsigned long lHdlVar;
		m_nErr = AdsSyncReadWriteReq(m_pAddr, ADSIGRP_SYM_HNDBYNAME, 0x0, sizeof(lHdlVar), &lHdlVar, varSize, varName);;

		if (m_nErr != 0 && m_nErr != m_nPrevSetErr)
		{
			CIronLib::WriteDebugLog("Error: AdsSyncReadWriteReq: %d, Variable Name : %s (%s:%d)", m_nErr, varName, __FILE__, __LINE__);
			m_nPrevSetErr = m_nErr;

			return false;
		}

		m_mHandle[varName] = lHdlVar;
	}

	m_nErr = AdsSyncWriteReq(m_pAddr, ADSIGRP_SYM_VALBYHND, m_mHandle[varName], uSize, pData);

	if (m_nErr != 0 &&m_nErr != m_nPrevGetErr)
	{
		CIronLib::WriteDebugLog("Error: AdsSyncWriteReq: %d, Variable Name : %s (%s:%d)", m_nErr, varName, __FILE__, __LINE__);
		m_nPrevGetErr = m_nErr;

		return false;
	}

	return true;
}

void __stdcall CDevice::Callback(AmsAddr* pAddr, AdsNotificationHeader* pNotification, ULONG hUser)
{
	CIronLock cLock(g_cLock);

	if (m_mNotification.find(pNotification->hNotification) != m_mNotification.end())
	{
		string varName = m_mNotification[pNotification->hNotification];

		if (m_mVariable.find(varName) != m_mVariable.end())
		{
			if (m_mVariable[varName] == NULL)
			{
				m_mVariable[varName] = (void*)new unsigned char[pNotification->cbSampleSize];
			}

			memcpy(m_mVariable[varName], pNotification->data, pNotification->cbSampleSize);
		}
	}
}
