// TwinCAT.cpp : NymDriver  α׷   մϴ.
//

#include "stdafx.h"
#include "TwinCAT.h"
#include "Device.h"


#ifdef _MANAGED
#pragma managed(push, off)
#endif

BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
    return TRUE;
}

#ifdef _MANAGED
#pragma managed(pop)
#endif

CDevice* g_pDevice = NULL;


TWINCAT_API LONG InitDriver(unsigned uPort, int nTimeout, bool bLog)
{
	if (g_pDevice == NULL)
	{
		int thread = CIronLib::ReadRegistryInt(HKEY_LOCAL_MACHINE, "SOFTWARE\\IronTools\\Application\\Driver\\TwinCAT\\", "Thread", 1);
		int DISize = CIronLib::ReadRegistryInt(HKEY_LOCAL_MACHINE, "SOFTWARE\\IronTools\\Application\\Driver\\TwinCAT\\", "DISize", 0);
		int DIOffset = CIronLib::ReadRegistryInt(HKEY_LOCAL_MACHINE, "SOFTWARE\\IronTools\\Application\\Driver\\TwinCAT\\", "DIOffset", 0);
		int DOSize = CIronLib::ReadRegistryInt(HKEY_LOCAL_MACHINE, "SOFTWARE\\IronTools\\Application\\Driver\\TwinCAT\\", "DOSize", 0);
		int DOOffset = CIronLib::ReadRegistryInt(HKEY_LOCAL_MACHINE, "SOFTWARE\\IronTools\\Application\\Driver\\TwinCAT\\", "DOOffset", 0);
		int AISize = CIronLib::ReadRegistryInt(HKEY_LOCAL_MACHINE, "SOFTWARE\\IronTools\\Application\\Driver\\TwinCAT\\", "AISize", 0);
		int AIOffset = CIronLib::ReadRegistryInt(HKEY_LOCAL_MACHINE, "SOFTWARE\\IronTools\\Application\\Driver\\TwinCAT\\", "AIOffset", 0);
		int AOSize = CIronLib::ReadRegistryInt(HKEY_LOCAL_MACHINE, "SOFTWARE\\IronTools\\Application\\Driver\\TwinCAT\\", "AOSize", 0);
		int AOOffset = CIronLib::ReadRegistryInt(HKEY_LOCAL_MACHINE, "SOFTWARE\\IronTools\\Application\\Driver\\TwinCAT\\", "AOOffset", 0);
		
		char netID[256];
		
		if (!CIronLib::ReadRegistryString(HKEY_LOCAL_MACHINE, "SOFTWARE\\IronTools\\Application\\Driver\\TwinCAT\\", "NetID", "", netID, 256))
		{
			return E_FAIL;
		}

		string strNetID = netID;

		vector<string> netIDItem = CIronLib::split(strNetID, ".");

		unsigned char* netIDs = NULL;

		if (netIDItem.size() == 6)
		{
			netIDs = new unsigned char[6];

			netIDs[0] = static_cast<unsigned char>(atoi(netIDItem[0].c_str()));
			netIDs[1] = static_cast<unsigned char>(atoi(netIDItem[1].c_str()));
			netIDs[2] = static_cast<unsigned char>(atoi(netIDItem[2].c_str()));
			netIDs[3] = static_cast<unsigned char>(atoi(netIDItem[3].c_str()));
			netIDs[4] = static_cast<unsigned char>(atoi(netIDItem[4].c_str()));
			netIDs[5] = static_cast<unsigned char>(atoi(netIDItem[5].c_str()));
		}

		g_pDevice = new CDevice(bLog, thread > 0, netIDs, uPort, DISize, DIOffset, DOSize, DOOffset, AISize, AIOffset, AOSize, AOOffset);

		if (netIDItem.size() == 6)
		{
			delete[] netIDs;
		}
	}
	else
		return E_FAIL;

	return S_OK;
}

TWINCAT_API LONG UninitDriver(void)
{
	if (g_pDevice != NULL)
		delete g_pDevice;
	else
		return E_FAIL;

	return S_OK;
}

TWINCAT_API LONG ReadAnalog(double &dInput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	bool bRet = false;

	if( uAddr >= 1000 )
		bRet = g_pDevice->GetAO(uAddr - 1000, uIndex, dInput, nTimeout);
	else
		bRet = g_pDevice->GetAI(uAddr, uIndex, dInput, nTimeout);

	if (!bRet)
		return E_FAIL;

	return S_OK;
}

TWINCAT_API LONG ReadDigital(int &nInput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	bool bRet = false;

	if (uAddr >= 1000)
	{
		bRet = g_pDevice->GetDO(uAddr - 1000, uIndex, nInput);
	}
	else
		bRet = g_pDevice->GetDI(uAddr, uIndex, nInput);

	if (!bRet)
		return E_FAIL;
	
	return S_OK;
}

TWINCAT_API LONG ReadString(LPTSTR strInput, unsigned uAddr, unsigned uIndex, unsigned uSize, int nTimeout)
{
	HRESULT hr = E_FAIL;
	
	switch (uAddr)
	{
	case 0:
/*		hr = g_pDevice->GetSI(strInput);
		break;*/
	default:
		CIronLib::WriteDebugLog("TwinCAT>>Invalid SI Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}
	
	return hr;
}

TWINCAT_API LONG ReadVariable(LPCTSTR strName, unsigned uAddr, unsigned uIndex, unsigned uSize, void* pData, int nTimeout)
{
	unsigned varSize = static_cast<unsigned>(strlen(strName) + 1);
	char* var = new char[varSize];

	strcpy_s(var, varSize, strName);

	bool bRet = g_pDevice->GetVariable(var, varSize, uSize, pData, uIndex > 0);

	delete[] var;

	if (bRet)
	{
		return S_OK;
	}

	return E_FAIL;
}

TWINCAT_API LONG WriteAnalog(double dOutput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	bool bRet = g_pDevice->SetAO(uAddr, uIndex, dOutput, nTimeout);

	if (!bRet)
		return E_FAIL;

	return S_OK;
}

TWINCAT_API LONG WriteDigital(int nOutput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	bool bRet = g_pDevice->SetDO(uAddr, uIndex, nOutput);

	if (!bRet)
		return E_FAIL;

	return S_OK;
}

TWINCAT_API LONG WriteString(LPCTSTR strOutput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	switch (uAddr)
	{
	case 0:
/*		hr = g_pDevice->SetSO(strOutput);
		break;*/
	default:
		CIronLib::WriteDebugLog("TwinCAT>>Invalid SO Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}
	
	return hr;
}

TWINCAT_API LONG WriteVariable(LPCTSTR strName, unsigned uAddr, unsigned uIndex, unsigned uSize, void* pData, int nTimeout)
{
	unsigned varSize = static_cast<unsigned>(strlen(strName) + 1);
	char* var = new char[varSize];
	
	strcpy_s(var, varSize, strName);

	bool bRet = g_pDevice->SetVariable(var, varSize, uSize, pData);

	delete[] var;

	if (bRet)
	{
		return S_OK;
	}

	return E_FAIL;
}