#include "StdAfx.h"
#include "Device.h"
#include <vector>

using namespace std;

CDevice::CDevice(void)
{
}

CDevice::CDevice(bool bLog, unsigned uPort)
: CSerial(bLog, uPort), m_dAngle(0), m_nWaferExistence(0), m_nComplete(1)
{
	m_hStopEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

	assert(m_hStopEvent);

	SetTimeout(1);
	BeginThread();
}

CDevice::~CDevice(void)
{
	SetEvent(m_hStopEvent);
	WaitFor();

	CloseHandle(m_hStopEvent);
}

void CDevice::OnExecute()
{
	while( WaitForSingleObject(m_hStopEvent, 1) )
	{
		CIronLock cLock(*this);

		if (m_nComplete == 0 && InWaiting() > 0)
		{
			string strRead = ReadLine('>');

			CheckNotification(strRead);
		}
	}
}

ACK CDevice::CheckNotification(string strRead)
{
	ACK ack = None;

	if (strRead.length() >= 4 && (strRead.substr(0, 1) == "#") && (strRead.substr(strRead.length() -1, 1) == ">"))
	{
		string strMsg = strRead.substr(1, strRead.length() - 4);

		if (strcmp(strMsg.c_str(), "OK") == 0)
		{
			m_nComplete = 1;
			ack = OK;
		}
		else if (strcmp(strMsg.c_str(), "ER") == 0)
		{
			m_nComplete = 1;
			ack = ER;
		}
	}

	return ack;
}

ACK CDevice::CheckAck(string strRead)
{
	ACK ack = None;

	if (strRead.length() >= 4 && (strRead.substr(0, 1) == "#") && (strRead.substr(strRead.length() -1, 1) == ">"))
	{
		string strMsg = strRead.substr(1, strRead.length() - 4);

		if (strcmp(strMsg.c_str(), "ST") == 0)
		{
			ack = ST;
		}
		else if (strcmp(strMsg.c_str(), "BU") == 0)
		{
			ack = BU;
		}
	}

	return ack;
}

LONG CDevice::GetAngleOffset(double& dVal)
{
	CIronLock cLock(*this);

	if (!Write("#ANOF\r"))
		return E_FAIL;

	string strRead = ReadLine('>');

	//Check Notification
	ACK ack = CheckNotification(strRead);
	
	if (ack == OK || ack == ER)
	{
		strRead = ReadLine('>');
	}
	//Check Notification End

	if (strRead.length() < 4 || (strRead.substr(0, 1) != "#") || (strRead.substr(strRead.length() -1, 1) != ">"))
	{
		CIronLib::WriteDebugLog("[WACCO]ANOF Error[%s]", strRead.c_str());
		return E_FAIL;
	}

	dVal = atoi(strRead.substr(1, strRead.length() - 4).c_str());

	return S_OK;
}

LONG CDevice::GetAngle(double& dVal)
{
	dVal = m_dAngle;

	return S_OK;
}

LONG CDevice::GetChuckPosition(int& nVal)
{
	CIronLock cLock(*this);

	string strMsg = "";

	if (!Write("#CPOS\r"))
		return E_FAIL;

	string strRead = ReadLine('>');

	//Check Notification
	ACK ack = CheckNotification(strRead);
	
	if (ack == OK || ack == ER)
	{
		strRead = ReadLine('>');
	}
	//Check Notification End


	if (strRead.length() < 4 || (strRead.substr(0, 1) != "#") || (strRead.substr(strRead.length() -1, 1) != ">"))
	{
		CIronLib::WriteDebugLog("[WACCO]CPOS Ack Error[%s]", strRead.c_str());
		return E_FAIL;
	}

	strMsg = strRead.substr(1, strRead.length() - 4);

	if (strcmp(strMsg.c_str(), "DN") == 0)
		nVal = 0;
	else if (strcmp(strMsg.c_str(), "BE") == 0)
		nVal = 1;
	else if (strcmp(strMsg.c_str(), "AB") == 0)
		nVal = 2;
	else if (strcmp(strMsg.c_str(), "UP") == 0)
		nVal = 3;
	else if (strcmp(strMsg.c_str(), "MO") == 0)
		nVal = 4;
	else if (strcmp(strMsg.c_str(), "NA") == 0)
		nVal = 5;
	else
		return E_FAIL;

	return S_OK;
}

LONG CDevice::GetErrorStatus(int& nVal)
{
	CIronLock cLock(*this);

	string strMsg = "";

	if (!Write("#ERST\r"))
		return E_FAIL;

	string strRead = ReadLine('>');

	//Check Notification
	ACK ack = CheckNotification(strRead);
	
	if (ack == OK || ack == ER)
	{
		strRead = ReadLine('>');
	}
	//Check Notification End

	if (strRead.length() < 4 || (strRead.substr(0, 1) != "#") || (strRead.substr(strRead.length() -1, 1) != ">"))
	{
		CIronLib::WriteDebugLog("[WACCO]ERST Ack Error[%s]", strRead.c_str());
		return E_FAIL;
	}

	strMsg = strRead.substr(1, strRead.length() - 4);

	nVal = atoi(strMsg.c_str());

	return S_OK;
}

LONG CDevice::GetSystemSetting(int& nVal)
{
	CIronLock cLock(*this);

	string strMsg = "";

	if (!Write("#SSET\r"))
		return E_FAIL;

	string strRead = ReadLine('>');

	//Check Notification
	ACK ack = CheckNotification(strRead);
	
	if (ack == OK || ack == ER)
	{
		strRead = ReadLine('>');
	}
	//Check Notification End

	if (strRead.length() < 4 || (strRead.substr(0, 1) != "#") || (strRead.substr(strRead.length() -1, 1) != ">"))
	{
		CIronLib::WriteDebugLog("[WACCO]SSET Ack Error[%s]", strRead.c_str());
		return E_FAIL;
	}

	strMsg = strRead.substr(1, strRead.length() - 4);

	nVal = CIronLib::HextoDec(strMsg);

	return S_OK;
}

LONG CDevice::GetSystemStatus(int& nVal)
{
	CIronLock cLock(*this);

	string strMsg = "";

	if (!Write("#STAT\r"))
		return E_FAIL;

	string strRead = ReadLine('>');

	//Check Notification
	ACK ack = CheckNotification(strRead);
	
	if (ack == OK || ack == ER)
	{
		strRead = ReadLine('>');
	}
	//Check Notification End

	if (strRead.length() < 4 || (strRead.substr(0, 1) != "#") || (strRead.substr(strRead.length() -1, 1) != ">"))
	{
		CIronLib::WriteDebugLog("[WACCO]STAT Ack Error[%s]", strRead.c_str());
		return E_FAIL;
	}

	strMsg = strRead.substr(1, strRead.length() - 4);

	nVal = CIronLib::HextoDec(strMsg);

	return S_OK;
}

LONG CDevice::GetVacuum(int& nVal)
{
	CIronLock cLock(*this);

	string strMsg = "";

	if (!Write("#VACH\r"))
		return E_FAIL;

	string strRead = ReadLine('>');

	//Check Notification
	ACK ack = CheckNotification(strRead);
	
	if (ack == OK || ack == ER)
	{
		strRead = ReadLine('>');
	}
	//Check Notification End

	if (strRead.length() < 4 || (strRead.substr(0, 1) != "#") || (strRead.substr(strRead.length() -1, 1) != ">"))
	{
		CIronLib::WriteDebugLog("[WACCO]VACH Ack Error[%s]", strRead.c_str());
		return E_FAIL;
	}

	strMsg = strRead.substr(1, strRead.length() - 4);

	nVal = atoi(strMsg.c_str());

	if(nVal == 0)
		nVal = 0;
	else if(nVal == 1)
		nVal = 1;
	else
		return E_FAIL;

	return S_OK;
}

LONG CDevice::GetWaferSize(int& nVal)
{
	CIronLock cLock(*this);

	string strMsg = "";

	if (!Write("#WSIZ\r"))
		return E_FAIL;

	string strRead = ReadLine('>');

	//Check Notification
	ACK ack = CheckNotification(strRead);
	
	if (ack == OK || ack == ER)
	{
		strRead = ReadLine('>');
	}
	//Check Notification End

	if (strRead.length() < 4 || (strRead.substr(0, 1) != "#") || (strRead.substr(strRead.length() -1, 1) != ">"))
	{
		CIronLib::WriteDebugLog("[WACCO]WSIZ Ack Error[%s]", strRead.c_str());
		return E_FAIL;
	}

	strMsg = strRead.substr(1, strRead.length() - 4);

	nVal = atoi(strMsg.c_str());

	if(nVal == 1)
		nVal = 0;
	else if(nVal == 2)
		nVal = 1;
	else if(nVal == 10)
		nVal = 0;
	else if(nVal == 20)
		nVal = 1;
	else
		return E_FAIL;

	return S_OK;
}

LONG CDevice::GetWaferExistence(int& nVal)
{
	nVal = m_nWaferExistence;
	return S_OK;
}

LONG CDevice::GetComplete(int& nVal)
{
	nVal = m_nComplete;
	return S_OK;
}

LONG CDevice::SetAngleOffset(double dVal)
{
	CIronLock cLock(*this);

	string sSendMsg = "#ANOF ";
	string strMsg = "";

	sSendMsg += CIronLib::itostring(dVal);
	sSendMsg += "\r";

	if (!Write(sSendMsg))
		return E_FAIL;

	string strRead = ReadLine('>');

	//Check Notification
	ACK ack = CheckNotification(strRead);
	
	if (ack == OK || ack == ER)
	{
		strRead = ReadLine('>');
	}
	//Check Notification End

	ack = CheckAck(strRead);

	if (ack == None)
	{
		CIronLib::WriteDebugLog("[WACCO]ANOF Ack Error[%s]", strRead.c_str());
		return E_FAIL;
	}

	return S_OK;
}

LONG CDevice::SetAngle(double dVal)
{
	m_dAngle = dVal;

	return S_OK;
}

LONG CDevice::SetChuckUpDown(int nVal)
{
	CIronLock cLock(*this);

	if (nVal == 0)
	{
		string strMsg = "";

		if (!Write("#CKDN\r"))
			return E_FAIL;

		string strRead = ReadLine('>');

		//Check Notification
		ACK ack = CheckNotification(strRead);
		
		if (ack == OK || ack == ER)
		{
			strRead = ReadLine('>');
		}
		//Check Notification End

		ack = CheckAck(strRead);

		if (ack == None)
		{
			CIronLib::WriteDebugLog("[WACCO]CKDN Ack Error[%s]", strRead.c_str());
			return E_FAIL;
		}

		m_nComplete = 0;
	}
	else
	{
		string strMsg="";

		if (!Write("#CKUP\r"))
			return E_FAIL;

		string strRead = ReadLine('>');

		//Check Notification
		ACK ack = CheckNotification(strRead);
		
		if (ack == OK || ack == ER)
		{
			strRead = ReadLine('>');
		}
		//Check Notification End

		ack = CheckAck(strRead);

		if (ack == None)
		{
			CIronLib::WriteDebugLog("[WACCO]CKUP Ack Error[%s]", strRead.c_str());
			return E_FAIL;
		}

		m_nComplete = 0;
	}
	return S_OK;
}

LONG CDevice::SetErrorClear(int nVal)
{
	CIronLock cLock(*this);

	string strMsg = "";

	if (!Write("#ECLR\r"))
		return E_FAIL;

	string strRead = ReadLine('>');

	//Check Notification
	ACK ack = CheckNotification(strRead);
	
	if (ack == OK || ack == ER)
	{
		strRead = ReadLine('>');
	}
	//Check Notification End

	ack = CheckAck(strRead);

	if (ack == None)
	{
		CIronLib::WriteDebugLog("[WACCO]ECLR Ack Error[%s]", strRead.c_str());
		return E_FAIL;
	}

	m_nComplete = 0;

	return S_OK;
}

LONG CDevice::SetRunCalibration(int nVal)
{
	CIronLock cLock(*this);

	string strMsg = "";

	if (!Write("#RCAL\r"))
		return E_FAIL;

	string strRead = ReadLine('>');

	//Check Notification
	ACK ack = CheckNotification(strRead);
	
	if (ack == OK || ack == ER)
	{
		strRead = ReadLine('>');
	}
	//Check Notification End

	ack = CheckAck(strRead);

	if (ack == None)
	{
		CIronLib::WriteDebugLog("[WACCO]RCAL Ack Error[%s]", strRead.c_str());
		return E_FAIL;
	}

	m_nComplete = 0;

	return S_OK;
}

LONG CDevice::SetReset(int nVal)
{
	CIronLock cLock(*this);

	string strMsg = "";

	if (!Write("#REST\r"))
		return E_FAIL;

	string strRead = ReadLine('>');

	if (strRead.length() < 1 || (strRead.substr(strRead.length() -1, 1) != ">"))
	{
		CIronLib::WriteDebugLog("[WACCO]REST Ack Error[%s]", strRead.c_str());
		return E_FAIL;
	}

	return S_OK;
}

LONG CDevice::SetInitialize(int nVal)
{
	CIronLock cLock(*this);

	string strMsg = "";

	if(nVal == 1)
	{
		if (!Write("#HOME\r"))
			return E_FAIL;

		string strRead = ReadLine('>');

		//Check Notification
		ACK ack = CheckNotification(strRead);
		
		if (ack == OK || ack == ER)
		{
			strRead = ReadLine('>');
		}
		//Check Notification End

		ack = CheckAck(strRead);

		if (ack == None)
		{
			CIronLib::WriteDebugLog("[WACCO]HOME Ack Error[%s]", strRead.c_str());
			return E_FAIL;
		}

		m_nComplete = 0;
	}

	return S_OK;
}

LONG CDevice::SetRotateWafer(int nVal)
{
	CIronLock cLock(*this);

	if (nVal == 1)
	{
		string sSendMsg = "#ROWA ";
		string strMsg = "";

		char temp[256];

		sprintf_s(temp, 256, "%.2f", m_dAngle);
		sSendMsg += string(temp);
		sSendMsg += "\r";

		if (!Write(sSendMsg))
			return E_FAIL;

		string strRead = ReadLine('>');

		//Check Notification
		ACK ack = CheckNotification(strRead);
		
		if (ack == OK || ack == ER)
		{
			strRead = ReadLine('>');
		}
		//Check Notification End

		ack = CheckAck(strRead);

		if (ack == None)
		{
			CIronLib::WriteDebugLog("[WACCO]ROWA Ack Error[%s]", strRead.c_str());
			return E_FAIL;
		}

		m_nComplete = 0;
	}

	return S_OK;
}

LONG CDevice::SetSystemSetting(int nVal)
{
	CIronLock cLock(*this);

	string sSendMsg = "#SSET ";
	string strMsg = "";

	sSendMsg += CIronLib::itostring(nVal);
	sSendMsg += "\r";

	if (!Write(sSendMsg))
		return E_FAIL;

	string strRead = ReadLine('>');

	//Check Notification
	ACK ack = CheckNotification(strRead);
	
	if (ack == OK || ack == ER)
	{
		strRead = ReadLine('>');
	}
	//Check Notification End

	ack = CheckAck(strRead);

	if (ack == None)
	{
		CIronLib::WriteDebugLog("[WACCO]SSET Ack Error[%s]", strRead.c_str());
		return E_FAIL;
	}

	m_nComplete = 0;

	return S_OK;
}

LONG CDevice::SetStartAlignment(int nVal)
{
	CIronLock cLock(*this);

	if (nVal == 1)
	{
		string sSendMsg = "#STAL ";
		string strMsg = "";

		char temp[256];

		sprintf_s(temp, 256, "%.2f", m_dAngle);
		sSendMsg += string(temp);
		sSendMsg += "\r";

		if (!Write(sSendMsg))
			return E_FAIL;

		string strRead = ReadLine('>');

		//Check Notification
		ACK ack = CheckNotification(strRead);
		
		if (ack == OK || ack == ER)
		{
			strRead = ReadLine('>');
		}
		//Check Notification End

		ack = CheckAck(strRead);

		if (ack == None)
		{
			CIronLib::WriteDebugLog("[WACCO]STAL Ack Error[%s]", strRead.c_str());
			return E_FAIL;
		}

		m_nComplete = 0;
	}

	return S_OK;
}

LONG CDevice::SetVacuum(int nVal)
{
	CIronLock cLock(*this);

	if (nVal == 0)
	{
		string strMsg = "";

		if (!Write("#VAOF\r"))
			return E_FAIL;

		string strRead = ReadLine('>');

		ACK ack = CheckNotification(strRead);
		
		if (ack == None)
		{
			CIronLib::WriteDebugLog("[WACCO]VAON Ack Error[%s]", strRead.c_str());
			return E_FAIL;
		}
	}
	else
	{
		string strMsg = "";

		if (!Write("#VAON\r"))
			return E_FAIL;

		string strRead = ReadLine('>');

		ACK ack = CheckNotification(strRead);
		
		if (ack == None)
		{
			CIronLib::WriteDebugLog("[WACCO]VAON Ack Error[%s]", strRead.c_str());
			return E_FAIL;
		}
	}

	return S_OK;
}

LONG CDevice::SetWaferSize(int nVal)
{
	CIronLock cLock(*this);

	if(nVal >= 0 || nVal <= 4 )
	{
		string sSendMsg = "#WSIZ ";
		string strMsg = "";

		//sSendMsg += CIronLib::itostring(nVal);
		if(nVal == 0)
			sSendMsg += "01";
		else if(nVal == 1)
			sSendMsg += "02";
		else if(nVal == 3)
			sSendMsg += "10";
		else if(nVal == 4)
			sSendMsg += "20";

		sSendMsg += "\r";

		if (!Write(sSendMsg))
			return E_FAIL;

		string strRead = ReadLine('>');

		//Check Notification
		ACK ack = CheckNotification(strRead);
		
		if (ack == OK || ack == ER)
		{
			strRead = ReadLine('>');
		}
		//Check Notification End

		ack = CheckAck(strRead);

		if (ack == None)
		{
			CIronLib::WriteDebugLog("[WACCO]WSIZ Ack Error[%s]", strRead.c_str());
			return E_FAIL;
		}

		m_nComplete = 0;
	}

	return S_OK;
}

LONG CDevice::SetCheckWaferExistence(int nVal)
{	
	CIronLock cLock(*this);

	string strMsg = "";

	if (!Write("#CHKW\r"))
		return E_FAIL;

	string strRead = ReadLine('>');

	//Check Notification
	ACK ack = CheckNotification(strRead);
	
	if (ack == OK || ack == ER)
	{
		strRead = ReadLine('>');
	}
	//Check Notification End

	ack = CheckAck(strRead);

	if (ack == None)
	{
		CIronLib::WriteDebugLog("[WACCO]WSIZ Ack Error[%s]", strRead.c_str());
		return E_FAIL;
	}

	m_nComplete = 0;

	return S_OK;
}
