// WACCO.cpp : NymDriver  α׷   մϴ.
//

#include "stdafx.h"
#include "WACCO.h"
#include <DriverBase.h>
#include "Device.h"


#ifdef _MANAGED
#pragma managed(push, off)
#endif

BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
    return TRUE;
}

#ifdef _MANAGED
#pragma managed(pop)
#endif

CDriverBase* g_pDriver = NULL;
double g_nOldTimeout = 0;
double g_nOldWriteTimeout = 0;

void SetTimeout(int nTimeout)
{
	if (g_pDriver == NULL)
		return;

	//Read Timeout Setting
	g_nOldTimeout = g_pDriver->GetTimeout();
			
	if (g_nOldTimeout != nTimeout)
	{
		g_pDriver->SetTimeout(nTimeout);
	}

	//Write Timeout Setting
	int nWriteTimeout = 0;

	if (nTimeout > 0)
		nWriteTimeout = nTimeout;

	g_nOldWriteTimeout = g_pDriver->GetWriteTimeout();
		
	if (g_nOldWriteTimeout != nWriteTimeout)
	{
		g_pDriver->SetWriteTimeout(nWriteTimeout);
	}
}

void ResetTimeout(int nTimeout)
{
	if (g_pDriver == NULL)
		return;

	//ReadTimeout Setting
	if (g_nOldTimeout != nTimeout)
	{
		g_pDriver->SetTimeout(g_nOldTimeout);
	}

	int nWriteTimeout = 0;

	if (nTimeout > 0)
		nWriteTimeout = nTimeout;

	//WtieTimeout Setting
	if (g_nOldWriteTimeout != nWriteTimeout)
	{
		g_pDriver->SetWriteTimeout(g_nOldWriteTimeout);
	}
}

WACCO_API LONG InitDriver(unsigned uPort, int nTimeout, bool bLog)
{
	if (g_pDriver == NULL)
		g_pDriver = new CDevice(bLog, uPort);
	else
		return E_FAIL;

	return S_OK;
}

WACCO_API LONG UninitDriver(void)
{
	if (g_pDriver != NULL)
		delete g_pDriver;
	else
		return E_FAIL;

	return S_OK;
}

WACCO_API LONG ReadAnalog(double &dInput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);
	
	switch (uAddr)
	{
	case AI_AngleOffset:
		hr = pDevice->GetAngleOffset(dInput);
		break;
	case AI_Angle:
		hr = pDevice->GetAngle(dInput);
		break;
	default:
		CIronLib::WriteDebugLog("WACCO>>Invalid AI Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}

	ResetTimeout(nTimeout);
	
	return hr;
}

WACCO_API LONG ReadDigital(int &nInput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);
	
	switch (uAddr)
	{
	case DI_WaferExistence:
		hr = pDevice->GetWaferExistence(nInput);
		break;
	case DI_ChuckPosition:
		hr = pDevice->GetChuckPosition(nInput);
		break;
	case DI_ErrorStatus:
		hr = pDevice->GetErrorStatus(nInput);
		break;
	case DI_SystemSetting:
		{
			int nVal = 0;
			hr = pDevice->GetSystemSetting(nVal);
			nInput = nVal & (1 << uIndex) ? 1 : 0;
		}
		break;
	case DI_SystemStatus:
		{
			int nVal = 0;
			hr = pDevice->GetSystemStatus(nVal);
			nInput = nVal & (1 << uIndex) ? 1 : 0;
		}
		break;
	case DI_Vacuum:
		hr = pDevice->GetVacuum(nInput);
		break;
	case DI_WaferSize:
		hr = pDevice->GetWaferSize(nInput);
		break;
	case DI_Complete:
		hr = pDevice->GetComplete(nInput);
		break;
	default:
		CIronLib::WriteDebugLog("WACCO>>Invalid DI Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}
	
	ResetTimeout(nTimeout);
	
	return hr;
}

WACCO_API LONG ReadString(LPTSTR strInput, unsigned uAddr, unsigned uIndex, unsigned uSize, int nTimeout)
{
	HRESULT hr = E_FAIL;
	
	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);

	switch (uAddr)
	{
	case 0:
/*		hr = pDevice->GetSI(strInput);
		break;*/
	default:
		CIronLib::WriteDebugLog("WACCO>>Invalid SI Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}
	
	ResetTimeout(nTimeout);

	return hr;
}

WACCO_API LONG ReadVariable(LPCTSTR strName, unsigned uAddr, unsigned uIndex, unsigned uSize, void* pData, int nTimeout)
{
		return E_FAIL;
}

WACCO_API LONG WriteAnalog(double dOutput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);

	switch (uAddr)
	{
	case AO_AngleOffset:
		hr = pDevice->SetAngleOffset(dOutput);
		break;
	case AO_Angle:
		hr = pDevice->SetAngle(dOutput);
		break;
	default:
		CIronLib::WriteDebugLog("WACCO>>Invalid AO Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}

	ResetTimeout(nTimeout);
	
	return hr;
}

WACCO_API LONG WriteDigital(int nOutput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;
	
	SetTimeout(nTimeout);
	
	switch (uAddr)
	{
	case DO_ChuckUpDown:
		hr = pDevice->SetChuckUpDown(nOutput);
		break;
	case DO_ErrorClear:
		hr = pDevice->SetErrorClear(nOutput);
		break;
	case DO_RunCalibration:
		hr = pDevice->SetRunCalibration(nOutput);
		break;
	case DO_Reset:
		hr = pDevice->SetReset(nOutput);
		break;
	case DO_Initialize:
		hr = pDevice->SetInitialize(nOutput);
		break;
	case DO_RotateWafer:
		hr = pDevice->SetRotateWafer(nOutput);
		break;
	case DO_SystemSetting:
		{
			int nVal = 0;
			hr = pDevice->GetSystemSetting(nVal);

			if (FAILED(hr))
				break;

			if (nOutput == 0)
			{
				nVal &= ~(1 << uIndex);
			}
			else
			{
				nVal |= (1 << uIndex);
			}

			hr = pDevice->SetSystemSetting(nVal);
		}
	case DO_StartAligment:
		hr = pDevice->SetStartAlignment(nOutput);
		break;
	case DO_Vacuum:
		hr = pDevice->SetVacuum(nOutput);
		break;
	case DO_WaferSize:
		hr = pDevice->SetWaferSize(nOutput);
		break;
	case DO_CheckWaferExistence:
		hr = pDevice->SetCheckWaferExistence(nOutput);
		break;
	default:
		CIronLib::WriteDebugLog("WACCO>>Invalid DI Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}

	ResetTimeout(nTimeout);
	
	return hr;
}

WACCO_API LONG WriteString(LPCTSTR strOutput, unsigned uAddr, unsigned uIndex, int nTimeout)
{
	HRESULT hr = E_FAIL;

	CDevice* pDevice = (CDevice*)g_pDriver;

	SetTimeout(nTimeout);
	
	switch (uAddr)
	{
	case 0:
/*		hr = pDevice->SetSO(strOutput);
		break;*/
	default:
		CIronLib::WriteDebugLog("WACCO>>Invalid SO Address[%d].(%s:%d)", uAddr, __FILE__, __LINE__);
		break;
	}
	
	ResetTimeout(nTimeout);
	
	return hr;
}

WACCO_API LONG WriteVariable(LPCTSTR strName, unsigned uAddr, unsigned uIndex, unsigned uSize, void* pData, int nTimeout)
{
		return E_FAIL;
}